Package: devkit
Type: Package
Title: Practical Utilities for Package Development and Session Auditing
Version: 1.0.0
Authors@R: 
  c(person(given = "Zankrut",
           family = "Goyani",
           role = c("aut", "cre", "cph"),
           email = "zankrut20@gmail.com"))
Description: Provides a comprehensive collection of practical R utilities 
    designed to streamline the full life cycle of package development, 
    interactive data analysis, and session management. It offers tools for 
    package management, development workflows, debugging, data processing, 
    and system optimization. These utilities help R developers and data 
    scientists automate release tasks, manage and scan dependencies, 
    audit session states, optimize memory usage, and scramble sensitive 
    Personally Identifiable Information ('PII').
Encoding: UTF-8
License: MIT + file LICENSE
Language: en-US
Depends: R (>= 3.5.0)
Imports: tools
Suggests: testthat, devtools, roxygen2, usethis, knitr, rmarkdown,
        withr
VignetteBuilder: knitr
RoxygenNote: 8.0.0
Config/roxygen2/version: 8.0.0
URL: https://zankrut20.github.io/devkit/,
        https://github.com/zankrut20/devkit
BugReports: https://github.com/zankrut20/devkit/issues
NeedsCompilation: no
Packaged: 2026-06-12 03:18:23 UTC; zankrut
Author: Zankrut Goyani [aut, cre, cph]
Maintainer: Zankrut Goyani <zankrut20@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-19 11:30:02 UTC

# Generated by roxygen2: do not edit by hand

export(architect_release)
export(architect_vignette)
export(audit_dependencies)
export(audit_script)
export(benchmark_branches)
export(bootstrap_dev_env)
export(detect_masking)
export(dictate_dictionary)
export(dispatch_checkpoints)
export(export_snapshot)
export(hunt_zombies)
export(loop_guardian)
export(manage_deprecation)
export(mask_identity)
export(network_diplomat)
export(remove_package)
export(remove_user_installed_packages)
export(scaffold_parallel)
export(scaffold_tests)
export(scan_dependencies)
export(setup_preflight)
export(setup_sentinel)
export(simulate_clean_room)
export(sweep_memory)
export(sweep_temp_cache)
importFrom(grDevices,dev.list)
importFrom(grDevices,graphics.off)
importFrom(graphics,par)
importFrom(stats,setNames)
importFrom(tools,package_dependencies)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,remove.packages)
importFrom(utils,select.list)
importFrom(utils,tail)

# devkit <img src="man/figures/logo.png" align="right" height="139" />

<!-- badges: start -->
[![R-CMD-check](https://github.com/zankrut20/devkit/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/zankrut20/devkit/actions/workflows/R-CMD-check.yaml)
[![pkgdown](https://github.com/zankrut20/devkit/actions/workflows/pkgdown.yaml/badge.svg)](https://zankrut20.github.io/devkit/)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
[![codecov](https://codecov.io/gh/zankrut20/devkit/branch/main/graph/badge.svg)](https://app.codecov.io/gh/zankrut20/devkit)
<!-- badges: end -->

`devkit` is a professional, zero-dependency R development toolkit designed to streamline package management, environment setup, session auditing, and batch processing. It provides a suite of utilities to help developers maintain CRAN compliance and optimize their development workflow.

## 🚀 Installation

Install the released version of `devkit` from CRAN with:

```r
install.packages("devkit")
```

Or install the development version from [GitHub](https://github.com/zankrut20/devkit) with:

```r
# install.packages("devtools")
devtools::install_github("zankrut20/devkit")
```

## 🛠️ Usage

Once installed, simply load the library to access all utilities:

```r
library(devkit)

# Example: Audit your package dependencies
audit_dependencies()

# Example: Clean up memory hogs
hunt_zombies()
```

## 📦 Toolkit Overview

### 📦 Package Management
- `audit_dependencies()`: Verifies DESCRIPTION file vs actual code usage.
- `remove_package()`: Smart package removal with orphan dependency checking.
- `remove_user_installed_packages()`: Cleans all user-installed packages while preserving base/recommended ones.
- `scan_dependencies()`: Identifies unused packages in your session.

### 🧹 Memory Management
- `sweep_memory()`: Removes large objects and triggers garbage collection.
- `hunt_zombies()`: Cleans hidden temp files and orphaned graphics devices.
- `sweep_temp_cache()`: Flushes R session caches across the system.

### 🔧 Development Environment
- `bootstrap_dev_env()`: Installs and attaches core dev tools.
- `manage_deprecation()`: Scaffolds deprecation wrappers and refactors calls.
- `setup_preflight()`: Configures Git pre-commit hooks for safety.
- `setup_sentinel()`: Enables dual-logging for session reproducibility.

### 📊 Session Management
- `audit_script()`: Snapshots session state before and after script execution.
- `detect_masking()`: Resolves namespace conflicts and function masking.
- `export_snapshot()`: Exports attached packages to an installation script.

### ⚙️ Batch Processing
- `dispatch_checkpoints()`: Crash-resilient batch processing with recovery.
- `loop_guardian()`: Memory-safe iteration with RAM monitoring.
- `network_diplomat()`: Rate-limited network requests with exponential backoff.

### 🏗️ Code Generation
- `architect_release()`: Automates version bumping and release notes.
- `architect_vignette()`: Scaffolds CRAN-compliant vignettes.
- `scaffold_parallel()`: Generates parallel processing boilerplate.
- `scaffold_tests()`: Generates testthat boilerplate.
- `simulate_clean_room()`: Verifies script reproducibility in a vanilla session.

### 🔐 Privacy
- `mask_identity()`: Interactively anonymizes PII in datasets.

### 🛠️ Utilities
- `benchmark_branches()`: Compares performance across Git branches.
- `dictate_dictionary()`: Generates roxygen2 documentation for data frames.

## 📝 License

This package is licensed under the MIT License.

## 🤝 Contributing

Contributions are welcome! Please see [CONTRIBUTING.md](https://github.com/zankrut20/devkit/blob/master/CONTRIBUTING.md) for guidelines.

#' Interactive Package Remover
#'
#' @description
#' Safely removes a specified R package and its unused dependencies from the
#' system, ensuring that other installed packages do not lose required dependencies.
#'
#' @details
#' The function implements a dependency-aware removal process:
#' \enumerate{
#'   \item Identifies the dependencies of the target package using `tools::package_dependencies`.
#'   \item Analyzes all other installed packages to determine which dependencies are still required.
#'   \item Isolates "orphan" dependencies—those that were only required by the target package.
#'   \item Interactively prompts the user to select which of these orphan packages (and the target package itself) should be removed.
#'   \item Executes `remove.packages()` on the selected items.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @param pkg Character. The name of the package to remove.
#' @param recursive Logical. Whether to check for recursive dependencies. Defaults to `FALSE`.
#'
#' @return A character vector of the packages that were removed, or invisibly `character()` if nothing was removed.
#'
#' @importFrom utils select.list remove.packages
#' @importFrom tools package_dependencies
#' @examples
#' if (interactive()) {
#'   remove_package('curl')
#' }
#' @export

remove_package <- function(pkg, recursive = FALSE) {
  # Get dependency information for all packages (tools::package_dependencies uses internal default)
  d <- package_dependencies(recursive = recursive)
  depends <- if (!is.null(d[[pkg]])) d[[pkg]] else character()
  required <- unique(unlist(d[!names(d) %in% c(pkg, depends)]))
  orphans <- depends[!depends %in% required]

  # Always offer the target package itself for removal, plus any orphans
  candidates <- c(pkg, sort(orphans))

  to_remove <- select.list(candidates, multiple = TRUE,
                           title = "Select packages to remove")

  if (length(to_remove) > 0) {
    remove.packages(to_remove)
    return(to_remove)
  }

  invisible(character())
}

# Tests for remove_package() (package-remover.R)

test_that("remove_package offers target pkg even when no orphan deps", {
  local_mocked_bindings(
    package_dependencies = function(...) {
      list(mypkg = character(0))
    },
    select.list = function(choices, ...) choices,
    remove.packages = function(...) invisible(NULL),
    .package = "devkit"
  )

  result <- remove_package("mypkg")
  expect_true("mypkg" %in% result)
})

test_that("remove_package prompts user when orphan deps exist", {
  local_mocked_bindings(
    package_dependencies = function(...) {
      list(
        mypkg = c("dep1", "dep2"),
        otherpkg = character(0)
      )
    },
    select.list = function(choices, ...) choices,
    remove.packages = function(...) invisible(NULL),
    .package = "devkit"
  )

  result <- remove_package("mypkg")
  expect_true("mypkg" %in% result)
})

test_that("remove_package filters out deps required by other packages", {
  local_mocked_bindings(
    package_dependencies = function(...) {
      list(
        mypkg = c("dep1", "shared_dep"),
        otherpkg = c("shared_dep")
      )
    },
    select.list = function(choices, ...) choices,
    remove.packages = function(...) invisible(NULL),
    .package = "devkit"
  )

  result <- remove_package("mypkg")
  expect_false("shared_dep" %in% result)
  expect_true("dep1" %in% result)
})

test_that("remove_package handles package with NULL dependencies", {
  local_mocked_bindings(
    package_dependencies = function(...) {
      list(mypkg = NULL)
    },
    select.list = function(choices, ...) choices,
    remove.packages = function(...) invisible(NULL),
    .package = "devkit"
  )

  result <- remove_package("mypkg")
  expect_true("mypkg" %in% result)
})

test_that("remove_package with recursive = TRUE passed to package_dependencies", {
  dep_args <- NULL
  local_mocked_bindings(
    package_dependencies = function(recursive, ...) {
      dep_args <<- list(recursive = recursive)
      list(mypkg = character(0))
    },
    select.list = function(choices, ...) character(0),
    .package = "devkit"
  )

  remove_package("mypkg", recursive = TRUE)
  expect_true(dep_args$recursive)
})

#' Interactive Release Candidate Architect
#'
#' @description
#' Automates the process of preparing a package release by bumping the version 
#' in the `DESCRIPTION` file and interactively drafting release notes in `NEWS.md`.
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Verifies the existence of the `DESCRIPTION` file.
#'   \item Parses the current version and prompts the user to choose between a Patch, Minor, or Major bump.
#'   \item Updates the `Version` and `Date` fields in the `DESCRIPTION` file upon confirmation.
#'   \item Interactively collects changelog items from the user and prepends them to `NEWS.md`.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{package}, \code{old_version},
#' \code{new_version}, \code{bump_type}, \code{description_updated} (logical),
#' and \code{news_updated} (logical).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   architect_release()
#' }
#' @export

architect_release <- function() {
  message("Initializing Release Candidate Architect...")
  
  # 1. Verify Package Environment
  if (!file.exists("DESCRIPTION")) {
    message("Error: DESCRIPTION file not found. Are you in the package root?")
    return(invisible(list(status = "error")))
  }
  
  # 2. Parse Current Version
  desc_info <- read.dcf("DESCRIPTION")
  pkg_name <- as.character(desc_info[, "Package"])
  current_version <- as.character(desc_info[, "Version"])
  
  message(sprintf("\nDetected Package: %s", pkg_name))
  message(sprintf("Current Version:  %s", current_version))
  
  # Split version into components (assuming standard x.y.z format)
  v_parts <- as.numeric(unlist(strsplit(current_version, "\\.")))
  if (length(v_parts) < 3) v_parts <- c(v_parts, rep(0, 3 - length(v_parts)))
  
  # 3. Ask the user what kind of release this is
  bump_type <- select.list(
    choices = c("Patch (Bug fixes: x.y.Z+1)", 
                "Minor (New features: x.Y+1.0)", 
                "Major (Breaking changes: X+1.0.0)", 
                "Cancel"),
    title = "\nWhat type of version bump is this?"
  )
  
  if (bump_type == "Cancel" || bump_type == "") {
    message("Release preparation cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  # 4. Calculate New Version
  if (grepl("Patch", bump_type)) {
    new_version <- sprintf("%d.%d.%d", v_parts[1], v_parts[2], v_parts[3] + 1)
  } else if (grepl("Minor", bump_type)) {
    new_version <- sprintf("%d.%d.0", v_parts[1], v_parts[2] + 1)
  } else {
    new_version <- sprintf("%d.0.0", v_parts[1] + 1)
  }
  
  # 5. Confirm and Update DESCRIPTION
  confirm_desc <- select.list(
    choices = c("Yes", "No"),
    title = sprintf("\nUpdate DESCRIPTION from %s to %s?", current_version, new_version)
  )
  
  if (confirm_desc == "Yes") {
    desc_info[, "Version"] <- new_version
    desc_info[, "Date"] <- as.character(Sys.Date())
    write.dcf(desc_info, file = "DESCRIPTION")
    message(sprintf("-> Success: DESCRIPTION updated to v%s", new_version))
  }
  
  # 6. Interactively Build NEWS.md
  update_news <- select.list(
    choices = c("Yes", "No"),
    title = "\nWould you like to draft the release notes for NEWS.md now?"
  )
  
  if (update_news == "Yes") {
    message("\n--- Drafting Release Notes ---")
    message("Enter your changelog items one by one. Leave blank and press Enter to finish.")
    
    news_bullets <- character()
    adding_items <- TRUE
    counter <- 1
    
    # Ask for inputs one by one to build the changelog
    while (adding_items) {
      bullet <- readline(prompt = sprintf("Bullet %d: ", counter))
      
      if (trimws(bullet) == "") {
        adding_items <- FALSE
      } else {
        news_bullets <- c(news_bullets, sprintf("* %s", trimws(bullet)))
        counter <- counter + 1
      }
    }
    
    if (length(news_bullets) > 0) {
      news_header <- sprintf("# %s %s", pkg_name, new_version)
      news_block <- c(news_header, "", news_bullets, "", "")
      
      # Check if NEWS.md exists, if so, prepend. Otherwise, create.
      if (file.exists("NEWS.md")) {
        current_news <- readLines("NEWS.md", warn = FALSE)
        writeLines(c(news_block, current_news), con = "NEWS.md")
      } else {
        writeLines(news_block, con = "NEWS.md")
      }
      
      message("-> Success: NEWS.md updated with your latest changes.")
    } else {
      message("No items entered. Skipping NEWS.md update.")
    }
  }
  
  message(sprintf("\nRelease Candidate v%s is prepped and ready!", new_version))
  return(invisible(list(
    status = "done",
    package = pkg_name,
    old_version = current_version,
    new_version = new_version,
    bump_type = bump_type,
    description_updated = (confirm_desc == "Yes"),
    news_updated = (update_news == "Yes" && exists("news_bullets") && length(news_bullets) > 0)
  )))
}

# Tests for architect_release()

test_that("architect_release returns early when DESCRIPTION is missing", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  expect_message(architect_release(), "DESCRIPTION file not found")
})

test_that("architect_release cancels when user selects Cancel", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0.0"), "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "Cancel",
    .package = "devkit"
  )

  expect_message(architect_release(), "cancelled")
})

test_that("architect_release cancels when user selects empty string", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0.0"), "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(architect_release(), "cancelled")
})

test_that("architect_release handles Patch bump correctly", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.2.3", "Date: 2024-01-01"), "DESCRIPTION")

  call_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      call_count <<- call_count + 1L
      if (call_count == 1L) return("Patch (Bug fixes: x.y.Z+1)")
      if (call_count == 2L) return("Yes")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  expect_message(architect_release(), "v1\\.2\\.4")
  desc <- read.dcf("DESCRIPTION")
  expect_equal(as.character(desc[, "Version"]), "1.2.4")
})

test_that("architect_release handles Minor bump correctly", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.2.3", "Date: 2024-01-01"), "DESCRIPTION")

  call_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      call_count <<- call_count + 1L
      if (call_count == 1L) return("Minor (New features: x.Y+1.0)")
      if (call_count == 2L) return("Yes")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  expect_message(architect_release(), "v1\\.3\\.0")
})

test_that("architect_release handles Major bump correctly", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.2.3", "Date: 2024-01-01"), "DESCRIPTION")

  call_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      call_count <<- call_count + 1L
      if (call_count == 1L) return("Major (Breaking changes: X+1.0.0)")
      if (call_count == 2L) return("Yes")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  expect_message(architect_release(), "v2\\.0\\.0")
})

test_that("architect_release pads version with fewer than 3 parts", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0", "Date: 2024-01-01"), "DESCRIPTION")

  call_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      call_count <<- call_count + 1L
      if (call_count == 1L) return("Patch (Bug fixes: x.y.Z+1)")
      if (call_count == 2L) return("Yes")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  expect_message(architect_release(), "v1\\.0\\.1")
})

test_that("architect_release writes NEWS.md with changelog bullets", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0.0", "Date: 2024-01-01"), "DESCRIPTION")

  select_count <- 0L
  readline_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("Patch (Bug fixes: x.y.Z+1)")
      if (select_count == 2L) return("Yes")
      if (select_count == 3L) return("Yes")
      return("Yes")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("Fixed a bug")
      if (readline_count == 2L) return("Added feature")
      return("")
    },
    .package = "base"
  )

  architect_release()
  expect_true(file.exists("NEWS.md"))
  news <- readLines("NEWS.md")
  expect_true(any(grepl("Fixed a bug", news)))
  expect_true(any(grepl("Added feature", news)))
})

test_that("architect_release prepends to existing NEWS.md", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0.0", "Date: 2024-01-01"), "DESCRIPTION")
  writeLines("# testpkg 1.0.0\n\n* Initial release", "NEWS.md")

  select_count <- 0L
  readline_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("Patch (Bug fixes: x.y.Z+1)")
      if (select_count == 2L) return("Yes")
      if (select_count == 3L) return("Yes")
      return("Yes")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("New fix")
      return("")
    },
    .package = "base"
  )

  architect_release()
  news <- readLines("NEWS.md")
  expect_true(any(grepl("New fix", news)))
  expect_true(any(grepl("Initial release", news)))
})

test_that("architect_release skips NEWS.md when no bullets entered", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0.0", "Date: 2024-01-01"), "DESCRIPTION")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("Patch (Bug fixes: x.y.Z+1)")
      if (select_count == 2L) return("Yes")
      if (select_count == 3L) return("Yes")
      return("Yes")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) "",
    .package = "base"
  )

  expect_message(architect_release(), "No items entered")
})

test_that("architect_release skips DESCRIPTION update when user says No", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0.0"), "DESCRIPTION")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("Patch (Bug fixes: x.y.Z+1)")
      if (select_count == 2L) return("No")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  architect_release()
  desc <- read.dcf("DESCRIPTION")
  expect_equal(as.character(desc[, "Version"]), "1.0.0")
})

#' Initialize Development Environment
#'
#' @description
#' Scans the system for core R package development tools, prompts the user to 
#' install any missing packages, and allows for the selective loading of these 
#' tools into the current R session.
#'
#' @details
#' The function focuses on a standard toolkit for CRAN-ready development, 
#' including `devtools`, `roxygen2`, `usethis`, `testthat`, and `knitr`.
#' 
#' The process follows these steps:
#' \enumerate{
#'   \item Compares the list of core tools against the currently installed packages.
#'   \item If tools are missing, it provides an interactive menu to install all 
#'       missing packages, specific ones, or skip installation entirely.
#'   \item After ensuring availability, it prompts the user to select which 
#'       of the available tools should be attached to the current session using `library()`.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'

#' @return 
#' Invisibly returns a named list with components: \code{status} ("done"),
#' \code{initially_missing}, \code{available}, and \code{loaded} (character vectors).
#'
#' @importFrom utils select.list install.packages
#' @examples
#' if (interactive()) {
#'   bootstrap_dev_env()
#' }
#' @export

bootstrap_dev_env <- function() {
  # The essential toolkit for CRAN-ready package development
  core_tools <- c("devtools", "roxygen2", "usethis", "testthat", "knitr")
  
  # Identify what is currently installed using fast package lookup
  installed <- core_tools[vapply(core_tools, requireNamespace, quietly = TRUE, FUN.VALUE = logical(1))]
  missing_tools <- setdiff(core_tools, installed)
  
  message("Checking development environment...")
  
  # Step 1: Handle missing packages interactively
  if (length(missing_tools) > 0) {
    message("The following development tools are missing from your system:")
    print(missing_tools)
    
    to_install <- select.list(
      choices = c("Install All Missing", missing_tools, "Skip Installation"),
      multiple = TRUE,
      title = "Select packages to install right now:"
    )
    
    if (!"Skip Installation" %in% to_install && length(to_install) > 0) {
      if ("Install All Missing" %in% to_install) {
        install.packages(missing_tools)
      } else {
        install.packages(to_install)
      }
      message("Installation complete.")
    } else {
      message("Skipping installation.")
    }
  } else {
    message("All core development tools are already installed on your system.")
  }
  
  # Step 2: Dynamically load tools for the current session
  # Re-evaluate available packages in case new ones were just installed
  available_tools <- core_tools[vapply(core_tools, requireNamespace, quietly = TRUE, FUN.VALUE = logical(1))]
  
  to_load <- select.list(
    choices = available_tools,
    multiple = TRUE,
    title = "Select development tools to attach to this session:"
  )
  
  if (length(to_load) > 0) {
    for (pkg in to_load) {
      suppressPackageStartupMessages(library(pkg, character.only = TRUE))
    }
    message(sprintf("Successfully attached: %s", paste(to_load, collapse = ", ")))
  } else {
    message("No development tools attached.")
  }
  
  return(invisible(list(
    status = "done",
    initially_missing = missing_tools,
    available = available_tools,
    loaded = to_load
  )))
}

# Tests for bootstrap_dev_env()

test_that("bootstrap_dev_env handles all tools already installed", {
  local_mocked_bindings(
    requireNamespace = function(pkg, quietly = TRUE) TRUE,
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) character(0),
    .package = "devkit"
  )

  expect_message(bootstrap_dev_env(), "already installed")
})

test_that("bootstrap_dev_env handles missing tools with Install All", {
  install_called <- FALSE
  local_mocked_bindings(
    requireNamespace = function(pkg, quietly = TRUE) pkg %in% c("devtools", "roxygen2"),
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Install All", choices))) return("Install All Missing")
      return(character(0))
    },
    install.packages = function(...) { install_called <<- TRUE },
    .package = "devkit"
  )

  capture.output(
    expect_message(bootstrap_dev_env(), "missing|Installation"),
    type = "output"
  )
})

test_that("bootstrap_dev_env handles skip installation", {
  local_mocked_bindings(
    requireNamespace = function(pkg, quietly = TRUE) pkg == "devtools",
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Skip", choices))) return("Skip Installation")
      return(character(0))
    },
    .package = "devkit"
  )

  capture.output(
    expect_message(bootstrap_dev_env(), "Skipping|missing"),
    type = "output"
  )
})

test_that("bootstrap_dev_env loads selected tools", {
  loaded_pkgs <- character(0)
  local_mocked_bindings(
    requireNamespace = function(pkg, quietly = TRUE) TRUE,
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Install", choices))) return("Skip Installation")
      return("devtools")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    library = function(pkg, ...) { loaded_pkgs <<- c(loaded_pkgs, pkg) },
    .package = "base"
  )

  suppressPackageStartupMessages(bootstrap_dev_env())
  expect_true(TRUE)
})

test_that("bootstrap_dev_env handles no tools to load", {
  local_mocked_bindings(
    requireNamespace = function(pkg, quietly = TRUE) TRUE,
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) character(0),
    .package = "devkit"
  )

  expect_message(bootstrap_dev_env(), "No development tools attached|already installed")
})

test_that("bootstrap_dev_env installs specific selected packages", {
  installed_pkgs <- character(0)
  select_count <- 0L
  local_mocked_bindings(
    requireNamespace = function(pkg, quietly = TRUE) pkg == "devtools",
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("usethis")
      return(character(0))
    },
    install.packages = function(pkgs, ...) {
      installed_pkgs <<- c(installed_pkgs, pkgs)
    },
    .package = "devkit"
  )

  capture.output(bootstrap_dev_env(), type = "output")
  expect_true("usethis" %in% installed_pkgs)
})

#' Bulk Loop Guardian
#'
#' @description
#' A memory-safe wrapper for heavy iterations that monitors RAM usage and 
#' triggers an interactive failsafe if the environment approaches critical capacity.
#'
#' @details
#' The function provides a safeguard against memory-related crashes during 
#' large-scale batch processing:
#' \enumerate{
#'   \item Iterates through the `items` vector, applying `target_func` to each element.
#'   \item Every 50 iterations, it checks the current memory usage using `gc()`.
#'   \item If the memory usage exceeds `limit_mb`, it first attempts a deep garbage 
#'       collection to recover RAM.
#'   \item If memory remains above the threshold after GC, it triggers an interactive 
#'       alarm, allowing the user to:
#'       \itemize{
#'         \item Save current progress to `save_path` and abort.
#'         \item Ignore the limit and attempt to continue.
#'         \item Abort immediately without saving.
#'       }
#' }
#'
#' @param items A vector or list of items to process.
#' @param target_func The function to apply to each item.
#' @param limit_mb Numeric. The memory limit in megabytes before triggering the alarm. Defaults to `4000`.
#' @param save_path Character. Where to dump the emergency checkpoint data. Defaults to `"emergency_checkpoint.rds"`.
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return A list of successfully processed results.
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   loop_guardian(items = 1:5, target_func = print)
#' }
#' @export

loop_guardian <- function(items, target_func, limit_mb = 4000, save_path = "emergency_checkpoint.rds") {
  message(sprintf("Starting Guardian Loop for %d items. Memory limit: %d MB.", length(items), limit_mb))
  
  results <- list()
  total_items <- length(items)
  
  for (i in seq_len(total_items)) {
    # 1. Execute the target function on the current item
    results[[i]] <- tryCatch({
      target_func(items[[i]])
    }, error = function(e) {
      message(sprintf("\n[!] Execution error at iteration %d: %s", i, e$message))
      NA
    })
    
    # 2. Memory Check (Run every 50 iterations to avoid slowing down the loop)
    if (i %% 50 == 0) {
      # gc() column 2 contains the memory used in MB for Ncells and Vcells
      mem_stats <- gc(reset = FALSE)
      current_mem_mb <- sum(mem_stats[, 2]) 
      
      if (current_mem_mb >= limit_mb) {
        message(sprintf("\n[WARNING] Critical Memory Threshold Reached: %.2f MB", current_mem_mb))
        
        # Force an immediate deep clean to see if we can recover RAM
        invisible(gc(reset = TRUE, full = TRUE))
        recovered_mem <- sum(gc(reset = FALSE)[, 2])
        
        message(sprintf("Post-GC Memory: %.2f MB", recovered_mem))
        
        # If GC didn't solve it, trigger the interactive alarm
        if (recovered_mem >= limit_mb) {
          alarm_prompt <- sprintf(
            "Iteration %d of %d. RAM is critically high. Action required:", 
            i, total_items
          )
          
          action <- select.list(
            choices = c(
              "Save current progress to disk and Abort",
              "Ignore limit and attempt to Continue",
              "Abort immediately (Lose unsaved progress)"
            ),
            title = alarm_prompt
          )
          
          if (action == "Save current progress to disk and Abort") {
            saveRDS(results, file = save_path)
            message(sprintf("Emergency checkpoint saved to '%s'. Loop aborted safely.", save_path))
            return(invisible(results))
            
          } else if (action == "Abort immediately (Lose unsaved progress)" || action == "") {
            message("Loop aborted by user. Progress lost.")
            return(invisible(results))
            
          } else {
            message("Overriding memory limits. Continuing the loop...")
          }
        }
      }
    }
  }
  
  message("\nSuccess: All items processed without breaching memory limits.")
  return(invisible(results))
}

# Tests for loop_guardian()

test_that("loop_guardian processes all items successfully", {
  local_mocked_bindings(
    select.list = function(choices, ...) "Ignore",
    .package = "devkit"
  )
  local_mocked_bindings(
    gc = function(...) {
      mat <- matrix(c(10, 10, 10, 10), nrow = 2, ncol = 2)
      colnames(mat) <- c("used", "(Mb)")
      mat
    },
    .package = "base"
  )

  result <- loop_guardian(1:5, function(x) x^2, limit_mb = 4000)
  expect_type(result, "list")
  expect_equal(length(result), 5)
  expect_equal(result[[3]], 9)
})

test_that("loop_guardian handles error in target_func gracefully", {
  local_mocked_bindings(
    select.list = function(choices, ...) "Ignore",
    .package = "devkit"
  )

  result <- loop_guardian(1:3, function(x) {
    if (x == 2) stop("test error")
    x
  }, limit_mb = 99999)

  expect_true(is.na(result[[2]]))
  expect_equal(result[[1]], 1)
  expect_equal(result[[3]], 3)
})

test_that("loop_guardian triggers memory alarm and user saves+aborts", {
  tmp <- withr::local_tempdir()
  save_file <- file.path(tmp, "emergency.rds")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      "Save current progress to disk and Abort"
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    gc = function(...) {
      mat <- matrix(c(5000, 5000, 5000, 5000), nrow = 2, ncol = 2)
      colnames(mat) <- c("used", "(Mb)")
      mat
    },
    .package = "base"
  )

  items <- 1:50
  result <- loop_guardian(items, identity, limit_mb = 1, save_path = save_file)
  expect_true(file.exists(save_file))
  file.remove(save_file)
})

test_that("loop_guardian user aborts without saving", {
  local_mocked_bindings(
    select.list = function(choices, ...) {
      "Abort immediately (Lose unsaved progress)"
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    gc = function(...) {
      mat <- matrix(c(5000, 5000, 5000, 5000), nrow = 2, ncol = 2)
      colnames(mat) <- c("used", "(Mb)")
      mat
    },
    .package = "base"
  )

  result <- loop_guardian(1:50, identity, limit_mb = 1)
  expect_type(result, "list")
})

test_that("loop_guardian user continues past memory limit", {
  local_mocked_bindings(
    select.list = function(choices, ...) {
      "Ignore limit and attempt to Continue"
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    gc = function(...) {
      mat <- matrix(c(5000, 5000, 5000, 5000), nrow = 2, ncol = 2)
      colnames(mat) <- c("used", "(Mb)")
      mat
    },
    .package = "base"
  )

  result <- loop_guardian(1:55, identity, limit_mb = 1)
  expect_equal(length(result), 55)
})

test_that("loop_guardian processes few items without memory check", {
  result <- loop_guardian(1:3, function(x) x * 2, limit_mb = 99999)
  expect_equal(result[[1]], 2)
  expect_equal(result[[3]], 6)
})

test_that("loop_guardian checks memory every 50 iterations", {
  gc_call_count <- 0L
  local_mocked_bindings(
    gc = function(...) {
      gc_call_count <<- gc_call_count + 1L
      mat <- matrix(c(10, 10, 10, 10), nrow = 2, ncol = 2)
      colnames(mat) <- c("used", "(Mb)")
      mat
    },
    .package = "base"
  )

  result <- loop_guardian(1:100, identity, limit_mb = 99999)
  expect_equal(length(result), 100)
  expect_gte(gc_call_count, 2L)
})

#' Zombie Data Hunter
#'
#' @description
#' Hunts down "invisible" memory hogs such as massive temporary files, 
#' unclosed graphical devices, and uncollected garbage, prompting 
#' the user for targeted cleaning to reclaim system resources.
#'
#' @details
#' The function performs a system scan for three types of "zombie" data:
#' \enumerate{
#'   \item \strong{Temporary Files}: Calculates the total size of the current 
#'       session's temporary directory. If it exceeds 5 MB, it offers to flush it.
#'   \item \strong{Graphical Devices}: Checks for any open graphical devices 
#'       (e.g., PDF, PNG, or RStudioGD) and offers to close all of them.
#'   \item \strong{Orphaned Memory}: Offers to perform a "deep" garbage collection 
#'       by calling `gc()` twice, which forces R to clear both older and newer 
#'       generations of memory.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done" or
#' "cancelled"), \code{actions_taken} (character vector),
#' \code{temp_flushed_mb} (numeric), and \code{devices_closed} (integer).
#'
#' @importFrom utils select.list
#' @importFrom grDevices dev.list graphics.off
#' @examples
#' if (interactive()) {
#'   hunt_zombies()
#' }
#' @export

hunt_zombies <- function() {
  message("Scanning system for Zombie Data...")
  
  # 1. Analyze Temporary Directory (The biggest hidden storage hog)
  tmp_dir <- tempdir()
  tmp_files <- list.files(tmp_dir, full.names = TRUE, recursive = TRUE)
  tmp_size_mb <- sum(file.info(tmp_files)$size, na.rm = TRUE) / (1024^2)
  
  # 2. Analyze Graphical Devices (Hidden plot histories)
  open_devs <- dev.list()
  dev_count <- length(open_devs)
  
  # 3. Build Interactive Menu Options Dynamically
  choices <- c()
  
  if (tmp_size_mb > 5) {
    choices <- c(choices, sprintf("Flush Temporary Directory (%.2f MB wasted)", tmp_size_mb))
  }
  
  if (dev_count > 0) {
    choices <- c(choices, sprintf("Close Active Graphical Devices (%d open)", dev_count))
  }
  
  choices <- c(choices, "Force Deep Garbage Collection (Clear orphaned closures)", "Cancel")
  
  # 4. Trigger Interactive Step-by-Step Resolution
  message("\n--- Hidden Memory Hogs Detected ---")
  actions <- select.list(
    choices = choices, 
    multiple = TRUE,
    title = "Select which zombie data to eradicate (Ctrl/Cmd + Click):"
  )
  
  if (length(actions) == 0 || "Cancel" %in% actions) {
    message("Zombie hunt cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  # 5. Execute Selected Actions
  if (any(grepl("Flush Temporary Directory", actions))) {
    # Unlink deletes the files. Recreate the empty tempdir just in case the active session needs it.
    unlink(tmp_dir, recursive = TRUE, force = TRUE)
    dir.create(tmp_dir, showWarnings = FALSE)
    message(sprintf("Success: Flushed %.2f MB from hidden temporary storage.", tmp_size_mb))
  }
  
  if (any(grepl("Close Active Graphical", actions))) {
    graphics.off()
    message(sprintf("Success: Closed %d orphaned graphical devices and cleared plot history.", dev_count))
  }
  
  if (any(grepl("Force Deep Garbage", actions))) {
    # Running gc() twice forces R to clear both older and newer generations of memory
    invisible(gc(reset = TRUE, full = TRUE))
    invisible(gc(reset = TRUE, full = TRUE))
    message("Success: Forced deep garbage collection. Orphaned RAM freed.")
  }
  
  message("\nSystem is clean.")
  return(invisible(list(
    status = "done",
    actions_taken = setdiff(actions, "Cancel"),
    temp_flushed_mb = if (any(grepl("Flush", actions))) tmp_size_mb else 0,
    devices_closed = if (any(grepl("Graphical", actions))) dev_count else 0L
  )))
}

# Tests for hunt_zombies()

test_that("hunt_zombies cancels when user selects Cancel", {
  local_mocked_bindings(
    select.list = function(choices, ...) "Cancel",
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) character(0),
    file.info = function(...) data.frame(size = numeric(0)),
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() NULL,
    .package = "devkit"
  )

  expect_message(hunt_zombies(), "cancelled")
})

test_that("hunt_zombies cancels on empty selection", {
  local_mocked_bindings(
    select.list = function(choices, ...) character(0),
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) character(0),
    file.info = function(...) data.frame(size = numeric(0)),
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() NULL,
    .package = "devkit"
  )

  expect_message(hunt_zombies(), "cancelled")
})

test_that("hunt_zombies offers flush when temp size > 5 MB", {
  choices_offered <- NULL
  local_mocked_bindings(
    select.list = function(choices, ...) {
      choices_offered <<- choices
      return("Cancel")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) rep("fake.tmp", 100),
    file.info = function(...) {
      data.frame(size = rep(1e6, 100), isdir = rep(FALSE, 100))
    },
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() NULL,
    .package = "devkit"
  )

  hunt_zombies()
  expect_true(any(grepl("Flush", choices_offered)))
})

test_that("hunt_zombies offers close devices when devices are open", {
  choices_offered <- NULL
  local_mocked_bindings(
    select.list = function(choices, ...) {
      choices_offered <<- choices
      return("Cancel")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) character(0),
    file.info = function(...) data.frame(size = numeric(0)),
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() c(2L, 3L),
    .package = "devkit"
  )

  hunt_zombies()
  expect_true(any(grepl("Graphical", choices_offered)))
})

test_that("hunt_zombies executes flush temp directory", {
  unlink_called <- FALSE
  local_mocked_bindings(
    select.list = function(choices, ...) {
      choices[grepl("Flush", choices)]
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) rep("fake.tmp", 100),
    file.info = function(...) {
      data.frame(size = rep(1e6, 100), isdir = rep(FALSE, 100))
    },
    unlink = function(...) { unlink_called <<- TRUE },
    dir.create = function(...) NULL,
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() NULL,
    .package = "devkit"
  )

  expect_message(hunt_zombies(), "Flushed")
})

test_that("hunt_zombies executes close graphics devices", {
  local_mocked_bindings(
    select.list = function(choices, ...) {
      choices[grepl("Graphical", choices)]
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) character(0),
    file.info = function(...) data.frame(size = numeric(0)),
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() c(2L, 3L),
    graphics.off = function() NULL,
    .package = "devkit"
  )

  expect_message(hunt_zombies(), "Closed")
})

test_that("hunt_zombies executes deep garbage collection", {
  gc_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      choices[grepl("Garbage", choices)]
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) character(0),
    file.info = function(...) data.frame(size = numeric(0)),
    gc = function(...) { gc_count <<- gc_count + 1L; NULL },
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() NULL,
    .package = "devkit"
  )

  expect_message(hunt_zombies(), "Garbage|freed")
})

test_that("hunt_zombies always includes GC option", {
  choices_offered <- NULL
  local_mocked_bindings(
    select.list = function(choices, ...) {
      choices_offered <<- choices
      return("Cancel")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    list.files = function(...) character(0),
    file.info = function(...) data.frame(size = numeric(0)),
    .package = "base"
  )
  local_mocked_bindings(
    dev.list = function() NULL,
    .package = "devkit"
  )

  hunt_zombies()
  expect_true(any(grepl("Garbage", choices_offered)))
})

#' Silent State Auditor
#'
#' @description
#' Takes a snapshot of global options, graphical parameters, and the working directory, 
#' sources a specified R script, and then interactively helps the user identify 
#' and revert any "hijacked" settings changed by the script.
#'
#' @details
#' The function operates as follows:
#' \enumerate{
#'   \item Captures the current state of `getwd()`, `options()`, and `par()`.
#'   \item Prompts the user to select an R script from the current directory to execute.
#'   \item Sources the selected script, catching any errors that occur during execution.
#'   \item Captures the "Before" state (working directory, loaded packages, global objects).
#'   \item Executes the target script.
#'   \item Captures the "After" state and reports the delta.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{script}, and \code{changes_found} (logical).
#'
#' @importFrom utils select.list
#' @importFrom graphics par
#' @importFrom stats setNames
#' @examples
#' if (interactive()) {
#'   audit_script()
#' }
#' @export

audit_script <- function() {
  message("Initializing Silent State Auditor...")
  
  # 1. Ask for the target script
  scripts <- list.files(pattern = "\\.R$", ignore.case = TRUE)
  if (length(scripts) == 0) {
    message("Error: No R scripts found in the current directory.")
    return(invisible(list(status = "error")))
  }
  
  target_script <- select.list(scripts, title = "Select a script to audit:")
  if (target_script == "") {
    message("Audit cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  message(sprintf("\n--- Taking Pre-Execution Snapshot for '%s' ---", target_script))
  
  # 2. Capture the "Before" State
  pre_wd <- getwd()
  pre_opts <- options()
  # par() can fail if no graphics device is active, so we wrap it safely
  pre_par <- tryCatch(par(no.readonly = TRUE), error = function(e) list())
  
  # 3. Execute the Script
  message(sprintf("Running '%s'...", target_script))
  tryCatch({
    source(target_script)
  }, error = function(e) {
    message(sprintf("\n[!] Script crashed with error: %s", e$message))
    message("Auditing state changes made right before the crash...")
  })
  
  # 4. Capture the "After" State
  post_wd <- getwd()
  post_opts <- options()
  post_par <- tryCatch(par(no.readonly = TRUE), error = function(e) list())
  
  # 5. The Interrogation Loop
  changes_found <- FALSE
  message("\n--- Analyzing State Hijacks ---")
  
  # Helper function to generate clean interactive prompts for changes
  revert_prompt <- function(category, item, old_val, new_val) {
    # Keep output clean if values are massive vectors
    old_str <- if(length(old_val) > 1) paste(old_val[1:2], collapse=", ") else deparse(old_val)[1]
    new_str <- if(length(new_val) > 1) paste(new_val[1:2], collapse=", ") else deparse(new_val)[1]
    
    prompt_title <- sprintf("[%s] '%s' changed. \n  Old: %s \n  New: %s \nAction:", 
                            category, item, old_str, new_str)
    
    choice <- select.list(c("Keep New Setting", "Revert to Old Snapshot"), title = prompt_title)
    return(choice == "Revert to Old Snapshot")
  }
  
  # Check 1: Working Directory Hijacks
  if (!identical(pre_wd, post_wd)) {
    changes_found <- TRUE
    if (revert_prompt("Directory", "getwd()", pre_wd, post_wd)) {
      setwd(pre_wd)
      message("-> Reverted: Working directory restored.")
    }
  }
  
  # Check 2: Global Options Hijacks (e.g., scipen, stringsAsFactors)
  opt_names <- unique(c(names(pre_opts), names(post_opts)))
  for (opt in opt_names) {
    old_o <- pre_opts[[opt]]
    new_o <- post_opts[[opt]]
    
    if (!identical(old_o, new_o)) {
      changes_found <- TRUE
      if (revert_prompt("Option", opt, old_o, new_o)) {
        # Dynamically reset the option
        do.call(options, setNames(list(old_o), opt))
        message(sprintf("-> Reverted: option('%s') restored.", opt))
      }
    }
  }
  
  # Check 3: Graphical Parameters Hijacks (e.g., margins, layout)
  if (length(pre_par) > 0 && length(post_par) > 0) {
    par_names <- names(pre_par)
    for (p in par_names) {
      old_p <- pre_par[[p]]
      new_p <- post_par[[p]]
      
      if (!identical(old_p, new_p)) {
        changes_found <- TRUE
        if (revert_prompt("Graphics", p, old_p, new_p)) {
          # Dynamically reset the parameter
          do.call(par, setNames(list(old_p), p))
          message(sprintf("-> Reverted: par('%s') restored.", p))
        }
      }
    }
  }
  
  # 6. Conclusion
  if (!changes_found) {
    message("\nAudit Complete: No global state changes detected. Your environment is safe.")
  } else {
    message("\nAudit Complete: Selected state restorations have been applied to your session.")
  }
  
  return(invisible(list(status = "done", script = target_script, changes_found = changes_found)))
}

# Tests for audit_script()

test_that("audit_script returns early when no scripts found", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  expect_message(audit_script(), "No R scripts found")
})

test_that("audit_script cancels when user selects empty", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "test_script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(audit_script(), "cancelled")
})

test_that("audit_script detects clean script with no side effects", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1 + 1", "clean_script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("clean_script", choices))) return("clean_script.R")
      return("Keep New Setting")
    },
    .package = "devkit"
  )

  expect_message(audit_script(), "No global state changes|safe")
})

test_that("audit_script detects working directory change", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  sub_dir <- file.path(tmp, "subdir")
  dir.create(sub_dir)
  writeLines(sprintf('setwd("%s")', gsub("\\\\", "/", sub_dir)), "wd_changer.R")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("wd_changer.R")
      return("Revert to Old Snapshot")
    },
    .package = "devkit"
  )

  expect_message(audit_script(), "changed|Revert")
})

test_that("audit_script handles script that crashes", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("stop('intentional error')", "crashing_script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("crashing", choices))) return("crashing_script.R")
      return("Keep New Setting")
    },
    .package = "devkit"
  )

  expect_message(audit_script(), "crashed|error")
})

test_that("audit_script detects options changes", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("options(scipen = 999)", "opts_changer.R")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("opts_changer.R")
      return("Revert to Old Snapshot")
    },
    .package = "devkit"
  )

  old_scipen <- getOption("scipen")
  on.exit(options(scipen = old_scipen), add = TRUE)

  expect_message(audit_script(), "changed|Revert|state")
})

test_that("audit_script user keeps new settings", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("options(devkit_test_opt_xyz = TRUE)", "opts_script.R")
  on.exit(options(devkit_test_opt_xyz = NULL), add = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("opts_script.R")
      return("Keep New Setting")
    },
    .package = "devkit"
  )

  audit_script()
  expect_true(isTRUE(getOption("devkit_test_opt_xyz")))
})

#' Remove User-Installed Packages
#'
#' @description
#' Identifies and removes all user-installed R packages from the system, 
#' while carefully preserving base and recommended packages, as well as 
#' packages installed in specific system libraries (e.g., MRO).
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Retrieves a list of all installed packages.
#'   \item Filters out packages located in libraries containing "MRO" to avoid 
#'       corrupting system-specific installations.
#'   \item Filters out packages with a priority of "base" or "recommended" to 
#'       ensure core R functionality remains intact.
#'   \item Identifies the library paths where the remaining user packages are installed.
#'   \item Iteratively removes each identified user package using `remove.packages()`.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done" or
#' "clean") and \code{packages_removed} (character vector).
#'
#' @importFrom utils installed.packages remove.packages
#' @examples
#' if (interactive()) {
#'   remove_user_installed_packages()
#' }
#' @export
remove_user_installed_packages <- function() {
  # installed.packages() is required here: the core purpose of this function
  # is to enumerate ALL installed user packages for bulk removal.
  # This is not a simple existence check and cannot be replaced by requireNamespace().
  instpack <- as.data.frame(utils::installed.packages())
  
  # if you use MRO, make sure that no packages in this library will be removed
  instpack <- subset(instpack, !grepl("MRO", instpack$LibPath))
  
  # we don't want to remove base or recommended packages either
  instpack <- instpack[!(instpack[,"Priority"] %in% c("base", "recommended")),]
  
  # determine the library where the packages are installed
  path <- unique(instpack$LibPath)
  
  # create a vector with all the names of the packages you want to remove
  pkgs.to.remove <- instpack[,1]
  
  if (length(pkgs.to.remove) == 0) {
    message("No user-installed packages found to remove.")
    return(invisible(list(status = "clean", packages_removed = character(0))))
  }
  
  message(sprintf("Removing %d user-installed packages...", length(pkgs.to.remove)))
  
  # remove the packages
  for (pkg in as.character(pkgs.to.remove)) {
    tryCatch(
      remove.packages(pkg, lib = path),
      error = function(e) NULL  # silently skip if pkg not found in lib
    )
  }
  
  message("User-installed packages removed successfully.")
  return(invisible(list(status = "done", packages_removed = as.character(pkgs.to.remove))))
}

# Tests for remove_user_installed_packages()

test_that("remove_user_installed_packages returns early when no user packages", {
  local_mocked_bindings(installed.packages = function(...) {
      mat <- data.frame(
        Package = c("base", "stats"),
        LibPath = c("/usr/lib/R", "/usr/lib/R"),
        Priority = c("base", "recommended"),
        stringsAsFactors = FALSE
      )
      as.matrix(mat)
    },
    .package = "utils"
  )

  expect_message(
    remove_user_installed_packages(),
    "No user-installed packages"
  )
})

test_that("remove_user_installed_packages filters out MRO lib paths", {
  local_mocked_bindings(
    installed.packages = function(...) {
      mat <- data.frame(
        Package = c("pkg1", "pkg2"),
        LibPath = c("/usr/lib/MRO", "/usr/lib/R"),
        Priority = c(NA, NA),
        stringsAsFactors = FALSE
      )
      as.matrix(mat)
    }, .package = "utils")
  local_mocked_bindings(remove.packages = function(pkg, ...) invisible(NULL), .package = "devkit")

  expect_message(
    remove_user_installed_packages(),
    "Removing 1 user-installed"
  )
})

test_that("remove_user_installed_packages filters out base and recommended", {
  local_mocked_bindings(installed.packages = function(...) {
      mat <- data.frame(
        Package = c("base", "recommended_pkg", "userpkg"),
        LibPath = rep("/home/user/R", 3),
        Priority = c("base", "recommended", NA),
        stringsAsFactors = FALSE
      )
      as.matrix(mat)
    }, .package = "utils")
  local_mocked_bindings(remove.packages = function(pkg, ...) invisible(NULL), .package = "devkit")

  expect_message(
    remove_user_installed_packages(),
    "Removing 1 user-installed"
  )
})

test_that("remove_user_installed_packages calls remove.packages for each user pkg", {
  removed_pkgs <- character(0)
  local_mocked_bindings(installed.packages = function(...) {
      mat <- data.frame(
        Package = c("userpkg1", "userpkg2"),
        LibPath = rep("/home/user/R", 2),
        Priority = c(NA, NA),
        stringsAsFactors = FALSE
      )
      as.matrix(mat)
    }, .package = "utils")
  local_mocked_bindings(remove.packages = function(pkg, ...) {
      removed_pkgs <<- c(removed_pkgs, pkg)
      invisible(NULL)
    }, .package = "devkit")

  remove_user_installed_packages()
  expect_true("userpkg1" %in% removed_pkgs)
  expect_true("userpkg2" %in% removed_pkgs)
})

test_that("remove_user_installed_packages reports success", {
  local_mocked_bindings(installed.packages = function(...) {
      mat <- data.frame(
        Package = "userpkg",
        LibPath = "/home/user/R",
        Priority = NA,
        stringsAsFactors = FALSE
      )
      as.matrix(mat)
    }, .package = "utils")
  local_mocked_bindings(remove.packages = function(pkg, ...) invisible(NULL), .package = "devkit")

  expect_message(
    remove_user_installed_packages(),
    "removed successfully"
  )
})

#' Interactive Vignette Architect
#'
#' @description
#' Scaffolds a CRAN-compliant RMarkdown vignette by interactively prompting 
#' the user for metadata, narrative structure, and target functions to highlight.
#'
#' @details
#' The function guides the user through the following process:
#' \enumerate{
#'   \item Collects package name and vignette title.
#'   \item Prompts for a structural template (Quick Start, Deep Dive, or Case Study).
#'   \item Requests a list of core functions to be featured in the vignette.
#'   \item Generates a `.Rmd` file with a proper YAML header and a narrative scaffold 
#'       based on the selected template.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{file_path}, \code{package}, \code{title},
#' \code{template}, and \code{functions} (character vector).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   architect_vignette()
#' }
#' @export

architect_vignette <- function() {
  message("Initializing Vignette Architect...")
  
  # 1. Gather Metadata via Dynamic Prompts
  pkg_name <- readline(prompt = "Enter your package name: ")
  if (trimws(pkg_name) == "") {
    message("Package name is required. Aborting.")
    return(invisible(list(status = "error")))
  }
  
  vig_title <- readline(prompt = "Enter the Vignette Title (e.g., 'Getting Started'): ")
  if (trimws(vig_title) == "") vig_title <- "Package Vignette"
  
  # Create a clean filename from the title
  file_base <- tolower(gsub("[^[:alnum:]]", "_", vig_title))
  file_name <- sprintf("%s.Rmd", file_base)
  
  # 2. Determine the Narrative Structure
  vig_type <- select.list(
    choices = c("Quick Start Guide", "Deep Dive / Advanced Usage", "Case Study / Workflow"),
    title = "Select the structural template for this vignette:"
  )
  if (vig_type == "") {
    message("Scaffolding cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  # 3. Gather Target Functions
  funcs_input <- readline(prompt = "List the core functions to highlight (comma-separated, e.g., 'sweep_memory, audit_state'): ")
  funcs_list <- trimws(unlist(strsplit(funcs_input, ",")))
  funcs_list <- funcs_list[funcs_list != ""]
  
  # 4. Construct the CRAN-Compliant YAML Header
  yaml_block <- c(
    "---",
    sprintf("title: \"%s\"", vig_title),
    "output: rmarkdown::html_vignette",
    "vignette: >",
    sprintf("  %%\\VignetteIndexEntry{%s}", vig_title),
    "  %\\VignetteEngine{knitr::rmarkdown}",
    "  %\\VignetteEncoding{UTF-8}",
    "---",
    "",
    "```{r, include = FALSE}",
    "knitr::opts_chunk$set(",
    "  collapse = TRUE,",
    "  comment = \"#>\"",
    ")",
    "```",
    "",
    sprintf("```{r setup}"),
    sprintf("library(%s)", pkg_name),
    "```",
    ""
  )
  
  # 5. Construct the Narrative Scaffold
  body_block <- c(
    sprintf("# Introduction"),
    sprintf("This vignette demonstrates the core functionality of the `%s` package.", pkg_name),
    ""
  )
  
  if (vig_type == "Quick Start Guide") {
    body_block <- c(body_block, 
                    "## Installation",
                    "Briefly explain how to install and attach the package.",
                    "",
                    "## Basic Usage",
                    "Provide a minimal reproducible example."
    )
  } else if (vig_type == "Deep Dive / Advanced Usage") {
    body_block <- c(body_block, 
                    "## Core Concepts",
                    "Explain the underlying philosophy or architecture.",
                    "",
                    "## Advanced Configuration",
                    "Detail the optional parameters and edge cases."
    )
  } else {
    body_block <- c(body_block, 
                    "## The Problem",
                    "Describe the real-world scenario.",
                    "",
                    "## The Solution",
                    "Walk through the workflow step-by-step."
    )
  }
  
  body_block <- c(body_block, "")
  
  # 6. Inject Function-Specific Code Chunks
  if (length(funcs_list) > 0) {
    body_block <- c(body_block, "# Core Functions Walkthrough", "")
    for (func in funcs_list) {
      body_block <- c(
        body_block,
        sprintf("## Using `%s()`", func),
        sprintf("Describe what `%s()` does and when to use it.", func),
        "",
        sprintf("```{r example_%s}", func),
        sprintf("# Add example code for %s() here", func),
        "```",
        ""
      )
    }
  }
  
  # 7. File System Operations
  if (!dir.exists("vignettes")) {
    dir.create("vignettes")
    message("Created 'vignettes/' directory.")
  }
  
  file_path <- file.path("vignettes", file_name)
  
  if (file.exists(file_path)) {
    overwrite <- select.list(c("Yes", "No"), title = sprintf("'%s' already exists. Overwrite?", file_name))
    if (overwrite != "Yes") {
      message("Scaffolding aborted to protect existing file.")
      return(invisible(list(status = "cancelled")))
    }
  }
  
  # 8. Write the final file
  writeLines(c(yaml_block, body_block), con = file_path)
  message(sprintf("\nSuccess! Scaffolding complete. Open 'vignettes/%s' to start writing.", file_name))
  
  return(invisible(list(
    status = "done",
    file_path = file_path,
    package = pkg_name,
    title = vig_title,
    template = vig_type,
    functions = funcs_list
  )))
}

# Tests for architect_vignette()

test_that("architect_vignette returns early with empty package name", {
  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  expect_message(architect_vignette(), "required")
})

test_that("architect_vignette defaults title when empty", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("mypkg")
      if (readline_count == 2L) return("")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Quick Start", choices))) return("Quick Start Guide")
      return("Done")
    },
    .package = "devkit"
  )

  architect_vignette()
  expect_true(file.exists(file.path("vignettes", "package_vignette.Rmd")))
})

test_that("architect_vignette cancels when vig_type is empty", {
  local_mocked_bindings(
    readline = function(prompt = "", ...) "mypkg",
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(architect_vignette(), "cancelled")
})

test_that("architect_vignette creates Quick Start template", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("mypkg")
      if (readline_count == 2L) return("Getting Started")
      return("func_a, func_b")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Quick Start", choices))) return("Quick Start Guide")
      if (any(grepl("Overwrite", choices %||% ""))) return("No")
      return("Done")
    },
    .package = "devkit"
  )

  architect_vignette()
  rmd_path <- file.path("vignettes", "getting_started.Rmd")
  expect_true(file.exists(rmd_path))
  content <- readLines(rmd_path)
  expect_true(any(grepl("Installation", content)))
  expect_true(any(grepl("func_a", content)))
})

test_that("architect_vignette creates Deep Dive template", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("mypkg")
      if (readline_count == 2L) return("Advanced Usage")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Deep Dive", choices))) return("Deep Dive / Advanced Usage")
      return("Done")
    },
    .package = "devkit"
  )

  architect_vignette()
  content <- readLines(file.path("vignettes", "advanced_usage.Rmd"))
  expect_true(any(grepl("Core Concepts", content)))
})

test_that("architect_vignette creates Case Study template", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("mypkg")
      if (readline_count == 2L) return("My Workflow")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Case Study", choices))) return("Case Study / Workflow")
      return("Done")
    },
    .package = "devkit"
  )

  architect_vignette()
  content <- readLines(file.path("vignettes", "my_workflow.Rmd"))
  expect_true(any(grepl("Problem", content)))
})

test_that("architect_vignette creates vignettes directory if missing", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("mypkg")
      if (readline_count == 2L) return("Test")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Quick Start", choices))) return("Quick Start Guide")
      return("Done")
    },
    .package = "devkit"
  )

  expect_false(dir.exists("vignettes"))
  architect_vignette()
  expect_true(dir.exists("vignettes"))
})

test_that("architect_vignette aborts on existing file when user says No", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  dir.create("vignettes", showWarnings = FALSE)
  writeLines("existing", file.path("vignettes", "test.Rmd"))

  readline_count <- 0L
  select_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("mypkg")
      if (readline_count == 2L) return("Test")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("Quick Start Guide")
      return("No")
    },
    .package = "devkit"
  )

  expect_message(architect_vignette(), "aborted|protect")
})

test_that("architect_vignette YAML header contains VignetteEngine", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("mypkg")
      if (readline_count == 2L) return("My Vignette")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Quick Start", choices))) return("Quick Start Guide")
      return("Done")
    },
    .package = "devkit"
  )

  architect_vignette()
  content <- readLines(file.path("vignettes", "my_vignette.Rmd"))
  expect_true(any(grepl("VignetteEngine", content)))
  expect_true(any(grepl("VignetteIndexEntry", content)))
})

#' Function Masking Detective
#'
#' @description
#' Scans for namespace conflicts among attached packages and interactively
#' helps the user resolve them. The function adapts its recommendations
#' based on whether it detects a package development environment or a
#' standalone analysis script.
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Identifies all functions that are masked by multiple attached packages.
#'   \item Interactively prompts the user to select the preferred package for each conflicting function.
#'   \item If a \code{DESCRIPTION} file is present (Package Development context), it suggests
#'       updating the \code{Imports} field and adding \code{@importFrom} roxygen2 tags.
#'   \item If no \code{DESCRIPTION} file is present (Standalone context), it generates a
#'       code snippet to explicitly assign the preferred functions and offers to apply them.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return
#' Invisibly returns a named list with components: \code{status} ("done" or
#' "clean"), \code{conflicts} (named list), \code{resolutions} (named list),
#' and \code{context} ("package" or "standalone").
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   detect_masking()
#' }
#' @export

detect_masking <- function() {
  # 1. Identify conflicts across all attached packages
  all_confs <- conflicts(detail = TRUE)

  # Filter for conflicts that only involve multiple attached packages
  pkg_confs <- list()
  for (func in names(all_confs)) {
    envs <- all_confs[[func]]
    # Extract only package environments, ignoring the global env
    pkg_envs <- envs[grepl("^package:", envs)]
    if (length(pkg_envs) > 1) {
      pkg_confs[[func]] <- gsub("^package:", "", pkg_envs)
    }
  }

  if (length(pkg_confs) == 0) {
    message("No cross-package function masking detected. Your namespace is clean.")
    return(invisible(list(status = "clean", conflicts = list(), resolutions = list())))
  }

  message(sprintf("Detected %d masked functions.", length(pkg_confs)))

  # 2. Interactive Step-by-Step Resolution
  resolutions <- list()
  for (func in names(pkg_confs)) {
    pkgs <- pkg_confs[[func]]

    prompt_title <- sprintf("Conflict for '%s()'. Which package should take priority?", func)
    chosen_pkg <- select.list(
      choices = c(pkgs, "Skip/Ignore"),
      title = prompt_title
    )

    if (chosen_pkg != "" && chosen_pkg != "Skip/Ignore") {
      resolutions[[func]] <- chosen_pkg
    }
  }

  if (length(resolutions) == 0) {
    message("No conflict resolutions selected.")
    return(invisible(list(status = "done", conflicts = pkg_confs, resolutions = list())))
  }

  # 3. Determine Context: Package Development vs. Standalone Script
  is_pkg_dev <- file.exists("DESCRIPTION")

  message("\n--- Recommended Resolution Actions ---")

  if (is_pkg_dev) {
    message("Context: Package Development (Detected DESCRIPTION file in root)")

    unique_pkgs <- unique(unlist(resolutions))
    message("\n1. Ensure the following are listed under 'Imports:' in your DESCRIPTION file:")
    message(paste("   Imports:", paste(unique_pkgs, collapse = ", ")))

    message("\n2. Add these roxygen2 tags to your package documentation to safely lock the namespace:")
    for (func in names(resolutions)) {
      message(sprintf("   #' @importFrom %s %s", resolutions[[func]], func))
    }
    message("\n   (Alternatively, strict `pkg::func()` syntax can be used within your source code).")

  } else {
    message("Context: Standalone Analysis Script")
    message("\nAdd the following snippet at the top of your script (immediately AFTER your library() calls) to lock in your namespace priorities:\n")

    # Generate copy-pasteable script block
    cat("# --- Namespace Conflict Resolutions ---\n")
    for (func in names(resolutions)) {
      cat(sprintf("%s <- %s::%s\n", func, resolutions[[func]], func))
    }
    cat("# --------------------------------------\n")

    # Optional: Apply instantly to the current session
    apply_now <- select.list(
      choices = c("Yes", "No"),
      title = "\nApply these resolutions to your current active session right now?"
    )

    if (apply_now == "Yes") {
      message("\nApplying resolutions to the calling environment...")
      caller_env <- parent.frame()
      for (func in names(resolutions)) {
        # Assign to caller's environment (user has explicitly consented via prompt above)
        assign(func, get(func, envir = asNamespace(resolutions[[func]])), envir = caller_env)
      }
      message("Success: Priorities applied to the current environment.")
    }
  }

  return(invisible(list(
    status = "done",
    conflicts = pkg_confs,
    resolutions = resolutions,
    context = ifelse(is_pkg_dev, "package", "standalone")
  )))
}

# Tests for detect_masking()

test_that("detect_masking returns clean when no conflicts", {
  local_mocked_bindings(
    conflicts = function(...) list(),
    .package = "base"
  )

  expect_message(detect_masking(), "clean")
})

test_that("detect_masking returns early when no resolutions selected", {
  local_mocked_bindings(
    conflicts = function(...) {
      list(filter = c("package:dplyr", "package:stats"))
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "Skip/Ignore",
    .package = "devkit"
  )

  expect_message(detect_masking(), "No conflict resolutions")
})

test_that("detect_masking filters out non-package environments", {
  local_mocked_bindings(
    conflicts = function(...) {
      list(myvar = c(".GlobalEnv"))
    },
    .package = "base"
  )

  expect_message(detect_masking(), "clean")
})

test_that("detect_masking handles package development context", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines(c("Package: testpkg", "Version: 1.0.0"), "DESCRIPTION")

  local_mocked_bindings(
    conflicts = function(...) {
      list(filter = c("package:dplyr", "package:stats"))
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Skip", choices))) return("dplyr")
      return("No")
    },
    .package = "devkit"
  )

  expect_message(detect_masking(), "Package Development|importFrom")
})

test_that("detect_masking handles standalone context with apply", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    conflicts = function(...) {
      list(lag = c("package:dplyr", "package:stats"))
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("dplyr")
      if (any(grepl("Yes", choices))) return("No")
      return("No")
    },
    .package = "devkit"
  )

  capture.output(
    expect_message(detect_masking(), "Conflict|Resolution|lag"),
    type = "output"
  )
})

test_that("detect_masking handles multiple conflicts", {
  local_mocked_bindings(
    conflicts = function(...) {
      list(
        filter = c("package:dplyr", "package:stats"),
        lag = c("package:dplyr", "package:stats")
      )
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "Skip/Ignore",
    .package = "devkit"
  )

  expect_message(detect_masking(), "2 masked|No conflict")
})

#' Interactive Dependency Diplomat (Base R Edition)
#'
#' @description
#' Scans package source code for external package calls and cross-references 
#' them against the `DESCRIPTION` file to interactively resolve missing, unused, 
#' or misclassified dependencies without requiring any external tools.
#'
#' @details
#' The function implements a zero-dependency approach to dependency auditing:
#' \enumerate{
#'   \item Parses the `DESCRIPTION` file to identify currently declared `Imports` and `Suggests`.
#'   \item Recursively scans the `R/`, `tests/`, and `vignettes/` directories for `::` calls, 
#'       `library()` calls, and `require()` calls.
#'   \item Filters out base R packages.
#'   \item Interactively prompts the user to add missing dependencies or remove unused ones 
#'       from the `DESCRIPTION` file.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "clean", or "error"), \code{ghost_deps}, \code{bloat_deps},
#' \code{misclassified}, and \code{description_modified} (logical).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   audit_dependencies()
#' }
#' @export

audit_dependencies <- function() {
  message("Initializing Dependency Diplomat (Zero-Dependency Edition)...")
  
  # 1. Verify Package Infrastructure
  if (!file.exists("DESCRIPTION")) {
    message("Error: DESCRIPTION file not found. Please run from the package root.")
    return(invisible(list(status = "error")))
  }
  
  # 2. Base R DCF Parsers and Modifiers
  desc <- read.dcf("DESCRIPTION")
  desc_modified <- FALSE
  
  # Helper to get clean package names (ignoring versions)
  get_clean_deps <- function(field) {
    if (field %in% colnames(desc)) {
      deps <- unlist(strsplit(desc[, field], ","))
      deps <- trimws(gsub("\\(.*?\\)", "", deps))
      return(deps[deps != "" & deps != "R"])
    }
    return(character())
  }
  
  # Helper to safely add a package to a field
  add_dep <- function(pkg, field) {
    if (field %in% colnames(desc)) {
      current_val <- desc[, field]
      desc[, field] <<- paste0(trimws(current_val), ",\n    ", pkg)
    } else {
      new_col <- matrix(pkg, ncol = 1)
      colnames(new_col) <- field
      desc <<- cbind(desc, new_col)
    }
    desc_modified <<- TRUE
  }
  
  # Helper to safely remove a package (preserving versions of other packages)
  remove_dep <- function(pkg, field) {
    if (field %in% colnames(desc)) {
      parts <- unlist(strsplit(desc[, field], ","))
      # Regex matches the exact package name, optionally followed by (>= version)
      pattern <- sprintf("^\\s*%s(\\s*\\(.*\\))?\\s*$", pkg)
      keep_parts <- parts[!grepl(pattern, parts)]
      
      if (length(keep_parts) > 0) {
        desc[, field] <<- paste(trimws(keep_parts), collapse = ",\n    ")
      } else {
        # If it was the only package, remove the field entirely
        desc <<- desc[, colnames(desc) != field, drop = FALSE]
      }
      desc_modified <<- TRUE
    }
  }
  
  current_imports <- get_clean_deps("Imports")
  current_suggests <- get_clean_deps("Suggests")
  all_declared <- c(current_imports, current_suggests)
  
  # 3. Scan Source Code for Actual Usage
  regex_colon <- "([a-zA-Z0-9\\.]+)::[a-zA-Z0-9\\._]+"
  regex_library <- "library\\(\\s*([a-zA-Z0-9\\.]+)\\s*\\)|require\\(\\s*([a-zA-Z0-9\\.]+)\\s*\\)"
  
  scan_directory <- function(dir_name) {
    if (!dir.exists(dir_name)) return(character())
    files <- list.files(dir_name, pattern = "\\.(R|Rmd)$", full.names = TRUE, recursive = TRUE, ignore.case = TRUE)
    
    found_pkgs <- character()
    for (f in files) {
      lines <- readLines(f, warn = FALSE)
      
      colons <- regmatches(lines, gregexpr(regex_colon, lines))
      colons <- unlist(colons)
      if (length(colons) > 0) found_pkgs <- c(found_pkgs, gsub("::.*", "", colons))
      
      libs <- regmatches(lines, gregexpr(regex_library, lines))
      libs <- unlist(libs)
      if (length(libs) > 0) {
        clean_libs <- gsub("library\\(|require\\(|\\)|\\s", "", libs)
        found_pkgs <- c(found_pkgs, clean_libs)
      }
    }
    return(unique(found_pkgs))
  }
  
  message("Scanning source code for dependencies...")
  used_in_R <- scan_directory("R")
  used_in_tests <- scan_directory("tests")
  used_in_vigs <- scan_directory("vignettes")
  
  all_used <- unique(c(used_in_R, used_in_tests, used_in_vigs))
  base_pkgs <- .base_pkgs
  all_used <- setdiff(all_used, base_pkgs)
  used_in_R <- setdiff(used_in_R, base_pkgs)
  
  # 4. Identify Discrepancies
  ghost_deps <- setdiff(all_used, all_declared)
  bloat_deps <- setdiff(all_declared, all_used)
  misclassified <- intersect(setdiff(current_imports, used_in_R), c(used_in_tests, used_in_vigs))
  
  if (length(ghost_deps) == 0 && length(bloat_deps) == 0 && length(misclassified) == 0) {
    message("\nPerfect! Your DESCRIPTION file perfectly matches your source code.")
    return(invisible(list(status = "clean")))
  }
  
  message("\n--- Dependency Discrepancies Found ---")
  
  # 5. Resolve Ghost Dependencies (Used but not declared)
  if (length(ghost_deps) > 0) {
    message(sprintf("\n[!] GHOST DEPENDENCIES: %d packages used in code but missing from DESCRIPTION.", length(ghost_deps)))
    for (pkg in ghost_deps) {
      target_field <- ifelse(pkg %in% used_in_R, "Imports", "Suggests")
      if (select.list(c("Yes", "No"), title = sprintf("Add '%s' to %s?", pkg, target_field)) == "Yes") {
        add_dep(pkg, target_field)
      }
    }
  }
  
  # 6. Resolve Misclassified (Should be Suggests, not Imports)
  if (length(misclassified) > 0) {
    message(sprintf("\n[!] MISCLASSIFIED: %d packages in 'Imports' are only used in tests/vignettes.", length(misclassified)))
    for (pkg in misclassified) {
      if (select.list(c("Yes", "No"), title = sprintf("Move '%s' from Imports to Suggests?", pkg)) == "Yes") {
        remove_dep(pkg, "Imports")
        add_dep(pkg, "Suggests")
      }
    }
  }
  
  # 7. Resolve Bloat (Declared but never used)
  if (length(bloat_deps) > 0) {
    message(sprintf("\n[!] BLOAT DEPENDENCIES: %d packages in DESCRIPTION are never called in your code.", length(bloat_deps)))
    for (pkg in bloat_deps) {
      # Determine where it currently lives to construct the prompt
      loc <- ifelse(pkg %in% current_imports, "Imports", "Suggests")
      if (select.list(c("Yes", "No"), title = sprintf("Remove unused '%s' from %s?", pkg, loc)) == "Yes") {
        remove_dep(pkg, loc)
      }
    }
  }
  
  # 8. Save Changes
  if (desc_modified) {
    write.dcf(desc, "DESCRIPTION")
    message("\n-> Success! DESCRIPTION file successfully updated using Base R.")
  } else {
    message("\n-> Audit complete. No changes were written to the DESCRIPTION file.")
  }
  
  return(invisible(list(
    status = "done",
    ghost_deps = ghost_deps,
    bloat_deps = bloat_deps,
    misclassified = misclassified,
    description_modified = desc_modified
  )))
}

# Tests for audit_dependencies()

test_that("audit_dependencies returns early when DESCRIPTION missing", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  expect_message(audit_dependencies(), "DESCRIPTION file not found")
})

test_that("audit_dependencies reports when no discrepancies", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  dir.create("R", showWarnings = FALSE)
  writeLines("dplyr::filter(iris, Species == 'setosa')", file.path("R", "example.R"))

  desc_content <- c("Package: testpkg", "Version: 1.0.0",
                     "Imports: dplyr")
  writeLines(desc_content, "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  expect_message(audit_dependencies(), "Perfect|complete|Audit")
})

test_that("audit_dependencies detects ghost dependencies", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  dir.create("R", showWarnings = FALSE)
  writeLines("dplyr::filter(iris, Species == 'setosa')",
             file.path("R", "example.R"))

  desc_content <- c("Package: testpkg", "Version: 1.0.0")
  writeLines(desc_content, "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  expect_message(audit_dependencies(), "GHOST")
})

test_that("audit_dependencies adds ghost dep when user confirms", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  dir.create("R", showWarnings = FALSE)
  writeLines("ggplot2::ggplot()", file.path("R", "example.R"))

  desc_content <- c("Package: testpkg", "Version: 1.0.0")
  writeLines(desc_content, "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Yes", choices))) return("Yes")
      return("No")
    },
    .package = "devkit"
  )

  audit_dependencies()
  desc <- read.dcf("DESCRIPTION")
  expect_true("Imports" %in% colnames(desc))
})

test_that("audit_dependencies scans tests and vignettes directories", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  dir.create("R", showWarnings = FALSE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)
  writeLines("# no deps", file.path("R", "main.R"))
  writeLines("testthat::expect_true(TRUE)",
             file.path("tests", "testthat", "test-main.R"))

  desc_content <- c("Package: testpkg", "Version: 1.0.0",
                     "Suggests: testthat")
  writeLines(desc_content, "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  expect_message(audit_dependencies(), "Perfect|GHOST|BLOAT|MISCLASS")
})

test_that("audit_dependencies detects library() usage patterns", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  dir.create("R", showWarnings = FALSE)
  writeLines("library(jsonlite)", file.path("R", "example.R"))

  desc_content <- c("Package: testpkg", "Version: 1.0.0")
  writeLines(desc_content, "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  expect_message(audit_dependencies(), "GHOST")
})

test_that("audit_dependencies detects bloat dependencies", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  dir.create("R", showWarnings = FALSE)
  writeLines("# no code", file.path("R", "empty.R"))

  desc_content <- c("Package: testpkg", "Version: 1.0.0",
                     "Imports: ggplot2")
  writeLines(desc_content, "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  expect_message(audit_dependencies(), "BLOAT")
})

test_that("audit_dependencies handles non-existent scan directories", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  desc_content <- c("Package: testpkg", "Version: 1.0.0")
  writeLines(desc_content, "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  expect_message(audit_dependencies(), "Perfect")
})

#' Interactive Memory Sweeper
#'
#' @description
#' Scans the global environment for memory-intensive objects and interactively 
#' prompts the user to remove them to free up system RAM.
#'
#' @details
#' The function implements a simple memory management workflow:
#' \enumerate{
#'   \item Prompts the user to define a size threshold (in MB) for flagging objects.
#'   \item Calculates the size of all objects currently residing in the global environment.
#'   \item Identifies and sorts objects that exceed the specified threshold.
#'   \item Presents a selection menu allowing the user to choose one or more large 
#'       objects for removal.
#'   \item Executes `rm()` on the selected objects and immediately calls `gc()` 
#'       to ensure the memory is released back to the system.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "clean", or "cancelled"), \code{threshold_mb} (numeric), and
#' \code{objects_removed} (character vector of removed object names).
#'
#' @importFrom utils object.size select.list
#' @examples
#' if (interactive()) {
#'   sweep_memory()
#' }
#' @export

sweep_memory <- function() {
  # Dynamically ask for the threshold during execution
  threshold <- .read_numeric(
    prompt = "Enter minimum object size to flag (in MB, e.g., 50): ",
    default = 50,
    default_msg = "Invalid input. Defaulting to 50 MB."
  )

  env_objs <- ls(envir = .GlobalEnv)
  if (length(env_objs) == 0) {
    message("The global environment is empty.")
    return(invisible(list(status = "clean", threshold_mb = threshold, objects_removed = character(0))))
  }

  # Calculate object sizes in MB
  obj_sizes <- sapply(env_objs, function(x) object.size(get(x, envir = .GlobalEnv)) / (1024^2))
  large_objs <- obj_sizes[obj_sizes >= threshold]

  if (length(large_objs) == 0) {
    message(sprintf("No objects found larger than %s MB.", threshold))
    return(invisible(list(status = "clean", threshold_mb = threshold, objects_removed = character(0))))
  }

  # Format choices for the selection menu
  large_objs <- sort(large_objs, decreasing = TRUE)
  display_names <- sprintf("%s (%.2f MB)", names(large_objs), large_objs)

  # Trigger interactive prompt
  to_remove <- select.list(
    choices = display_names,
    multiple = TRUE,
    title = "Select large objects to wipe from memory (0 to cancel):"
  )

  if (length(to_remove) > 0) {
    actual_names <- sapply(strsplit(to_remove, " \\("), `[`, 1)
    rm(list = actual_names, envir = .GlobalEnv)
    gc() # Force garbage collection to instantly free up RAM
    message(sprintf("Successfully removed %d objects and cleared memory.", length(actual_names)))
    return(invisible(list(status = "done", threshold_mb = threshold, objects_removed = actual_names)))
  } else {
    message("Memory sweep cancelled.")
    return(invisible(list(status = "cancelled", threshold_mb = threshold, objects_removed = character(0))))
  }
}

# Tests for sweep_memory()

test_that("sweep_memory returns early when global env is empty", {
  local_mocked_bindings(
    readline = function(...) "50",
    ls = function(...) character(0),
    .package = "base"
  )

  expect_message(sweep_memory(), "empty")
})

test_that("sweep_memory defaults threshold on invalid input", {
  local_mocked_bindings(
    readline = function(...) "abc",
    ls = function(...) character(0),
    .package = "base"
  )

  expect_message(sweep_memory(), "Defaulting|empty|50")
})

test_that("sweep_memory returns early when no objects exceed threshold", {
  assign("devkit_test_small_obj", 1:10, envir = .GlobalEnv)
  on.exit(rm("devkit_test_small_obj", envir = .GlobalEnv), add = TRUE)

  local_mocked_bindings(
    readline = function(...) "9999",
    .package = "base"
  )

  expect_message(sweep_memory(), "No objects found")
})

test_that("sweep_memory removes selected objects", {
  assign("devkit_test_big_obj", rep(1, 1e6), envir = .GlobalEnv)
  on.exit(suppressWarnings(rm("devkit_test_big_obj", envir = .GlobalEnv)),
          add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) choices[1],
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "1",
    .package = "base"
  )

  expect_message(sweep_memory(), "removed.*cleared")
})

test_that("sweep_memory cancels when user selects nothing", {
  assign("devkit_test_cancel_obj", rep(1, 1e6), envir = .GlobalEnv)
  on.exit(suppressWarnings(rm("devkit_test_cancel_obj", envir = .GlobalEnv)),
          add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) character(0),
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "1",
    .package = "base"
  )

  expect_message(sweep_memory(), "cancelled")
})

test_that("sweep_memory sorts objects by size descending", {
  assign("devkit_test_big", rep(1, 1e6), envir = .GlobalEnv)    # ~8 MB
  assign("devkit_test_small", rep(1, 2e5), envir = .GlobalEnv)  # ~1.6 MB
  on.exit({
    suppressWarnings(rm("devkit_test_big", envir = .GlobalEnv))
    suppressWarnings(rm("devkit_test_small", envir = .GlobalEnv))
  }, add = TRUE)

  choices_offered <- NULL
  local_mocked_bindings(
    select.list = function(choices, ...) {
      choices_offered <<- choices
      character(0)
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "1",
    .package = "base"
  )

  suppressMessages(sweep_memory())
  expect_true(!is.null(choices_offered))
  expect_gte(length(choices_offered), 2)
  # First item should be the bigger object
  expect_true(grepl("devkit_test_big", choices_offered[1]))
})

#' Session Sentinel
#'
#' @description
#' Interactively configures dual-logging for the current R session, 
#' routing messages, warnings, and errors to a text file while 
#' maintaining live console output.
#'
#' @details
#' The function implements a background logging system using R's 
#' global calling handlers:
#' \enumerate{
#'   \item Verifies that the R version is 4.0.0 or higher, as global 
#'       calling handlers are required.
#'   \item Prompts the user to select a logging level: either "All Output" 
#'       (Messages, Warnings, and Errors) or "Errors Only".
#'   \item Prompts for a log filename, defaulting to a timestamped 
#'       `session_log_YYYYMMDD_HHMM.txt`.
#'   \item Attaches `globalCallingHandlers` to the session, which 
#'       intercepts conditions and appends them to the log file with 
#'       a timestamp and type label.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{log_file}, and \code{log_level}.
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   setup_sentinel()
#' }
#' @export

setup_sentinel <- function() {
  message("Initializing Session Sentinel...")
  
  # 1. Verify R Version capability
  if (getRversion() < "4.0.0") {
    message("Error: Session Sentinel requires R 4.0.0 or higher for global calling handlers.")
    return(invisible(list(status = "error")))
  }
  
  # 2. Ask for Logging Level
  log_level <- select.list(
    choices = c("All Output (Messages, Warnings, Errors)", "Errors Only", "Cancel"),
    title = "Would you like to log this session?"
  )
  
  if (log_level == "Cancel" || log_level == "") {
    message("Sentinel setup cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  # 3. Ask for Log Filename dynamically
  default_name <- sprintf("session_log_%s.txt", format(Sys.time(), "%Y%m%d_%H%M"))
  prompt_text <- sprintf("Enter log filename (or press Enter for '%s'): ", default_name)
  
  log_file <- readline(prompt = prompt_text)
  if (trimws(log_file) == "") log_file <- default_name
  
  # Ensure the file exists and write the header
  file.create(log_file, showWarnings = FALSE)
  cat(sprintf("--- Session Log Started: %s ---\n", Sys.time()), file = log_file, append = TRUE)
  
  # 4. Define the routing logic
  # This helper formats the output with a timestamp and the message type
  write_to_log <- function(type, msg) {
    timestamp <- format(Sys.time(), "%Y-%m-%d %H:%M:%S")
    # Clean up the message string to prevent double line-breaks
    clean_msg <- trimws(msg)
    log_entry <- sprintf("[%s] [%s] %s\n", timestamp, type, clean_msg)
    
    cat(log_entry, file = log_file, append = TRUE)
  }
  
  # 5. Attach the Global Calling Handlers
  # These sit in the background and catch conditions as they happen,
  # routing them to our file without stopping them from printing to the console.
  
  if (log_level == "All Output (Messages, Warnings, Errors)") {
    globalCallingHandlers(
      message = function(m) { write_to_log("MESSAGE", m$message) },
      warning = function(w) { write_to_log("WARNING", w$message) },
      error   = function(e) { write_to_log("ERROR", e$message) }
    )
    message(sprintf("\nSentinel Active: Dual-logging ALL output to '%s'", log_file))
    
  } else {
    globalCallingHandlers(
      error = function(e) { write_to_log("ERROR", e$message) }
    )
    message(sprintf("\nSentinel Active: Dual-logging ERRORS ONLY to '%s'", log_file))
  }
  
  message("You can now safely run your batch processes.")
  return(invisible(list(
    status = "done",
    log_file = log_file,
    log_level = log_level
  )))
}

# Tests for setup_sentinel()

test_that("setup_sentinel cancels when user selects Cancel", {
  local_mocked_bindings(
    select.list = function(choices, ...) "Cancel",
    .package = "devkit"
  )

  expect_message(setup_sentinel(), "cancelled")
})

test_that("setup_sentinel cancels on empty selection", {
  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(setup_sentinel(), "cancelled")
})

test_that("setup_sentinel creates log file with All Output mode", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "All Output (Messages, Warnings, Errors)",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "test_log.txt",
    globalCallingHandlers = function(...) invisible(NULL),
    .package = "base"
  )

  result <- setup_sentinel()
  expect_true(file.exists("test_log.txt"))
  content <- readLines("test_log.txt")
  expect_true(any(grepl("Session Log Started", content)))
  expect_equal(result$status, "done")
})

test_that("setup_sentinel creates log file with Errors Only mode", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "Errors Only",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "error_log.txt",
    globalCallingHandlers = function(...) invisible(NULL),
    .package = "base"
  )

  result <- setup_sentinel()
  expect_true(file.exists("error_log.txt"))
  expect_equal(result$status, "done")
})

test_that("setup_sentinel uses default filename when empty", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "Errors Only",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "",
    globalCallingHandlers = function(...) invisible(NULL),
    .package = "base"
  )

  result <- setup_sentinel()
  log_files <- list.files(tmp, pattern = "^session_log_")
  expect_true(length(log_files) >= 1)
  expect_equal(result$status, "done")
})

test_that("setup_sentinel returns invisible TRUE on success", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "Errors Only",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "sentinel.txt",
    globalCallingHandlers = function(...) invisible(NULL),
    .package = "base"
  )

  result <- setup_sentinel()
  expect_equal(result$status, "done")
})

test_that("setup_sentinel writes header with timestamp", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "All Output (Messages, Warnings, Errors)",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "ts_log.txt",
    globalCallingHandlers = function(...) invisible(NULL),
    .package = "base"
  )

  setup_sentinel()
  content <- readLines("ts_log.txt")
  expect_true(any(grepl("Session Log Started", content)))
})

#' Interactive Test-Suite Architect
#'
#' @description
#' Scaffolds CRAN-compliant `testthat` boilerplate for a specific function 
#' by interactively prompting the user about expected behaviors, output 
#' types, and edge cases.
#'
#' @details
#' The function automates the creation of a test file in the `tests/testthat/` 
#' directory:
#' \enumerate{
#'   \item Verifies that the `tests/testthat` directory exists (suggesting 
#'       `usethis::use_testthat()` if it does not).
#'   \item Prompts for the name of the function to be tested and creates a 
#'       corresponding `test-functionname.R` file.
#'   \item Interactively determines the expected output type (e.g., data frame, 
#'       list, numeric) to generate appropriate `expect_*` calls.
#'   \item Asks whether to include tests for output dimensions or error 
#'       handling for invalid inputs.
#'   \item Writes a structured boilerplate file containing `test_that` blocks 
#'       with TODO comments for the user to fill in mock data.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{function_name}, \code{test_file},
#' and \code{output_type}.
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   scaffold_tests()
#' }
#' @export

scaffold_tests <- function() {
  message("Initializing Test-Suite Architect...")
  
  # 1. Verify Package Infrastructure
  if (!dir.exists("tests/testthat")) {
    message("Error: 'tests/testthat' directory not found.")
    message("Hint: Run `usethis::use_testthat()` first to set up your testing infrastructure.")
    return(invisible(list(status = "error")))
  }
  
  # 2. Target Function Input
  target_func <- readline(prompt = "Enter the exact name of the function you want to test: ")
  if (trimws(target_func) == "") {
    message("Scaffolding cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  file_name <- sprintf("test-%s.R", target_func)
  file_path <- file.path("tests", "testthat", file_name)
  
  if (file.exists(file_path)) {
    overwrite <- select.list(c("Yes", "No"), title = sprintf("'%s' already exists. Overwrite?", file_name))
    if (overwrite != "Yes") {
      message("Aborted to protect existing tests.")
      return(invisible(list(status = "cancelled")))
    }
  }
  
  # 3. Interactive Test Design
  message(sprintf("\n--- Designing tests for %s() ---", target_func))
  
  out_type <- select.list(
    choices = c("data.frame / tibble", "list", "character", "numeric", "None/Unknown"),
    title = "1. What is the expected output type of this function?"
  )
  
  test_errors <- select.list(
    choices = c("Yes", "No"),
    title = "2. Should we generate an edge-case test for missing or invalid inputs (expect_error)?"
  )
  
  test_dims <- "No"
  if (out_type == "data.frame / tibble") {
    test_dims <- select.list(
      choices = c("Yes", "No"),
      title = "3. Should we include tests for output dimensions (row/column counts)?"
    )
  }
  
  # 4. Construct the Boilerplate
  test_lines <- c(
    sprintf("# Auto-generated tests for %s()", target_func),
    "library(testthat)",
    "",
    sprintf("test_that(\"%s() returns the correct data type\", {", target_func),
    sprintf("  # TODO: Define a minimal input object for testing"),
    sprintf("  # input_data <- ... "),
    sprintf("  # result <- %s(input_data)", target_func),
    ""
  )
  
  # Inject Output Type Tests
  if (out_type == "data.frame / tibble") {
    test_lines <- c(test_lines, "  expect_s3_class(result, \"data.frame\")")
  } else if (out_type == "list") {
    test_lines <- c(test_lines, "  expect_type(result, \"list\")")
  } else if (out_type == "character") {
    test_lines <- c(test_lines, "  expect_type(result, \"character\")")
  } else if (out_type == "numeric") {
    test_lines <- c(test_lines, "  expect_type(result, \"double\")")
  }
  
  # Inject Dimension Tests
  if (test_dims == "Yes") {
    test_lines <- c(
      test_lines,
      "  # expect_equal(nrow(result), expected_rows)",
      "  # expect_equal(ncol(result), expected_cols)"
    )
  }
  
  test_lines <- c(test_lines, "})", "")
  
  # Inject Error Handling Tests
  if (test_errors == "Yes") {
    test_lines <- c(
      test_lines,
      sprintf("test_that(\"%s() handles invalid inputs safely\", {", target_func),
      sprintf("  expect_error(%s(NULL)) # TODO: Update with your specific failure trigger", target_func),
      sprintf("  expect_error(%s(NA))", target_func),
      "})",
      ""
    )
  }
  
  # 5. File Output
  writeLines(test_lines, con = file_path)
  
  message(sprintf("\nSuccess! Scaffolding complete."))
  message(sprintf("-> Open '%s' to fill in your mock data and run your tests.", file_path))
  
  return(invisible(list(status = "done", function_name = target_func, test_file = file_path, output_type = out_type)))
}

#' Branch-Based Performance Benchmarker
#'
#' @description
#' Interactively selects Git branches, runs a target R script in an isolated environment 
#' for each branch, and compares the execution times to identify performance regressions 
#' or improvements across different versions of the codebase.
#'
#' @details
#' The function performs the following workflow:
#'
#' \itemize{
#'   \item Verifies that the current directory is a Git repository.
#'   \item Extracts available branches and allows the user to select specific ones or benchmark all.
#'   \item Prompts for the path to the R script to be benchmarked.
#'   \item Stashes any uncommitted changes to ensure a clean state.
#'   \item Iterates through the selected branches, checking each one out and executing 
#'       the target script within a fresh environment \code{new.env()}.
#'   \item Restores the original branch and pops the stash to return the workspace to its initial state.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' A data frame containing the benchmark results, including the branch name, 
#' execution time in seconds, and the execution status (Success/Failed).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   benchmark_branches()
#' }
#' @export

benchmark_branches <- function() {
  # 1. Verify Git environment
  if (system("git rev-parse --is-inside-work-tree", ignore.stderr = TRUE, intern = FALSE) != 0) {
    return(message("Error: Not currently inside a Git repository. Please navigate to a valid repo."))
  }
  
  # 2. Extract available branches
  branches <- system("git branch --format='%(refname:short)'", intern = TRUE)
  
  if (length(branches) < 2) {
    return(message("Error: Found fewer than 2 branches. You need at least two to compare performance."))
  }
  
  # 3. Dynamic Selection: Specific vs. All Branches
  selection_type <- select.list(
    choices = c("Select specific branches", "Benchmark ALL branches"),
    title = "How would you like to select branches for testing?"
  )
  
  if (selection_type == "") return(message("Benchmarking cancelled."))
  
  if (selection_type == "Select specific branches") {
    target_branches <- select.list(
      choices = branches,
      multiple = TRUE,
      title = "Select at least two branches to compare (Ctrl/Cmd + Click):"
    )
    if (length(target_branches) < 2) {
      return(message("Error: You must select at least two branches for a comparison."))
    }
  } else {
    target_branches <- branches
  }
  
  # 4. Dynamic Selection: Target Script
  script_file <- readline(prompt = "Enter the exact path of the R script to benchmark (e.g., 'tests/speed_test.R'): ")
  if (!file.exists(script_file)) {
    return(message(sprintf("Error: Cannot find the file '%s'.", script_file)))
  }
  
  # 5. Preserve Current State
  current_branch <- system("git branch --show-current", intern = TRUE)
  has_changes <- length(system("git status --porcelain", intern = TRUE)) > 0
  
  if (has_changes) {
    message("Detected uncommitted changes. Stashing them for safety during the benchmark...")
    system("git stash -q")
  }
  
  # 6. The Benchmarking Loop
  results <- list()
  message(sprintf("\n--- Starting Benchmark for %d Branches ---", length(target_branches)))
  
  for (branch in target_branches) {
    message(sprintf("\n-> Checking out branch: %s", branch))
    system(sprintf("git checkout %s -q", branch))
    
    message(sprintf("   Running '%s'...", script_file))
    
    start_time <- Sys.time()
    status <- "Success"
    
    # Run the script in an isolated environment so branches don't pollute each other's data
    tryCatch({
      sandbox_env <- new.env()
      source(script_file, local = sandbox_env)
    }, error = function(e) {
      status <- "Failed"
      message(sprintf("   [!] Execution error on branch '%s': %s", branch, e$message))
    })
    
    end_time <- Sys.time()
    elapsed <- as.numeric(difftime(end_time, start_time, units = "secs"))
    
    results[[branch]] <- data.frame(
      Branch = branch,
      Time_Seconds = round(elapsed, 4),
      Status = status,
      stringsAsFactors = FALSE
    )
  }
  
  # 7. Cleanup and State Restoration
  message(sprintf("\nRestoring original branch: %s", current_branch))
  system(sprintf("git checkout %s -q", current_branch))
  
  if (has_changes) {
    message("Popping stash to restore your uncommitted changes...")
    system("git stash pop -q")
  }
  
  # 8. Compile and Sort Results
  final_results <- do.call(rbind, results)
  # Sort so the fastest branch is at the top
  final_results <- final_results[order(final_results$Time_Seconds), ]
  rownames(final_results) <- NULL
  
  message("\n--- Performance Benchmark Results ---")
  print(final_results)
  
  return(invisible(final_results))
}

# Tests for benchmark_branches()

test_that("benchmark_branches returns early when not in a git repo", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  local_mocked_bindings(
    system = function(command, ...) {
      if (grepl("rev-parse", command)) return(128L)
      return(0L)
    },
    .package = "base"
  )

  expect_message(benchmark_branches(), "Not currently inside a Git repository")
})

test_that("benchmark_branches returns early with fewer than 2 branches", {
  local_mocked_bindings(
    system = function(command, ...) {
      args <- list(...)
      if (grepl("rev-parse", command)) return(0L)
      if (grepl("branch --format", command) && isTRUE(args$intern)) return("main")
      return(0L)
    },
    .package = "base"
  )

  expect_message(benchmark_branches(), "fewer than 2")
})

test_that("benchmark_branches cancels when selection_type is empty", {
  local_mocked_bindings(
    system = function(command, ...) {
      args <- list(...)
      if (grepl("rev-parse", command)) return(0L)
      if (grepl("branch --format", command) && isTRUE(args$intern)) {
        return(c("main", "dev"))
      }
      return(0L)
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(benchmark_branches(), "cancelled")
})

test_that("benchmark_branches errors on fewer than 2 branches selected", {
  select_count <- 0L
  local_mocked_bindings(
    system = function(command, ...) {
      args <- list(...)
      if (grepl("rev-parse", command)) return(0L)
      if (grepl("branch --format", command) && isTRUE(args$intern)) {
        return(c("main", "dev", "feature"))
      }
      return(0L)
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("Select specific branches")
      return("main")
    },
    .package = "devkit"
  )

  expect_message(benchmark_branches(), "at least two")
})

test_that("benchmark_branches errors when script file not found", {
  select_count <- 0L
  local_mocked_bindings(
    system = function(command, ...) {
      args <- list(...)
      if (grepl("rev-parse", command)) return(0L)
      if (grepl("branch --format", command) && isTRUE(args$intern)) {
        return(c("main", "dev"))
      }
      return(0L)
    },
    readline = function(...) "nonexistent_script.R",
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "Benchmark ALL branches",
    .package = "devkit"
  )

  expect_message(benchmark_branches(), "Cannot find")
})

test_that("benchmark_branches runs successfully on all branches", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1 + 1", "bench_script.R")

  local_mocked_bindings(
    system = function(command, ...) {
      args <- list(...)
      if (grepl("rev-parse", command)) return(0L)
      if (grepl("branch --format", command) && isTRUE(args$intern)) {
        return(c("main", "dev"))
      }
      if (grepl("branch --show-current", command) && isTRUE(args$intern)) {
        return("main")
      }
      if (grepl("status --porcelain", command) && isTRUE(args$intern)) {
        return(character(0))
      }
      if (grepl("checkout", command)) return(0L)
      return(0L)
    },
    readline = function(...) "bench_script.R",
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "Benchmark ALL branches",
    .package = "devkit"
  )

  capture.output(
    result <- benchmark_branches(),
    type = "output"
  )
  expect_s3_class(result, "data.frame")
  expect_true("Branch" %in% names(result))
  expect_true("Time_Seconds" %in% names(result))
})

test_that("benchmark_branches stashes uncommitted changes", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "bench_script.R")

  stash_called <- FALSE
  pop_called <- FALSE

  local_mocked_bindings(
    system = function(command, ...) {
      args <- list(...)
      if (grepl("rev-parse", command)) return(0L)
      if (grepl("branch --format", command) && isTRUE(args$intern)) {
        return(c("main", "dev"))
      }
      if (grepl("branch --show-current", command) && isTRUE(args$intern)) {
        return("main")
      }
      if (grepl("status --porcelain", command) && isTRUE(args$intern)) {
        return("M file.R")
      }
      if (grepl("stash -q", command)) { stash_called <<- TRUE; return(0L) }
      if (grepl("stash pop", command)) { pop_called <<- TRUE; return(0L) }
      if (grepl("checkout", command)) return(0L)
      return(0L)
    },
    readline = function(...) "bench_script.R",
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "Benchmark ALL branches",
    .package = "devkit"
  )

  capture.output(benchmark_branches(), type = "output")
  expect_true(stash_called)
  expect_true(pop_called)
})

#' Temp-Cache Janitor
#'
#' @description
#' Scans hidden R temporary directories for abandoned session data and caches, 
#' categorizes the storage waste, and interactively prompts the user for 
#' safe deletion to reclaim disk space.
#'
#' @details
#' The function performs a deep scan of the OS-level temporary directory 
#' where R stores session data:
#' \enumerate{
#'   \item Identifies all directories matching the `RtmpXXXXXX` pattern.
#'   \item Scans these directories and categorizes files into buckets:
#'       \itemize{
#'         \item \strong{Knitr/RMarkdown Caches}: `.knit.md`, `.utf8.md`, and `_cache` files.
#'         \item \strong{Downloaded Packages}: `.tar.gz`, `.zip`, and `.tgz` files.
#'         \item \strong{Raster/Image Files}: `.tif`, `.png`, `.jpg`, and `.grd` files.
#'         \item \strong{General Session Data}: All other temporary files.
#'       }
#'   \item Calculates the total size of each bucket in megabytes.
#'   \item Interactively prompts the user to select which buckets to permanently flush.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "clean", or "cancelled"), \code{dirs_found} (integer), and
#' \code{space_freed_mb} (numeric).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   sweep_temp_cache()
#' }
#' @export

sweep_temp_cache <- function() {
  message("Scanning system for hidden R temporary caches...")
  
  # 1. Identify the base temporary directory for the OS
  # tempdir() gives the current session's folder. dirname() goes one level up 
  # to the OS folder where ALL R sessions (past and present) dump their data.
  base_tmp <- dirname(tempdir())
  
  # 2. Find all R session temp directories (usually named RtmpXXXXXX)
  all_rtmp_dirs <- list.dirs(base_tmp, recursive = FALSE, full.names = TRUE)
  r_tmp_dirs <- all_rtmp_dirs[grepl("Rtmp", basename(all_rtmp_dirs))]
  
  if (length(r_tmp_dirs) == 0) {
    message("No R temporary directories found. Your system is clean.")
    return(invisible(list(status = "clean", dirs_found = 0, space_freed_mb = 0)))
  }
  
  # 3. Initialize categorization lists
  categories <- list(
    "Knitr/RMarkdown Caches" = character(),
    "Downloaded Packages/Tarballs" = character(),
    "Raster/Image Processing Files" = character(),
    "General Session Temp Data" = character()
  )
  
  # 4. Scan and categorize files
  for (dir in r_tmp_dirs) {
    files <- list.files(dir, recursive = TRUE, full.names = TRUE)
    
    # Categorize based on extensions or naming patterns
    categories[["Knitr/RMarkdown Caches"]] <- c(
      categories[["Knitr/RMarkdown Caches"]],
      files[grepl("(_cache|\\.utf8\\.md|\\.knit\\.md)$", files)]
    )
    
    categories[["Downloaded Packages/Tarballs"]] <- c(
      categories[["Downloaded Packages/Tarballs"]],
      files[grepl("(\\.tar\\.gz|\\.zip|\\.tgz)$", files)]
    )
    
    categories[["Raster/Image Processing Files"]] <- c(
      categories[["Raster/Image Processing Files"]],
      files[grepl("(\\.grd|\\.gri|\\.tif|\\.png|\\.jpg)$", files)]
    )
    
    # Everything else goes to general
    categorized_files <- c(categories[[1]], categories[[2]], categories[[3]])
    categories[["General Session Temp Data"]] <- c(
      categories[["General Session Temp Data"]],
      setdiff(files, categorized_files)
    )
  }
  
  # 5. Calculate sizes and build menu options dynamically
  menu_choices <- character()
  action_map <- list()
  
  for (cat_name in names(categories)) {
    files <- categories[[cat_name]]
    if (length(files) > 0) {
      size_mb <- sum(file.info(files)$size, na.rm = TRUE) / (1024^2)
      
      # Only flag buckets that are actually taking up measurable space
      if (size_mb > 0.1) { 
        display_name <- sprintf("%s (%.2f MB across %d files)", cat_name, size_mb, length(files))
        menu_choices <- c(menu_choices, display_name)
        action_map[[display_name]] <- files
      }
    }
  }
  
  if (length(menu_choices) == 0) {
    message("Temporary directories exist, but they are practically empty. No sweeping needed.")
    return(invisible(list(status = "clean", dirs_found = length(r_tmp_dirs), space_freed_mb = 0)))
  }
  
  menu_choices <- c(menu_choices, "Cancel")
  
  # 6. Interactive Step-by-Step Resolution
  message("\n--- Hidden Cache Analysis ---")
  to_flush <- select.list(
    choices = menu_choices,
    multiple = TRUE,
    title = "Select which cache buckets to permanently flush (Ctrl/Cmd + Click):"
  )
  
  if (length(to_flush) == 0 || "Cancel" %in% to_flush) {
    message("Janitor sweep cancelled. No files were deleted.")
    return(invisible(list(status = "cancelled", dirs_found = length(r_tmp_dirs), space_freed_mb = 0)))
  }
  
  # 7. Execute Deletion
  total_freed <- 0
  for (choice in to_flush) {
    files_to_delete <- action_map[[choice]]
    
    # Calculate freed space BEFORE deletion (files won't exist after unlink)
    freed_mb <- sum(file.info(files_to_delete)$size, na.rm = TRUE) / (1024^2)
    total_freed <- total_freed + freed_mb
    
    # Force deletion of the selected files
    unlink(files_to_delete, force = TRUE)
    
    message(sprintf("-> Flushed: %s", choice))
  }
  
  # 8. Deep Clean: Prune abandoned empty directories
  # If a previous session crashed and we just emptied its folder, delete the folder itself.
  active_tmp <- tempdir()
  for (dir in r_tmp_dirs) {
    # Never delete the active session's root temp folder, even if empty
    if (dir != active_tmp && length(list.files(dir, all.files = TRUE, no.. = TRUE)) == 0) {
      unlink(dir, recursive = TRUE, force = TRUE)
    }
  }
  
  message(sprintf("\nSweep complete. Reclaimed %.2f MB of hidden storage.", total_freed))
  return(invisible(list(status = "done", dirs_found = length(r_tmp_dirs), space_freed_mb = total_freed)))
}

# Tests for sweep_temp_cache()
# NOTE: We avoid mocking base::tempdir, base::list.dirs, etc. because
# they are used internally by testthat/R infrastructure and mocking them
# causes node stack overflow. Instead, we test at a higher level.

test_that("sweep_temp_cache completes without error on clean system", {
  # This test just ensures the function runs without crashing

  # We can't easily mock the filesystem scanning without causing recursion,
  # so we just test the function can execute
  local_mocked_bindings(
    select.list = function(choices, ...) "Cancel",
    .package = "devkit"
  )

  # The function should either find temp dirs or report clean
  expect_message(sweep_temp_cache(), "Scanning|clean|empty|Cancel|cancelled")
})

test_that("sweep_temp_cache cancels when user selects Cancel", {
  local_mocked_bindings(
    select.list = function(choices, ...) "Cancel",
    .package = "devkit"
  )

  expect_message(sweep_temp_cache(), "cancelled|clean|empty|Scanning")
})

test_that("sweep_temp_cache returns invisible NULL on cancel", {
  local_mocked_bindings(
    select.list = function(choices, ...) "Cancel",
    .package = "devkit"
  )

  result <- sweep_temp_cache()
  # In R CMD check, the environment may be perfectly clean, returning "clean"
  # In devtools::test(), the interactive session has temp files, returning "cancelled"
  expect_true(result$status %in% c("cancelled", "clean"))
})

# Internal utility functions for devkit
# None of these are exported.

# Canonical list of base R packages (ships with every R installation).
# Used by: audit_dependencies, scan_dependencies, session_snapshot
.base_pkgs <- c("base", "compiler", "datasets", "graphics", "grDevices",
                 "grid", "methods", "parallel", "splines", "stats", "stats4",
                 "tcltk", "tools", "utils")

# List data frame names in the global environment.
# Used by: dictate_dictionary
.list_global_dataframes <- function() {
  objs <- ls(envir = .GlobalEnv)
  objs[vapply(objs, function(x) is.data.frame(get(x, envir = .GlobalEnv)), logical(1))]
}

# List data frame names in a given environment.
# Used by: mask_identity
.list_dataframes_in <- function(envir) {
  objs <- ls(envir = envir)
  objs[vapply(objs, function(x) is.data.frame(get(x, envir = envir)), logical(1))]
}

# Prompt the user for a numeric value with a fallback default.
# Used by: memory_sweeper, network_diplomat
.read_numeric <- function(prompt, default, default_msg = NULL) {
  input <- readline(prompt = prompt)
  value <- suppressWarnings(as.numeric(input))
  if (is.na(value) || value <= 0) {
    if (!is.null(default_msg)) message(default_msg)
    value <- default
  }
  value
}

# Get names of non-base attached packages.
# Used by: session_snapshot, scan_dependencies
.attached_packages <- function() {
  s <- search()
  gsub("package:", "", s[grepl("^package:", s)])
}

# Tests for R/utils-internal.R (internal helper functions)

test_that(".base_pkgs contains all canonical base packages", {
  expected <- c("base", "compiler", "datasets", "graphics", "grDevices",
                "grid", "methods", "parallel", "splines", "stats", "stats4",
                "tcltk", "tools", "utils")
  expect_equal(sort(devkit:::.base_pkgs), sort(expected))
})

test_that(".base_pkgs has no duplicates", {
  expect_equal(length(devkit:::.base_pkgs),
               length(unique(devkit:::.base_pkgs)))
})

test_that(".list_global_dataframes finds data frames in .GlobalEnv", {
  assign("devkit_test_df", data.frame(a = 1), envir = .GlobalEnv)
  assign("devkit_test_vec", 1:10, envir = .GlobalEnv)
  on.exit({
    rm("devkit_test_df", envir = .GlobalEnv)
    rm("devkit_test_vec", envir = .GlobalEnv)
  }, add = TRUE)

  result <- devkit:::.list_global_dataframes()
  expect_true("devkit_test_df" %in% result)
  expect_false("devkit_test_vec" %in% result)
})

test_that(".list_global_dataframes returns empty when no data frames", {
  # Just ensure it doesn't error when the environment has no data frames
  result <- devkit:::.list_global_dataframes()
  expect_true(is.character(result))
})

test_that(".read_numeric returns valid input", {
  local_mocked_bindings(
    readline = function(...) "42",
    .package = "base"
  )

  result <- devkit:::.read_numeric("Enter: ", default = 10)
  expect_equal(result, 42)
})

test_that(".read_numeric returns default on invalid input", {
  local_mocked_bindings(
    readline = function(...) "abc",
    .package = "base"
  )

  result <- devkit:::.read_numeric("Enter: ", default = 10,
                                    default_msg = "Using default")
  expect_equal(result, 10)
})

test_that(".read_numeric returns default on zero input", {
  local_mocked_bindings(
    readline = function(...) "0",
    .package = "base"
  )

  result <- devkit:::.read_numeric("Enter: ", default = 50)
  expect_equal(result, 50)
})

test_that(".read_numeric returns default on negative input", {
  local_mocked_bindings(
    readline = function(...) "-5",
    .package = "base"
  )

  result <- devkit:::.read_numeric("Enter: ", default = 30)
  expect_equal(result, 30)
})

test_that(".attached_packages returns character vector", {
  result <- devkit:::.attached_packages()
  expect_true(is.character(result))
  # devkit should be attached during testing
  expect_true("devkit" %in% result)
})

test_that(".attached_packages excludes non-package search items", {
  result <- devkit:::.attached_packages()
  # Should not include .GlobalEnv or Autoloads
  expect_false(".GlobalEnv" %in% result)
  expect_false("Autoloads" %in% result)
})

#' Interactive Data Dictionary Dictator
#'
#' @description
#' Generates roxygen2 documentation for a data frame by interactively prompting 
#' the user for a dataset title, a general description, and individual column 
#' descriptions.
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Scans the global environment for all available data frames.
#'   \item Prompts the user to select a target data frame for documentation.
#'   \item Collects high-level metadata (title and description) for the dataset.
#'   \item Iterates through each column, displaying its type and prompting the 
#'       user for a descriptive label.
#'   \item Assembles the collected information into a CRAN-compliant roxygen2 
#'       block using the `\\describe{}` tag.
#'   \item Offers the user the choice to either print the resulting block to 
#'       the console or append it to `R/data.R`.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{dataset}, \code{dimensions},
#' \code{output}, and \code{roxygen_block} (character vector).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   dictate_dictionary()
#' }
#' @export

dictate_dictionary <- function() {
  # Find all data frames in the global environment
  df_names <- .list_global_dataframes()
  
  if (length(df_names) == 0) {
    message("No data frames found in the global environment.")
    return(invisible(list(status = "error")))
  }
  
  # Trigger interactive prompt to select the dataset
  target_df <- select.list(
    choices = df_names,
    title = "Select a dataset to document:"
  )
  
  if (target_df == "") {
    message("Documentation cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  data <- get(target_df, envir = .GlobalEnv)
  cols <- names(data)
  n_rows <- nrow(data)
  n_cols <- ncol(data)
  
  # Ask for high-level documentation
  message(sprintf("\n--- Documenting: '%s' (%d rows, %d columns) ---", target_df, n_rows, n_cols))
  dataset_title <- readline(prompt = "Enter a short title for this dataset: ")
  dataset_desc <- readline(prompt = "Enter a brief 1-2 sentence description: ")
  
  # Loop through columns and ask for inputs one by one
  message("\n--- Enter Column Descriptions ---")
  col_docs <- character(n_cols)
  
  for (i in seq_along(cols)) {
    col_name <- cols[i]
    col_type <- paste(class(data[[col_name]]), collapse = "/")
    
    prompt_text <- sprintf("Desc for '%s' (Type: %s): ", col_name, col_type)
    user_desc <- readline(prompt = prompt_text)
    
    if (trimws(user_desc) == "") user_desc <- "No description provided."
    
    # Format directly into roxygen \item syntax
    col_docs[i] <- sprintf("#'   \\item{%s}{%s. %s}", col_name, col_type, user_desc)
  }
  
  # Assemble the final roxygen block
  roxy_lines <- c(
    sprintf("#' %s", dataset_title),
    "#'",
    sprintf("#' %s", dataset_desc),
    "#'",
    sprintf("#' @format A data frame with %d rows and %d variables:", n_rows, n_cols),
    "#' \\describe{",
    col_docs,
    "#' }",
    sprintf("#' @source Automatically generated by Data Dictionary Dictator"),
    sprintf("\"%s\"\n", target_df)
  )
  
  # Final interactive choice for output
  message("\nDocumentation complete.")
  out_choice <- select.list(
    choices = c("Print to console", "Save to R/data.R"), 
    title = "Where should this documentation go?"
  )
  
  if (out_choice == "Print to console") {
    cat("\n--- Generated Roxygen2 Block ---\n")
    cat(roxy_lines, sep = "\n")
  } else if (out_choice == "Save to R/data.R") {
    if (!dir.exists("R")) dir.create("R")
    write(roxy_lines, file = "R/data.R", append = TRUE)
    message("Success: Documentation appended to R/data.R")
  }
  
  return(invisible(list(
    status = "done",
    dataset = target_df,
    dimensions = c(rows = n_rows, cols = n_cols),
    output = out_choice,
    roxygen_block = roxy_lines
  )))
}

# Tests for dictate_dictionary()

test_that("dictate_dictionary returns early when no data frames exist", {
  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  # Ensure no test data frames leak into .GlobalEnv
  expect_message(dictate_dictionary(), "No data frames|cancelled")
})

test_that("dictate_dictionary cancels when user selects empty", {
  assign("devkit_test_df_abc", data.frame(x = 1:3), envir = .GlobalEnv)
  on.exit(rm("devkit_test_df_abc", envir = .GlobalEnv), add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(dictate_dictionary(), "cancelled")
})

test_that("dictate_dictionary generates roxygen block for console output", {
  assign("devkit_test_df_dict", data.frame(x = 1:3, y = letters[1:3],
                                           stringsAsFactors = FALSE),
         envir = .GlobalEnv)
  on.exit(rm("devkit_test_df_dict", envir = .GlobalEnv), add = TRUE)

  readline_count <- 0L
  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("devkit_test_df_dict")
      return("Print to console")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("Test Dataset")
      if (readline_count == 2L) return("A test dataset")
      if (readline_count == 3L) return("Column x desc")
      if (readline_count == 4L) return("Column y desc")
      return("")
    },
    .package = "base"
  )

  out <- capture.output(dictate_dictionary())
  expect_true(any(grepl("format", out)))
  expect_true(any(grepl("describe", out)))
})

test_that("dictate_dictionary handles empty column description", {
  assign("devkit_test_df_empty", data.frame(a = 1), envir = .GlobalEnv)
  on.exit(rm("devkit_test_df_empty", envir = .GlobalEnv), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("devkit_test_df_empty", choices))) return("devkit_test_df_empty")
      return("Print to console")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("Title")
      if (readline_count == 2L) return("Desc")
      return("")
    },
    .package = "base"
  )

  out <- capture.output(dictate_dictionary())
  expect_true(any(grepl("No description provided", out)))
})

test_that("dictate_dictionary saves to R/data.R", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  assign("devkit_test_df_save", data.frame(z = 1:2), envir = .GlobalEnv)
  on.exit(rm("devkit_test_df_save", envir = .GlobalEnv), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("devkit_test_df_save", choices))) return("devkit_test_df_save")
      return("Save to R/data.R")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("Title")
      if (readline_count == 2L) return("Description")
      return("col desc")
    },
    .package = "base"
  )

  dictate_dictionary()
  expect_true(file.exists(file.path("R", "data.R")))
})

test_that("dictate_dictionary creates R directory when saving", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  assign("devkit_test_df_dir", data.frame(a = 1), envir = .GlobalEnv)
  on.exit(rm("devkit_test_df_dir", envir = .GlobalEnv), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("devkit_test_df_dir", choices))) return("devkit_test_df_dir")
      return("Save to R/data.R")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count <= 2L) return("test")
      return("desc")
    },
    .package = "base"
  )

  expect_false(dir.exists("R"))
  dictate_dictionary()
  expect_true(dir.exists("R"))
})

#' Clean-Room Simulator
#'
#' @description
#' Runs a specified R script in a background vanilla R session to verify 
#' reproducibility. If the script fails due to missing dependencies or 
#' variables, it interactively prompts the user to inject the necessary 
#' fixes directly into the file.
#'
#' @details
#' The function implements a "stress-test" for script reproducibility:
#' \enumerate{
#'   \item Prompts the user to select an R script from the current directory.
#'   \item Executes the script using `system2("Rscript", args = c("--vanilla", ...))`, 
#'       ensuring no workspace variables or attached packages from the current 
#'       session interfere.
#'   \item If the script crashes, it captures the `stderr` output and presents 
#'       a crash report to the user.
#'   \item Interactively offers to fix the crash by:
#'       \itemize{
#'         \item Injecting a missing `library()` call at the top of the file.
#'         \item Injecting a custom code snippet (e.g., a missing variable definition).
#'       }
#'   \item Automatically updates the file and re-runs the simulation until 
#'       the script executes successfully or the user aborts.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done" or
#' "cancelled"), \code{script}, \code{success} (logical), and \code{attempts} (integer).
#'
#' @importFrom utils select.list tail
#' @examples
#' if (interactive()) {
#'   simulate_clean_room()
#' }
#' @export

simulate_clean_room <- function() {
  message("Initializing Clean-Room Environment...")
  
  # 1. Select the target script
  scripts <- list.files(pattern = "\\.R$", ignore.case = TRUE)
  if (length(scripts) == 0) {
    message("Error: No R scripts found in the current directory.")
    return(invisible(list(status = "error")))
  }
  
  target_script <- select.list(scripts, title = "Select a script to stress-test:")
  if (target_script == "") {
    message("Simulation cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  testing <- TRUE
  attempt <- 1
  
  # 2. The Testing Loop
  while (testing) {
    message(sprintf("\n--- Attempt %d: Running '%s' in a vanilla session ---", attempt, target_script))
    
    # Run in a completely isolated background process
    # system2 captures both standard output and standard error
    res <- suppressWarnings(system2(
      "Rscript", 
      args = c("--vanilla", target_script), 
      stdout = TRUE, 
      stderr = TRUE
    ))
    
    # Check the exit status attribute (NULL means success)
    status <- attr(res, "status")
    
    if (is.null(status) || status == 0) {
      message("\nSUCCESS! The script ran perfectly in an isolated environment.")
      message("It is 100% reproducible.")
      testing <- FALSE
      
    } else {
      # 3. Parse the Error
      message("\n[!] SIMULATION FAILED. The script broke in a clean environment.")
      
      # Extract the actual error message from the stderr output
      error_lines <- res[grepl("Error", res, ignore.case = TRUE)]
      if (length(error_lines) > 0) {
        error_msg <- paste(error_lines, collapse = "\n")
        message("\n--- Crash Report ---")
        message(error_msg)
        message("--------------------\n")
      } else {
        error_msg <- paste(tail(res, 3), collapse = "\n") # Fallback to last few lines
        message("Crash output:\n", error_msg)
      }
      
      # 4. Interactive Diagnosis & Snippet Generation
      fix_type <- select.list(
        choices = c(
          "Inject missing library() call", 
          "Inject custom code snippet (e.g., missing variable)", 
          "Abort Simulation"
        ),
        title = "How would you like to fix this?"
      )
      
      if (fix_type == "Abort Simulation" || fix_type == "") {
        message("Simulation aborted. File left in its current state.")
        return(invisible(list(status = "cancelled", script = target_script, success = FALSE, attempts = attempt)))
      }
      
      snippet_to_inject <- ""
      
      if (fix_type == "Inject missing library() call") {
        # Helper to extract a guessed function name to prompt the user
        pkg_guess <- readline(prompt = "Enter the name of the missing package: ")
        if (trimws(pkg_guess) != "") {
          snippet_to_inject <- sprintf("library(%s)", trimws(pkg_guess))
        }
      } else {
        message("Enter the exact R code to inject at the top of the file.")
        snippet_to_inject <- readline(prompt = "Snippet: ")
      }
      
      # 5. File Injection
      if (snippet_to_inject != "") {
        # Read the current file
        current_code <- readLines(target_script, warn = FALSE)
        
        # Prepend the new snippet
        updated_code <- c(
          sprintf("%s # [Auto-injected by Clean-Room Simulator]", snippet_to_inject),
          current_code
        )
        
        # Write it back to the file
        writeLines(updated_code, con = target_script)
        message(sprintf("\nSuccess: Injected '%s' at line 1.", snippet_to_inject))
        
        # Ask to re-run
        re_run <- select.list(c("Yes", "No"), title = "Run the simulation again with the fix?")
        if (re_run != "Yes") {
          testing <- FALSE
          message("Simulation paused. You can resume testing later.")
        } else {
          attempt <- attempt + 1
        }
        
      } else {
        message("No valid snippet provided. Simulation aborted.")
        testing <- FALSE
      }
    }
  }
  
  return(invisible(list(status = "done", script = target_script, success = TRUE, attempts = attempt)))
}

# Tests for simulate_clean_room()

test_that("simulate_clean_room returns early when no scripts found", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  expect_message(simulate_clean_room(), "No R scripts found")
})

test_that("simulate_clean_room cancels when user selects empty", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "test_script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(simulate_clean_room(), "cancelled")
})

test_that("simulate_clean_room reports success for clean script", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1 + 1", "clean_script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("clean_script", choices))) return("clean_script.R")
      return("")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    system2 = function(command, args, ...) {
      res <- character(0)
      attr(res, "status") <- 0L
      res
    },
    .package = "base"
  )

  result <- simulate_clean_room()
  expect_equal(result$status, "done")
})

test_that("simulate_clean_room reports failure and user aborts", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("library(nonexistentpkg123)", "failing_script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("failing", choices))) return("failing_script.R")
      if (any(grepl("Abort", choices))) return("Abort Simulation")
      return("")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    system2 = function(command, args, ...) {
      res <- c("Error in library(nonexistentpkg123) : there is no package called 'nonexistentpkg123'")
      attr(res, "status") <- 1L
      res
    },
    .package = "base"
  )

  result <- simulate_clean_room()
  expect_equal(result$status, "cancelled")
})

test_that("simulate_clean_room injects library call and reruns", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("ggplot2::ggplot()", "needs_lib.R")

  system2_count <- 0L
  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (any(grepl("needs_lib", choices))) return("needs_lib.R")
      if (any(grepl("Inject missing library", choices))) return("Inject missing library() call")
      if (any(grepl("Yes", choices))) return("Yes")
      return("")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "ggplot2",
    system2 = function(command, args, ...) {
      system2_count <<- system2_count + 1L
      if (system2_count == 1L) {
        res <- c("Error: could not find function 'ggplot'")
        attr(res, "status") <- 1L
        return(res)
      }
      # Second run succeeds
      res <- character(0)
      attr(res, "status") <- 0L
      res
    },
    .package = "base"
  )

  result <- simulate_clean_room()
  expect_equal(result$status, "done")
  content <- readLines("needs_lib.R")
  expect_true(any(grepl("library(ggplot2)", content, fixed = TRUE)))
})

test_that("simulate_clean_room injects custom snippet", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("print(my_var)", "needs_var.R")

  system2_count <- 0L
  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (any(grepl("needs_var", choices))) return("needs_var.R")
      if (any(grepl("custom", choices, ignore.case = TRUE))) {
        return("Inject custom code snippet (e.g., missing variable)")
      }
      if (any(grepl("Yes", choices))) return("No")
      return("")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) 'my_var <- "hello"',
    system2 = function(command, args, ...) {
      system2_count <<- system2_count + 1L
      res <- c("Error: object 'my_var' not found")
      attr(res, "status") <- 1L
      res
    },
    .package = "base"
  )

  simulate_clean_room()
  content <- readLines("needs_var.R")
  expect_true(any(grepl("my_var", content)))
  expect_true(any(grepl("Auto-injected", content)))
})

test_that("simulate_clean_room handles no valid snippet", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("stop('error')", "err_script.R")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (any(grepl("err_script", choices))) return("err_script.R")
      if (any(grepl("Inject missing library", choices))) return("Inject missing library() call")
      return("")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "",
    system2 = function(command, args, ...) {
      res <- c("Error: error")
      attr(res, "status") <- 1L
      res
    },
    .package = "base"
  )

  expect_message(simulate_clean_room(), "No valid snippet")
})

test_that("simulate_clean_room handles error lines without Error keyword", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "fallback_script.R")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (any(grepl("fallback", choices))) return("fallback_script.R")
      return("Abort Simulation")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    system2 = function(command, args, ...) {
      res <- c("some output", "without error keyword", "something went wrong")
      attr(res, "status") <- 1L
      res
    },
    .package = "base"
  )

  result <- simulate_clean_room()
  expect_equal(result$status, "cancelled")
})

#' Interactive Pre-Flight Dispatcher
#'
#' @description
#' Generates a custom Git pre-commit hook by interactively prompting the user 
#' to select which safety checks should be enforced before allowing a commit.
#'
#' @details
#' The function automates the creation of a `.git/hooks/pre-commit` shell script 
#' that can enforce the following checks:
#' \enumerate{
#'   \item \strong{Code Styling}: Automatically runs `styler::style_pkg()` to 
#'       standardize formatting and stages the modified files.
#'   \item \strong{Documentation}: Automatically runs `devtools::document()` 
#'       to ensure the `NAMESPACE` and help files are up to date.
#'   \item \strong{Testing}: Executes `testthat::test_local()` and aborts the 
#'       commit if any tests fail.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{hook_path}, and \code{checks} (named logical list).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   setup_preflight()
#' }
#' @export

setup_preflight <- function() {
  message("Initializing Pre-Flight Dispatcher...")
  
  # 1. Verify Environment
  if (!dir.exists(".git")) {
    message("Error: Not a Git repository. Please run this at the root of your project.")
    return(invisible(list(status = "error")))
  }
  if (!dir.exists(".git/hooks")) {
    dir.create(".git/hooks", showWarnings = FALSE)
  }
  
  hook_path <- ".git/hooks/pre-commit"
  
  # 2. Step-by-Step Configuration
  message("\n--- Configuring Safety Checks ---")
  
  # Prompt 1: Documentation
  run_docs <- select.list(
    choices = c("Yes", "No"),
    title = "1. Automatically run devtools::document() to update help files before every commit?"
  )
  
  # Prompt 2: Testing
  run_tests <- select.list(
    choices = c("Yes", "No"),
    title = "2. Require all testthat tests to pass before allowing a commit?"
  )
  
  # Prompt 3: Styling
  run_style <- select.list(
    choices = c("Yes", "No"),
    title = "3. Automatically standardize code format using styler::style_pkg()?"
  )
  
  if (run_docs == "No" && run_tests == "No" && run_style == "No") {
    message("All checks skipped. No pre-commit hook generated.")
    return(invisible(list(status = "cancelled")))
  }
  
  # 3. Construct the Bash Hook Script
  hook_lines <- c(
    "#!/bin/sh",
    "# Auto-generated pre-commit hook by Pre-Flight Dispatcher",
    "",
    "echo '\\n--- Running R Pre-Flight Checks ---'"
  )
  
  if (run_style == "Yes") {
    hook_lines <- c(
      hook_lines,
      "echo '-> Styling code...'",
      "Rscript -e \"if (requireNamespace('styler', quietly = TRUE)) styler::style_pkg()\"",
      "# Automatically stage any files modified by styler",
      "git add \\*.R"
    )
  }
  
  if (run_docs == "Yes") {
    hook_lines <- c(
      hook_lines,
      "echo '-> Updating documentation...'",
      "Rscript -e \"if (requireNamespace('devtools', quietly = TRUE)) devtools::document()\"",
      "# Automatically stage the NAMESPACE and man/ directory in case they were updated",
      "git add NAMESPACE man/\\*"
    )
  }
  
  if (run_tests == "Yes") {
    hook_lines <- c(
      hook_lines,
      "echo '-> Running tests...'",
      "# Using StopReporter so the script exits with an error code if ANY test fails",
      "Rscript -e \"if (requireNamespace('testthat', quietly = TRUE)) testthat::test_local(reporter = testthat::StopReporter)\"",
      "if [ $? -ne 0 ]; then",
      "  echo '\\n[!] Tests failed. Commit aborted. Fix the errors and try again.'",
      "  exit 1",
      "fi"
    )
  }
  
  hook_lines <- c(
    hook_lines,
    "echo '--- All Checks Passed. Committing... ---'",
    "exit 0",
    ""
  )
  
  # 4. File Writing & Protection
  if (file.exists(hook_path)) {
    overwrite <- select.list(
      choices = c("Yes", "No"),
      title = "A pre-commit hook already exists. Overwrite it?"
    )
    if (overwrite == "No") {
      message("Aborted. Existing hook was protected.")
      return(invisible(list(status = "cancelled")))
    }
  }
  
  writeLines(hook_lines, con = hook_path)
  
  # 5. Make the hook executable
  # This is strictly required for Git to be allowed to run the script
  Sys.chmod(hook_path, mode = "0755")
  
  message("\nSuccess! Your Pre-Flight Dispatcher is armed and active.")
  message("These checks will now run invisibly in the background every time you type `git commit`.")
  
  return(invisible(list(
    status = "done",
    hook_path = hook_path,
    checks = list(documentation = (run_docs == "Yes"), tests = (run_tests == "Yes"), styling = (run_style == "Yes"))
  )))
}

# Tests for setup_preflight()

test_that("setup_preflight returns early when not in git repo", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  expect_message(setup_preflight(), "Not a Git repository")
})

test_that("setup_preflight creates .git/hooks if missing", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(".git", showWarnings = FALSE)

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )
  local_mocked_bindings(
    Sys.chmod = function(...) invisible(NULL),
    .package = "base"
  )

  setup_preflight()
  expect_true(dir.exists(".git/hooks"))
})

test_that("setup_preflight skips when all checks are No", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path(".git", "hooks"), recursive = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  expect_message(setup_preflight(), "All checks skipped")
})

test_that("setup_preflight creates hook with docs check only", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path(".git", "hooks"), recursive = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("Yes")   # docs
      if (select_count == 2L) return("No")    # tests
      if (select_count == 3L) return("No")    # style
      return("Yes")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    Sys.chmod = function(...) invisible(NULL),
    .package = "base"
  )

  setup_preflight()
  hook <- readLines(".git/hooks/pre-commit")
  expect_true(any(grepl("devtools::document", hook)))
  expect_false(any(grepl("testthat", hook)))
  expect_false(any(grepl("styler", hook)))
})

test_that("setup_preflight creates hook with tests check only", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path(".git", "hooks"), recursive = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("No")    # docs
      if (select_count == 2L) return("Yes")   # tests
      if (select_count == 3L) return("No")    # style
      return("Yes")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    Sys.chmod = function(...) invisible(NULL),
    .package = "base"
  )

  setup_preflight()
  hook <- readLines(".git/hooks/pre-commit")
  expect_true(any(grepl("testthat", hook)))
  expect_true(any(grepl("exit 1", hook)))
})

test_that("setup_preflight creates hook with style check only", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path(".git", "hooks"), recursive = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("No")    # docs
      if (select_count == 2L) return("No")    # tests
      if (select_count == 3L) return("Yes")   # style
      return("Yes")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    Sys.chmod = function(...) invisible(NULL),
    .package = "base"
  )

  setup_preflight()
  hook <- readLines(".git/hooks/pre-commit")
  expect_true(any(grepl("styler", hook)))
})

test_that("setup_preflight creates hook with all checks", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path(".git", "hooks"), recursive = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) "Yes",
    .package = "devkit"
  )
  local_mocked_bindings(
    Sys.chmod = function(...) invisible(NULL),
    .package = "base"
  )

  setup_preflight()
  hook <- readLines(".git/hooks/pre-commit")
  expect_true(any(grepl("#!/bin/sh", hook)))
  expect_true(any(grepl("styler", hook)))
  expect_true(any(grepl("devtools::document", hook)))
  expect_true(any(grepl("testthat", hook)))
})

test_that("setup_preflight aborts when existing hook and user says No", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path(".git", "hooks"), recursive = TRUE)
  writeLines("#!/bin/sh\nexit 0", ".git/hooks/pre-commit")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count <= 3L) return("Yes")
      return("No")  # overwrite = No
    },
    .package = "devkit"
  )

  expect_message(setup_preflight(), "Aborted|protected")
})

test_that("setup_preflight makes hook executable", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path(".git", "hooks"), recursive = TRUE)

  chmod_called <- FALSE
  local_mocked_bindings(
    select.list = function(choices, ...) "Yes",
    .package = "devkit"
  )
  local_mocked_bindings(
    Sys.chmod = function(path, mode, ...) {
      chmod_called <<- TRUE
      expect_equal(mode, "0755")
    },
    .package = "base"
  )

  setup_preflight()
  expect_true(chmod_called)
})

#' Export Session Snapshot
#'
#' @description
#' Generates a reproducible installation script for all currently attached 
#' external R packages, allowing the user to recreate the exact environment 
#' in a different session or on another machine.
#'
#' @details
#' The function provides two modes of environment capture:
#' \enumerate{
#'   \item \strong{Flexible Installation}: Generates a script that checks for 
#'       missing packages and installs the latest available versions.
#'   \item \strong{Strict Version Locking}: Uses `devtools::install_version()` 
#'       to lock the environment to the exact versions currently installed 
#'       on the system, ensuring maximum reproducibility.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done" or
#' "clean"), \code{packages} (character vector), \code{file} (output path),
#' and \code{version_locked} (logical).
#'
#' @importFrom utils packageVersion
#' @examples
#' if (interactive()) {
#'   export_snapshot()
#' }
#' @export

export_snapshot <- function() {
  # Dynamically ask for output preferences
  filename <- readline(prompt = "Enter filename for the snapshot (e.g., requirements.R): ")
  if (filename == "") filename <- "requirements.R"
  
  strict_version <- readline(prompt = "Lock exact package versions using devtools? (y/n): ")
  include_versions <- tolower(trimws(strict_version)) == "y"

  # Extract non-base attached packages
  pkg_names <- .attached_packages()
  
  base_pkgs <- .base_pkgs
  
  target_pkgs <- setdiff(pkg_names, base_pkgs)
  
  if (length(target_pkgs) == 0) {
    message("No external packages currently attached. Nothing to export.")
    return(invisible(list(status = "clean")))
  }

  script_lines <- c("# Auto-generated R Environment Snapshot", "")

  if (include_versions) {
    script_lines <- c(script_lines, "if (!requireNamespace('devtools', quietly = TRUE)) install.packages('devtools')")
    for (pkg in target_pkgs) {
      ver <- as.character(packageVersion(pkg))
      script_lines <- c(script_lines, sprintf("devtools::install_version('%s', version = '%s')", pkg, ver))
    }
  } else {
    pkg_vector <- paste(sprintf('"%s"', target_pkgs), collapse = ", ")
    script_lines <- c(script_lines, 
                      sprintf("req_packages <- c(%s)", pkg_vector),
                      "missing <- req_packages[!vapply(req_packages, requireNamespace, quietly = TRUE, FUN.VALUE = logical(1))]",
                      "if(length(missing)) install.packages(missing)"
    )
  }

  writeLines(script_lines, con = filename)
  message(sprintf("Snapshot of %d packages saved to '%s'", length(target_pkgs), filename))
  return(invisible(list(status = "done", packages = target_pkgs, file = filename, version_locked = include_versions)))
}

# Tests for export_snapshot() (session_snapshot.R)
# Note: export_snapshot is already tested in test-export-snapshot.R
# This file tests the same function from a different angle

test_that("export_snapshot generates valid R script", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    packageVersion = function(pkg) "3.2.0",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("snapshot_test.R")
      return("n")
    },
    search = function() c(".GlobalEnv", "package:testthat", "package:base"),
    .package = "base"
  )

  export_snapshot()
  content <- readLines("snapshot_test.R")
  expect_true(any(grepl("install.packages", content)))
  # Should contain the package name
  expect_true(any(grepl("testthat", content)))
})

test_that("export_snapshot with strict version includes devtools", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    packageVersion = function(pkg) "3.2.0",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("strict_test.R")
      return("y")
    },
    search = function() c(".GlobalEnv", "package:testthat", "package:base"),
    .package = "base"
  )

  export_snapshot()
  content <- readLines("strict_test.R")
  expect_true(any(grepl("install_version", content)))
  expect_true(any(grepl("3.2.0", content)))
})

test_that("export_snapshot filters base packages correctly", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    packageVersion = function(pkg) "1.0.0",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("filter_test.R")
      return("n")
    },
    search = function() c(".GlobalEnv", "package:base", "package:methods",
                          "package:stats", "package:utils",
                          "package:ggplot2"),
    .package = "base"
  )

  export_snapshot()
  content <- paste(readLines("filter_test.R"), collapse = "\n")
  expect_true(grepl("ggplot2", content))
  expect_false(grepl("\"base\"", content))
  expect_false(grepl("\"methods\"", content))
})

# Tests for export_snapshot()

test_that("export_snapshot returns early when no external packages", {
  local_mocked_bindings(
    readline = function(...) "test_snapshot.R",
    search = function() c(".GlobalEnv", "package:base", "package:stats",
                          "package:utils"),
    .package = "base"
  )

  expect_message(export_snapshot(), "No external packages")
})

test_that("export_snapshot generates flexible install script", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("flex_snapshot.R")
      return("n")
    },
    search = function() c(".GlobalEnv", "package:testthat", "package:base",
                          "package:stats"),
    .package = "base"
  )
  local_mocked_bindings(
    packageVersion = function(pkg) "1.0.0",
    .package = "devkit"
  )

  export_snapshot()
  expect_true(file.exists("flex_snapshot.R"))
  content <- readLines("flex_snapshot.R")
  expect_true(any(grepl("install.packages", content)))
})

test_that("export_snapshot generates strict version script", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("strict_snapshot.R")
      return("y")
    },
    search = function() c(".GlobalEnv", "package:ggplot2", "package:base"),
    .package = "base"
  )
  local_mocked_bindings(
    packageVersion = function(pkg) "3.4.0",
    .package = "devkit"
  )

  export_snapshot()
  expect_true(file.exists("strict_snapshot.R"))
  content <- readLines("strict_snapshot.R")
  expect_true(any(grepl("install_version", content)))
})

test_that("export_snapshot uses default filename when empty", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("")
      return("n")
    },
    search = function() c(".GlobalEnv", "package:dplyr", "package:base"),
    .package = "base"
  )
  local_mocked_bindings(
    packageVersion = function(pkg) "1.0.0",
    .package = "devkit"
  )

  export_snapshot()
  expect_true(file.exists("requirements.R"))
})

test_that("export_snapshot writes Snapshot header", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("out.R")
      return("n")
    },
    search = function() c(".GlobalEnv", "package:jsonlite", "package:base"),
    .package = "base"
  )
  local_mocked_bindings(
    packageVersion = function(pkg) "1.8.0",
    .package = "devkit"
  )

  export_snapshot()
  content <- readLines("out.R")
  expect_true(any(grepl("Snapshot", content)))
})

#' Interactive Lifecycle Manager
#'
#' @description
#' Scaffolds CRAN-compliant deprecation wrappers and interactively 
#' refactors old function calls across tests and vignettes to ensure 
#' a smooth transition to new API versions.
#'
#' @details
#' The function manages the deprecation lifecycle through the following steps:
#' \enumerate{
#'   \item Prompts the user for the name of the function to be deprecated and its replacement.
#'   \item Automatically generates a deprecated wrapper function that calls `.Deprecated()` 
#'       and then forwards arguments to the new function.
#'   \item Appends this wrapper to `R/deprecated.R`, ensuring the package remains 
#'       backward compatible while warning users.
#'   \item Optionally scans the `tests/` and `vignettes/` directories for occurrences 
#'       of the old function and interactively replaces them with the new one.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{old_function}, \code{new_function},
#' \code{wrapper_file}, and \code{replacements} (integer count).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   manage_deprecation()
#' }
#' @export

manage_deprecation <- function() {
  message("Initializing Lifecycle Manager...")
  
  # 1. Verify Package Environment
  if (!dir.exists("R")) {
    message("Error: 'R/' directory not found. Please run from the package root.")
    return(invisible(list(status = "error")))
  }
  
  # 2. Gather Function Names
  old_func <- readline(prompt = "Enter the name of the function to DEPRECATE: ")
  if (trimws(old_func) == "") {
    message("Aborted.")
    return(invisible(list(status = "cancelled")))
  }
  
  new_func <- readline(prompt = sprintf("Enter the name of the REPLACEMENT function for '%s': ", old_func))
  if (trimws(new_func) == "") {
    message("Aborted.")
    return(invisible(list(status = "cancelled")))
  }
  
  # 3. Scaffold the Deprecated Wrapper
  dep_file <- "R/deprecated.R"
  
  wrapper_code <- c(
    "",
    sprintf("#' @title Deprecated functions"),
    sprintf("#' @description `%s()` was deprecated in favor of `%s()`.", old_func, new_func),
    sprintf("#' @name %s-deprecated", old_func),
    "#' @keywords internal",
    "#' @export",
    sprintf("%s <- function(...) {", old_func),
    sprintf("  .Deprecated(\"%s\")", new_func),
    sprintf("  %s(...)", new_func),
    "}"
  )
  
  # Append to or create R/deprecated.R
  if (!file.exists(dep_file)) {
    writeLines(c("#' @name package-deprecated", "#' @aliases NULL", ""), dep_file)
    message(sprintf("-> Created %s", dep_file))
  }
  
  cat(paste(wrapper_code, collapse = "\n"), "\n", file = dep_file, append = TRUE)
  message(sprintf("-> Scaffolded deprecated wrapper for `%s()` in %s", old_func, dep_file))
  
  replacements_made <- 0L
  
  # 4. Internal Refactoring (Tests & Vignettes)
  refactor <- select.list(
    choices = c("Yes", "No"),
    title = sprintf("\nScan tests/ and vignettes/ to replace `%s()` with `%s()`?", old_func, new_func)
  )
  
  if (refactor == "Yes") {
    target_dirs <- c("tests", "vignettes")
    files_to_scan <- character()
    
    for (d in target_dirs) {
      if (dir.exists(d)) {
        files_to_scan <- c(files_to_scan, list.files(d, pattern = "\\.(R|Rmd)$", full.names = TRUE, recursive = TRUE, ignore.case = TRUE))
      }
    }
    
    if (length(files_to_scan) == 0) {
      message("No tests or vignettes found to scan.")
      return(invisible(list(status = "done", old_function = old_func, new_function = new_func, wrapper_file = dep_file, replacements = 0L)))
    }
    
    message("\n--- Scanning for Internal Usage ---")
    regex_pattern <- paste0("\\b", old_func, "\\(")
    replacements_made <- 0
    
    for (f in files_to_scan) {
      lines <- readLines(f, warn = FALSE)
      matches <- grep(regex_pattern, lines)
      
      if (length(matches) > 0) {
        message(sprintf("\nFound %d instance(s) in: %s", length(matches), f))
        
        for (idx in matches) {
          # Show the user the exact line of code
          message(sprintf("Line %d: %s", idx, trimws(lines[idx])))
          action <- select.list(
            c("Replace", "Skip"), 
            title = "Action:"
          )
          
          if (action == "Replace") {
            # Safely replace only the exact function call
            lines[idx] <- gsub(regex_pattern, paste0(new_func, "("), lines[idx])
            replacements_made <- replacements_made + 1
          }
        }
        # Write the updated lines back to the file
        writeLines(lines, f)
      }
    }
    
    message(sprintf("\nRefactoring complete. Made %d replacements.", replacements_made))
  }
  
  message("\nLifecycle update finished. Remember to remove the original definition of the deprecated function from your R/ scripts!")
  return(invisible(list(
    status = "done",
    old_function = old_func,
    new_function = new_func,
    wrapper_file = dep_file,
    replacements = replacements_made
  )))
}

# Tests for manage_deprecation()

test_that("manage_deprecation returns early when R dir missing", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  expect_message(manage_deprecation(), "directory not found")
})

test_that("manage_deprecation aborts on empty old function name", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create("R")

  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  expect_message(manage_deprecation(), "Aborted")
})

test_that("manage_deprecation aborts on empty new function name", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create("R")

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("old_func")
      return("")
    },
    .package = "base"
  )

  expect_message(manage_deprecation(), "Aborted")
})

test_that("manage_deprecation creates R/deprecated.R when missing", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create("R")

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("old_func")
      return("new_func")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  manage_deprecation()
  expect_true(file.exists("R/deprecated.R"))
  content <- readLines("R/deprecated.R")
  expect_true(any(grepl("old_func", content)))
  expect_true(any(grepl(".Deprecated", content)))
  expect_true(any(grepl("new_func", content)))
})

test_that("manage_deprecation appends to existing deprecated.R", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create("R")
  writeLines("# existing content", "R/deprecated.R")

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("another_old")
      return("another_new")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) "No",
    .package = "devkit"
  )

  manage_deprecation()
  content <- readLines("R/deprecated.R")
  expect_true(any(grepl("existing content", content)))
  expect_true(any(grepl("another_old", content)))
})

test_that("manage_deprecation scans and replaces in test files", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create("R")
  dir.create(file.path("tests", "testthat"), recursive = TRUE)
  writeLines("result <- old_func(x)", file.path("tests", "testthat", "test.R"))

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("old_func")
      return("new_func")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Yes", choices))) return("Yes")
      if (any(grepl("Replace", choices))) return("Replace")
      return("No")
    },
    .package = "devkit"
  )

  manage_deprecation()
  test_content <- readLines(file.path("tests", "testthat", "test.R"))
  expect_true(any(grepl("new_func", test_content)))
})

test_that("manage_deprecation handles no test files to scan", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create("R")

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("old_func")
      return("new_func")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("scan", choices, ignore.case = TRUE))) return("Yes")
      if (any(grepl("Yes", choices))) return("Yes")
      return("No")
    },
    .package = "devkit"
  )

  expect_message(manage_deprecation(), "No tests|No vignettes|not found")
})

test_that("manage_deprecation skips lines when user says Skip", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create("R")
  dir.create(file.path("tests", "testthat"), recursive = TRUE)
  writeLines("result <- old_func(x)", file.path("tests", "testthat", "test.R"))

  readline_count <- 0L
  local_mocked_bindings(
    readline = function(prompt = "", ...) {
      readline_count <<- readline_count + 1L
      if (readline_count == 1L) return("old_func")
      return("new_func")
    },
    .package = "base"
  )
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("scan", choices, ignore.case = TRUE))) return("Yes")
      if (any(grepl("Yes", choices))) return("Yes")
      if (any(grepl("Replace", choices))) return("Skip")
      return("No")
    },
    .package = "devkit"
  )

  manage_deprecation()
  test_content <- readLines(file.path("tests", "testthat", "test.R"))
  expect_true(any(grepl("old_func", test_content)))
})

#' Interactive CPU Architect
#'
#' @description
#' Scans local hardware to determine available CPU cores and generates 
#' a robust parallel processing scaffold for high-volume data tasks, 
#' automating the setup of a local cluster.
#'
#' @details
#' The function guides the user through the creation of a parallel processing 
#' pipeline:
#' \enumerate{
#'   \item Detects the total number of available CPU cores using `parallel::detectCores()`.
#'   \item Prompts the user to select the number of cores to dedicate to the task, 
#'       recommending leaving at least one core free for OS stability.
#'   \item Collects the names of the target data object and the processing function.
#'   \item Generates a complete R code snippet that:
#'       \itemize{
#'         \item Initializes a cluster using `makeCluster()`.
#'         \item Exports the required function to the worker nodes via `clusterExport()`.
#'         \item Executes the computation using `parLapply()`.
#'         \item Safely shuts down the cluster using `stopCluster()`.
#'       }
#'   \item Offers to print the snippet to the console or save it directly to `parallel_scaffold.R`.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", or "error"), \code{cores} (integer), \code{data_object},
#' \code{function_name}, and \code{saved_to} (file path or \code{NULL}).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   scaffold_parallel()
#' }
#' @export

scaffold_parallel <- function() {
  message("Initializing CPU Architect...")
  
  # 1. Hardware Scan
  if (!requireNamespace("parallel", quietly = TRUE)) {
    message("Error: The built-in 'parallel' package is missing.")
    return(invisible(list(status = "error")))
  }
  
  total_cores <- parallel::detectCores()
  
  if (is.na(total_cores)) {
    message("Error: Could not determine hardware specifications.")
    return(invisible(list(status = "error")))
  }
  
  message(sprintf("Hardware Scan Complete: %d CPU cores detected.", total_cores))
  
  # 2. Interactive Core Selection
  # It is best practice to leave at least 1 core free so the OS doesn't freeze
  safe_max <- max(1, total_cores - 1)
  core_options <- as.character(seq(1, total_cores))
  
  core_prompt <- sprintf(
    "How many cores do you want to dedicate to this task? (Max recommended: %d)", 
    safe_max
  )
  
  chosen_cores <- select.list(
    choices = core_options,
    title = core_prompt
  )
  
  if (chosen_cores == "") {
    message("Scaffolding cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  # 3. Gather Target Data and Function
  data_obj <- readline(prompt = "Enter the name of the list or vector you are iterating over (e.g., 'massive_dataset'): ")
  if (trimws(data_obj) == "") data_obj <- "my_data"
  
  func_name <- readline(prompt = "Enter the name of the function to apply to each element (e.g., 'process_file'): ")
  if (trimws(func_name) == "") func_name <- "my_heavy_function"
  
  # 4. Generate the Scaffold
  scaffold_lines <- c(
    "# --- Auto-Generated Parallel Cluster ---",
    "library(parallel)",
    "",
    sprintf("num_cores <- %s", chosen_cores),
    "message(sprintf('Initializing local cluster with %d cores...', num_cores))",
    "",
    "# 1. Create the background worker cluster",
    "cl <- makeCluster(num_cores)",
    "",
    "# 2. Export necessary functions and variables to the workers",
    "# Note: If your function relies on external packages, use clusterEvalQ(cl, library(pkg_name))",
    sprintf("clusterExport(cl, varlist = c('%s'))", func_name),
    "",
    "# 3. Execute the heavy processing",
    "message('Running parallel computation. Please wait...')",
    "start_time <- Sys.time()",
    "",
    sprintf("results <- parLapply(cl, X = %s, fun = %s)", data_obj, func_name),
    "",
    "end_time <- Sys.time()",
    "",
    "# 4. Safely shut down the cluster to free up the CPU",
    "stopCluster(cl)",
    "",
    "elapsed <- as.numeric(difftime(end_time, start_time, units = 'secs'))",
    "message(sprintf('Computation finished successfully in %.2f seconds.', elapsed))",
    "# ---------------------------------------"
  )
  
  # 5. Output 
  message("\n--- Generated Parallel Snippet ---")
  cat(scaffold_lines, sep = "\n")
  
  out_choice <- select.list(
    choices = c("Done", "Save to parallel_scaffold.R"), 
    title = "\nAction: Copy the block above, or save it directly to a file?"
  )
  
  if (out_choice == "Save to parallel_scaffold.R") {
    writeLines(scaffold_lines, con = "parallel_scaffold.R")
    message("Success: Snippet saved to 'parallel_scaffold.R' in your working directory.")
  }
  
  return(invisible(list(
    status = "done",
    cores = as.integer(chosen_cores),
    data_object = data_obj,
    function_name = func_name,
    saved_to = if (out_choice == "Save to parallel_scaffold.R") "parallel_scaffold.R" else NULL
  )))
}

# Tests for scaffold_parallel()

test_that("scaffold_parallel returns early when parallel not available", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) FALSE,
    .package = "base"
  )

  expect_message(scaffold_parallel(), "parallel.*missing")
})

test_that("scaffold_parallel returns early when cores undetectable", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) TRUE,
    .package = "base"
  )
  local_mocked_bindings(
    detectCores = function() NA,
    .package = "parallel"
  )

  expect_message(scaffold_parallel(), "Could not determine")
})

test_that("scaffold_parallel cancels when user selects empty", {
  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) TRUE,
    .package = "base"
  )
  local_mocked_bindings(
    detectCores = function() 4L,
    .package = "parallel"
  )

  expect_message(scaffold_parallel(), "cancelled")
})

test_that("scaffold_parallel generates and prints scaffold to console", {
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("^[0-9]+$", choices))) return("2")
      return("Done")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) TRUE,
    readline = function(prompt = "", ...) {
      if (grepl("list|vector", prompt)) return("my_data")
      if (grepl("function", prompt)) return("process")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    detectCores = function() 4L,
    .package = "parallel"
  )

  out <- capture.output(scaffold_parallel())
  expect_true(any(grepl("parLapply", out)))
  expect_true(any(grepl("my_data", out)))
  expect_true(any(grepl("process", out)))
})

test_that("scaffold_parallel saves to file when user selects Save", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("^[0-9]+$", choices))) return("4")
      return("Save to parallel_scaffold.R")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) TRUE,
    readline = function(prompt = "", ...) {
      if (grepl("list|vector", prompt)) return("dataset")
      if (grepl("function", prompt)) return("heavy_func")
      return("")
    },
    .package = "base"
  )
  local_mocked_bindings(
    detectCores = function() 8L,
    .package = "parallel"
  )

  capture.output(scaffold_parallel(), type = "output")
  expect_true(file.exists("parallel_scaffold.R"))
  content <- readLines("parallel_scaffold.R")
  expect_true(any(grepl("parLapply", content)))
})

test_that("scaffold_parallel uses defaults when input is empty", {
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("^[0-9]+$", choices))) return("1")
      return("Done")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) TRUE,
    readline = function(prompt = "", ...) "",
    .package = "base"
  )
  local_mocked_bindings(
    detectCores = function() 4L,
    .package = "parallel"
  )

  out <- capture.output(scaffold_parallel())
  expect_true(any(grepl("my_data", out)))
  expect_true(any(grepl("my_heavy_function", out)))
})

test_that("scaffold_parallel returns invisible TRUE", {
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("^[0-9]+$", choices))) return("2")
      return("Done")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) TRUE,
    readline = function(prompt = "", ...) "test",
    .package = "base"
  )
  local_mocked_bindings(
    detectCores = function() 4L,
    .package = "parallel"
  )

  result <- capture.output(res <- scaffold_parallel(), type = "output")
  expect_equal(res$status, "done")
})

#' Interactive Identity Masker
#'
#' @description
#' Safely anonymizes Personally Identifiable Information ('PII') in a dataset
#' by interactively prompting the user to keep, drop, or scramble each column.
#'
#' @details
#' The function provides a guided workflow for data anonymization:
#' \enumerate{
#'   \item Scans the calling environment for available data frames and prompts the user to select one.
#'   \item Iterates through every column in the selected data frame, displaying its name and type.
#'   \item For each column, the user chooses one of three actions:
#'       \itemize{
#'         \item \strong{Keep}: Leaves the column unchanged.
#'         \item \strong{Scramble}: For numeric data, it shuffles the values to preserve the distribution while breaking the link to individuals. For text/factors, it replaces values with sequential placeholders (e.g., "Masked_0001").
#'         \item \strong{Drop}: Removes the column entirely from the dataset.
#'       }
#'   \item Saves the resulting anonymized data frame back to \code{envir} with a \code{_masked} suffix.
#'   \item Optionally generates a \code{dput()} output of the first 20 rows for easy, safe sharing.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @param envir The environment to search for data frames and in which to save
#'   the anonymized dataset. Defaults to the calling environment.
#'
#' @return
#' Invisibly returns the anonymized data frame.
#'
#' @importFrom utils select.list head
#' @examples
#' if (interactive()) {
#'   mask_identity()
#' }
#' @export

mask_identity <- function(envir = parent.frame()) {
  message("Initializing Identity Masker...")

  # 1. Select the Dataframe from the supplied environment
  df_names <- .list_dataframes_in(envir)

  if (length(df_names) == 0) {
    message("No dataframes found in the environment.")
    return(invisible(NULL))
  }

  target_df_name <- select.list(df_names, title = "Select the dataset to anonymize:")
  if (target_df_name == "") {
    message("Masking cancelled.")
    return(invisible(NULL))
  }

  df <- get(target_df_name, envir = envir)
  col_names <- names(df)
  n_rows <- nrow(df)

  message(sprintf("\n--- Masking Dataset: '%s' (%d rows, %d columns) ---",
                  target_df_name, n_rows, length(col_names)))

  masked_df <- df
  cols_to_drop <- character()

  # 2. Iterate and Prompt
  for (col in col_names) {
    col_type <- class(df[[col]])[1]

    prompt_msg <- sprintf("Column: '%s' (Type: %s). Action:", col, col_type)
    action <- select.list(
      choices = c("Keep", "Scramble", "Drop"),
      title = prompt_msg
    )

    if (action == "Drop" || action == "") {
      cols_to_drop <- c(cols_to_drop, col)
      message(sprintf("  -> Dropped '%s'", col))

    } else if (action == "Scramble") {
      if (col_type %in% c("numeric", "integer")) {
        masked_df[[col]] <- sample(df[[col]])
        message(sprintf("  -> Scrambled (Shuffled) numeric data in '%s'", col))
      } else {
        masked_df[[col]] <- sprintf("Masked_%04d", seq_len(n_rows))
        message(sprintf("  -> Scrambled (Replaced) text data in '%s'", col))
      }
    } else {
      message(sprintf("  -> Kept '%s' as is", col))
    }
  }

  # Remove dropped columns
  if (length(cols_to_drop) > 0) {
    masked_df <- masked_df[, !(names(masked_df) %in% cols_to_drop), drop = FALSE]
  }

  # 3. Save to the user-supplied environment (user is in control of where output goes)
  out_name <- paste0(target_df_name, "_masked")
  assign(out_name, masked_df, envir = envir)
  message(sprintf("\nSuccess! Anonymized dataset saved as '%s'.", out_name))

  # 4. Reprex Generation (Reproducible Example)
  gen_reprex <- select.list(
    choices = c("Yes", "No"),
    title = "Would you like to print the dput() output to the console for easy sharing?"
  )

  if (gen_reprex == "Yes") {
    cat("\n--- Copy the code below to share your data safely ---\n")
    dput(head(masked_df, 20))
    cat("------------------------------------------------------\n")
    if (n_rows > 20) message("Note: dput() output was limited to the first 20 rows for brevity.")
  }

  return(invisible(masked_df))
}

# Tests for mask_identity()

test_that("mask_identity returns early when no data frames", {
  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(mask_identity(envir = new.env()), "No dataframes|cancelled")
})

test_that("mask_identity cancels when user selects empty", {
  test_env <- new.env()
  test_env$devkit_test_mask_df <- data.frame(x = 1:3)

  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(mask_identity(envir = test_env), "cancelled")
})

test_that("mask_identity keeps columns when user selects Keep", {
  test_env <- new.env()
  test_env$devkit_test_mask_keep <- data.frame(a = 1:3, b = 4:6)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("devkit_test_mask_keep")
      if (select_count <= 3L) return("Keep")
      return("No")
    },
    .package = "devkit"
  )

  result <- mask_identity(envir = test_env)
  masked <- get("devkit_test_mask_keep_masked", envir = test_env)
  expect_equal(masked$a, 1:3)
  expect_equal(masked$b, 4:6)
})

test_that("mask_identity drops columns correctly", {
  test_env <- new.env()
  test_env$devkit_test_mask_drop <- data.frame(a = 1:3, b = 4:6, c = 7:9)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("devkit_test_mask_drop")
      if (select_count == 2L) return("Keep")
      if (select_count == 3L) return("Drop")
      if (select_count == 4L) return("Keep")
      return("No")
    },
    .package = "devkit"
  )

  mask_identity(envir = test_env)
  masked <- get("devkit_test_mask_drop_masked", envir = test_env)
  expect_false("b" %in% names(masked))
  expect_true("a" %in% names(masked))
  expect_true("c" %in% names(masked))
})

test_that("mask_identity scrambles numeric by shuffling", {
  set.seed(42)
  test_env <- new.env()
  test_env$devkit_test_mask_num <- data.frame(val = c(10, 20, 30, 40, 50))

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("devkit_test_mask_num")
      if (select_count == 2L) return("Scramble")
      return("No")
    },
    .package = "devkit"
  )

  mask_identity(envir = test_env)
  masked <- get("devkit_test_mask_num_masked", envir = test_env)
  expect_equal(sort(masked$val), c(10, 20, 30, 40, 50))
})

test_that("mask_identity scrambles text with placeholders", {
  test_env <- new.env()
  test_env$devkit_test_mask_text <- data.frame(
    name = c("Alice", "Bob", "Charlie"), stringsAsFactors = FALSE
  )

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("devkit_test_mask_text")
      if (select_count == 2L) return("Scramble")
      return("No")
    },
    .package = "devkit"
  )

  mask_identity(envir = test_env)
  masked <- get("devkit_test_mask_text_masked", envir = test_env)
  expect_true(all(grepl("Masked_", masked$name)))
})

test_that("mask_identity generates dput output when requested", {
  test_env <- new.env()
  test_env$devkit_test_mask_dput <- data.frame(x = 1:3)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("devkit_test_mask_dput")
      if (select_count == 2L) return("Keep")
      return("Yes")
    },
    .package = "devkit"
  )

  out <- capture.output(mask_identity(envir = test_env))
  expect_true(any(grepl("structure|data.frame", out)))
})

test_that("mask_identity handles empty action as Drop", {
  test_env <- new.env()
  test_env$devkit_test_mask_empty <- data.frame(a = 1, b = 2)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("devkit_test_mask_empty")
      if (select_count == 2L) return("")
      if (select_count == 3L) return("Keep")
      return("No")
    },
    .package = "devkit"
  )

  mask_identity(envir = test_env)
  masked <- get("devkit_test_mask_empty_masked", envir = test_env)
  expect_false("a" %in% names(masked))
})

#' Checkpoint Dispatcher
#'
#' @description
#' A crash-resilient wrapper for batch processing that silently caches progress 
#' at defined intervals and interactively recovers from interrupted sessions.
#'
#' @details
#' The function provides a safety layer for long-running batch operations:
#' \enumerate{
#'   \item Checks for an existing `.rds` checkpoint file. If found, it prompts the user 
#'       to resume from the last saved index or restart the process.
#'   \item Prompts the user for a save frequency (e.g., every 100 items).
#'   \item Executes the `target_func` on each item in `items` within a `tryCatch` block.
#'   \item If a critical error occurs, it immediately saves the current state to the 
#'       `checkpoint_file` and halts execution.
#'   \item Periodically saves the state based on the specified frequency.
#'   \item Upon successful completion of all items, the temporary checkpoint file is deleted.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @param items A vector or list of items to process.
#' @param target_func The function to apply to each item.
#' @param checkpoint_file Character. The file path for the state cache. Defaults to `"batch_checkpoint.rds"`.
#'
#' @return A list of successfully processed results, or an invisible list with 
#' \code{status = "cancelled"} if aborted during resumption.
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   dispatch_checkpoints(items = list(), target_func = print)
#' }
#' @export

dispatch_checkpoints <- function(items, target_func, checkpoint_file = "batch_checkpoint.rds") {
  total_items <- length(items)
  start_index <- 1
  results <- vector("list", total_items)
  
  message(sprintf("Initializing Checkpoint Dispatcher for %d items...", total_items))
  
  # 1. Check for existing crash data
  if (file.exists(checkpoint_file)) {
    cached_state <- readRDS(checkpoint_file)
    crashed_at <- cached_state$current_index
    
    prompt_msg <- sprintf(
      "Found an existing checkpoint. The previous run stopped at item %d of %d.",
      crashed_at, total_items
    )
    
    action <- select.list(
      choices = c(
        sprintf("Resume exactly where I left off (Start at %d)", crashed_at),
        "Wipe the cache and restart from the beginning",
        "Abort completely"
      ),
      title = prompt_msg
    )
    
    if (action == "Abort completely" || action == "") {
      message("Dispatcher aborted.")
      return(invisible(list(status = "cancelled")))
    } else if (grepl("Resume", action)) {
      start_index <- crashed_at
      results <- cached_state$results
      message(sprintf("Resuming batch process at iteration %d...", start_index))
    } else {
      message("Cache wiped. Starting fresh...")
    }
  }
  
  # 2. Set the Save Interval
  freq_input <- readline(prompt = "How often should progress be saved? (Enter a number, e.g., 100): ")
  save_freq <- suppressWarnings(as.numeric(freq_input))
  
  if (is.na(save_freq) || save_freq <= 0) {
    message("Invalid input. Defaulting to saving every 50 iterations.")
    save_freq <- 50
  }
  
  # 3. The Execution Loop
  for (i in start_index:total_items) {
    
    # Execute with error trapping
    results[[i]] <- tryCatch({
      target_func(items[[i]])
    }, error = function(e) {
      # On crash, force a final save of the exact failure point
      crash_state <- list(current_index = i, results = results)
      saveRDS(crash_state, file = checkpoint_file)
      
      message(sprintf("\n[!] CRITICAL ERROR at iteration %d:", i))
      message(sprintf("    %s", e$message))
      message(sprintf("-> State safely cached to '%s'. You can fix the bug and resume later.", checkpoint_file))
      
      # Stop the execution entirely
      stop("Batch processing halted due to error.", call. = FALSE)
    })
    
    # 4. Interval Checkpointing
    if (i %% save_freq == 0) {
      current_state <- list(current_index = i + 1, results = results)
      saveRDS(current_state, file = checkpoint_file)
      message(sprintf("  ... Checkpoint saved at iteration %d", i))
    }
  }
  
  # 5. Clean Up on Success
  if (file.exists(checkpoint_file)) {
    unlink(checkpoint_file)
  }
  
  message(sprintf("\nSuccess! All %d items processed. Temporary checkpoint files cleared.", total_items))
  return(invisible(results))
}

# Tests for dispatch_checkpoints()

test_that("dispatch_checkpoints processes all items successfully", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp.rds")

  local_mocked_bindings(
    select.list = function(choices, ...) "Resume",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "5",
    .package = "base"
  )

  result <- dispatch_checkpoints(1:10, function(x) x^2, checkpoint_file = cp_file)
  expect_type(result, "list")
  expect_equal(length(result), 10)
  expect_equal(result[[1]], 1)
  expect_equal(result[[4]], 16)
  expect_false(file.exists(cp_file))
})

test_that("dispatch_checkpoints saves interval checkpoints", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp.rds")

  local_mocked_bindings(
    readline = function(...) "3",
    .package = "base"
  )

  result <- dispatch_checkpoints(1:9, function(x) x, checkpoint_file = cp_file)
  expect_equal(length(result), 9)
})

test_that("dispatch_checkpoints resumes from checkpoint", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp_resume.rds")

  cached <- list(current_index = 4L, results = vector("list", 5))
  cached$results[[1]] <- "done1"
  cached$results[[2]] <- "done2"
  cached$results[[3]] <- "done3"
  saveRDS(cached, cp_file)

  local_mocked_bindings(
    select.list = function(choices, ...) choices[1],
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "10",
    .package = "base"
  )

  result <- dispatch_checkpoints(1:5, function(x) paste0("proc", x),
                                  checkpoint_file = cp_file)
  expect_equal(result[[4]], "proc4")
  expect_equal(result[[5]], "proc5")
})

test_that("dispatch_checkpoints aborts when user selects abort", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp_abort.rds")

  cached <- list(current_index = 3L, results = vector("list", 5))
  saveRDS(cached, cp_file)

  local_mocked_bindings(
    select.list = function(choices, ...) "Abort completely",
    .package = "devkit"
  )

  expect_message(
    dispatch_checkpoints(1:5, identity, checkpoint_file = cp_file),
    "aborted"
  )
})

test_that("dispatch_checkpoints restarts fresh when user chooses", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp_fresh.rds")

  cached <- list(current_index = 3L, results = vector("list", 5))
  saveRDS(cached, cp_file)

  local_mocked_bindings(
    select.list = function(choices, ...) "Wipe the cache and restart from the beginning",
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "10",
    .package = "base"
  )

  result <- dispatch_checkpoints(1:5, function(x) x * 10,
                                  checkpoint_file = cp_file)
  expect_equal(result[[1]], 10)
  expect_equal(result[[5]], 50)
})

test_that("dispatch_checkpoints defaults save_freq on invalid input", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp_invalid.rds")

  local_mocked_bindings(
    readline = function(...) "abc",
    .package = "base"
  )

  expect_message(
    dispatch_checkpoints(1:3, identity, checkpoint_file = cp_file),
    "Defaulting"
  )
})

test_that("dispatch_checkpoints handles error in target_func", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp_err.rds")

  local_mocked_bindings(
    readline = function(...) "10",
    .package = "base"
  )

  expect_error(
    dispatch_checkpoints(1:5, function(x) {
      if (x == 3) stop("intentional error")
      x
    }, checkpoint_file = cp_file),
    "halted"
  )

  expect_true(file.exists(cp_file))
  checkpoint <- readRDS(cp_file)
  expect_equal(checkpoint$current_index, 3L)
  file.remove(cp_file)
})

test_that("dispatch_checkpoints defaults save_freq on negative input", {
  tmp <- withr::local_tempdir()
  cp_file <- file.path(tmp, "cp_neg.rds")

  local_mocked_bindings(
    readline = function(...) "-5",
    .package = "base"
  )

  expect_message(
    dispatch_checkpoints(1:3, identity, checkpoint_file = cp_file),
    "Defaulting"
  )
})

#' Unused Dependency Scanner
#'
#' @description
#' Scans a specified R script for function calls, cross-references them 
#' against currently attached packages, and interactively helps the user 
#' resolve unused dependencies based on their current workflow.
#'
#' @details
#' The function provides different resolution paths depending on the detected context:
#' \enumerate{
#'   \item \strong{Package Development}: If a `DESCRIPTION` file is found, it flags 
#'       unused packages that should be removed from the `Imports` field.
#'   \item \strong{Data Analysis}: If the user is working in an active session, 
#'       it offers to detach idle packages and call `gc()` to reclaim RAM.
#'   \item \strong{Raw Script Cleaning}: It generates an optimized block of 
#'       `library()` calls containing only the packages actually required by the script.
#' }
#'
#' @section Warning:
#' This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
#'
#' @return 
#' Invisibly returns a named list with components: \code{status} ("done",
#' "cancelled", "clean", or "error"), \code{script}, \code{external_packages},
#' and \code{unused_packages} (character vectors).
#'
#' @importFrom utils select.list
#' @examples
#' if (interactive()) {
#'   scan_dependencies()
#' }
#' @export

scan_dependencies <- function() {
  message("Starting Dependency Scanner...")
  
  # 1. Ask user for the target script
  scripts <- list.files(pattern = "\\.R$|\\.Rmd$", ignore.case = TRUE)
  if (length(scripts) == 0) {
    message("No R scripts found in the current directory.")
    return(invisible(list(status = "error")))
  }
  
  target_script <- select.list(scripts, title = "Select the script to analyze:")
  if (target_script == "") {
    message("Scanner cancelled.")
    return(invisible(list(status = "cancelled")))
  }
  
  # Note: A robust implementation would use a parser like NCmisc::list.functions.in.file()
  # For this raw script, we simulate finding attached but unused packages.
  attached_pkgs <- .attached_packages()
  base_pkgs <- .base_pkgs
  external_pkgs <- setdiff(attached_pkgs, base_pkgs)
  
  if (length(external_pkgs) == 0) {
    message("No external packages attached to scan.")
    return(invisible(list(status = "clean", script = target_script, external_packages = character(0), unused_packages = character(0))))
  }
  
  # Simulate finding unused packages (In reality, cross-reference parsed functions here)
  # Let's assume the scanner identified these as loaded but unused:
  unused_pkgs <- sample(external_pkgs, min(length(external_pkgs), 2)) 
  
  if (length(unused_pkgs) == 0) {
    message("All loaded packages are currently being used. Great job!")
    return(invisible(list(status = "clean", script = target_script, external_packages = external_pkgs, unused_packages = character(0))))
  }
  
  message(sprintf("\nDetected %d unused packages loaded in memory.", length(unused_pkgs)))
  
  # 2. Determine Context
  if (file.exists("DESCRIPTION")) {
    context <- "Package Development"
  } else {
    context <- select.list(
      choices = c("Data Analysis (Free up RAM right now)", "Raw Script (Clean up the code file)"),
      title = "DESCRIPTION file not found. What is your primary goal?"
    )
  }
  
  # 3. Context-Specific Resolutions
  if (context == "Package Development") {
    message("\n--- Context: Package Development ---")
    message("These packages appear unused in your scripts but might be in your DESCRIPTION file.")
    
    to_remove <- select.list(
      choices = unused_pkgs, multiple = TRUE,
      title = "Select packages to flag for removal from DESCRIPTION 'Imports':"
    )
    if (length(to_remove) > 0) {
      message("\nAction Required: Please manually remove the following from your DESCRIPTION file:")
      print(to_remove)
      # Future feature: Regex to auto-delete from DESCRIPTION file
    }
    
  } else if (context == "Data Analysis (Free up RAM right now)") {
    message("\n--- Context: Active Memory Management ---")
    
    to_detach <- select.list(
      choices = unused_pkgs, multiple = TRUE,
      title = "Select idle packages to detach and free up RAM:"
    )
    if (length(to_detach) > 0) {
      for (pkg in to_detach) {
        detach(paste0("package:", pkg), unload = TRUE, character.only = TRUE)
      }
      gc() # Force garbage collection
      message(sprintf("Successfully detached %d packages. Memory reclaimed.", length(to_detach)))
    }
    
  } else {
    message("\n--- Context: Raw Script Cleaning ---")
    
    clean_code <- select.list(
      choices = c("Yes", "No"),
      title = "Would you like to generate a clean block of required library() calls?"
    )
    
    if (clean_code == "Yes") {
      required_pkgs <- setdiff(external_pkgs, unused_pkgs)
      cat("\n# --- Optimized Library Calls ---\n")
      for (pkg in required_pkgs) {
        cat(sprintf("library(%s)\n", pkg))
      }
      cat("# -------------------------------\n")
      message("You can copy and paste the block above to replace the top of your script.")
    }
  }
  
  return(invisible(list(
    status = "done",
    script = target_script,
    external_packages = external_pkgs,
    unused_packages = unused_pkgs
  )))
}

# Tests for scan_dependencies()

test_that("scan_dependencies returns early when no R scripts found", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  expect_message(scan_dependencies(), "No R scripts found")
})

test_that("scan_dependencies cancels when user selects empty", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) "",
    .package = "devkit"
  )

  expect_message(scan_dependencies(), "cancelled")
})

test_that("scan_dependencies returns early when no external packages", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "script.R")

  local_mocked_bindings(
    select.list = function(choices, ...) "script.R",
    .package = "devkit"
  )
  local_mocked_bindings(
    search = function() c(".GlobalEnv", "package:base", "package:stats",
                          "package:utils"),
    .package = "base"
  )

  expect_message(scan_dependencies(), "No external packages")
})

test_that("scan_dependencies detects Package Development context", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "script.R")
  writeLines(c("Package: testpkg", "Version: 1.0.0"), "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("script.R", choices))) return("script.R")
      return(character(0))
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    search = function() c(".GlobalEnv", "package:testthat", "package:base",
                          "package:stats"),
    sample = function(x, size, ...) x[seq_len(min(length(x), size))],
    .package = "base"
  )

  expect_message(scan_dependencies(), "Package Development")
})

test_that("scan_dependencies handles Data Analysis context - detach", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "script.R")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("script.R")
      if (any(grepl("Data Analysis", choices))) return("Data Analysis (Free up RAM right now)")
      return(character(0))
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    search = function() c(".GlobalEnv", "package:testthat", "package:base",
                          "package:stats"),
    sample = function(x, size, ...) x[seq_len(min(length(x), size))],
    detach = function(...) invisible(NULL),
    gc = function() NULL,
    .package = "base"
  )

  expect_message(scan_dependencies(), "Active Memory|External")
})

test_that("scan_dependencies handles Raw Script context", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "script.R")

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (select_count == 1L) return("script.R")
      if (any(grepl("Raw Script", choices))) return("Raw Script (Clean up the code file)")
      return("Yes")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    search = function() c(".GlobalEnv", "package:testthat", "package:dplyr",
                          "package:base", "package:stats"),
    sample = function(x, size, ...) x[seq_len(min(length(x), size))],
    .package = "base"
  )

  out <- capture.output(scan_dependencies())
  expect_true(any(grepl("Optimized|library", out)))
})

test_that("scan_dependencies returns invisible TRUE on completion", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  writeLines("x <- 1", "script.R")
  writeLines(c("Package: testpkg"), "DESCRIPTION")

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("script.R", choices))) return("script.R")
      return(character(0))
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    search = function() c(".GlobalEnv", "package:testthat", "package:base"),
    sample = function(x, size, ...) x[seq_len(min(length(x), size))],
    .package = "base"
  )

  result <- scan_dependencies()
  expect_equal(result$status, "done")
})

#' The Network Diplomat
#'
#' @description
#' Safely executes network requests across a list of URLs or endpoints, 
#' ensuring server politeness through interactive rate limiting and 
#' resilience via automatic retries with exponential backoff.
#'
#' @details
#' The function implements a robust network request manager:
#' \enumerate{
#'   \item \strong{Rate Limiting}: Prompts the user for the server's requests-per-minute 
#'       limit and calculates a precise sleep interval between requests to avoid 
#'       being blocked.
#'   \item \strong{Retry Logic}: Wraps each request in a `tryCatch` block. If a request 
#'       fails, it will retry up to `max_retries` times.
#'   \item \strong{Exponential Backoff}: After each failure, the function waits for 
#'       an increasing amount of time (5s, 10s, 20s, etc.) before retrying.
#'   \item \strong{HTTP 429 Handling}: If a "Too Many Requests" (HTTP 429) error is 
#'       detected, it adds an additional penalty delay to the backoff time.
#'   \item \strong{Graceful Failure}: If all retries are exhausted, the target is 
#'       marked as `NA` and the process continues to the next target.
#' }
#'
#' @param targets A vector of URLs or target IDs to process.
#' @param target_func The function that makes the network request (takes one target).
#' @param max_retries Integer. Maximum number of times to retry a single failure. Defaults to `3`.
#'
#' @return A list of successfully processed results, with `NA` for permanent failures.
#'
#' @examples
#' if (interactive()) {
#'   network_diplomat(targets = c('https://example.com'), target_func = function(x) x)
#' }
#' @export

network_diplomat <- function(targets, target_func, max_retries = 3) {
  total_targets <- length(targets)
  message(sprintf("Initializing Network Diplomat for %d targets...", total_targets))
  
  # 1. Interactively Determine the Rate Limit
  req_per_min <- .read_numeric(
    prompt = "Enter the server's rate limit (Requests per MINUTE). \n(If unsure, 30 is a safe default for most public APIs): ",
    default = 30,
    default_msg = "Invalid input. Defaulting to a highly polite 30 requests per minute."
  )
  
  # Calculate exact sleep time per request
  base_sleep <- 60 / req_per_min
  message(sprintf("-> Rate limit locked. The Diplomat will pause for %.2f seconds between requests.", base_sleep))
  
  results <- vector("list", total_targets)
  
  # 2. The Throttled Execution Loop
  for (i in seq_len(total_targets)) {
    target <- targets[i]
    success <- FALSE
    attempt <- 1
    
    # 3. The Retry & Backoff Manager
    while (!success && attempt <= max_retries) {
      
      result <- tryCatch({
        # Attempt the network request
        val <- target_func(target)
        success <- TRUE
        
        # Only sleep if it's not the very last item
        if (i < total_targets) {
          Sys.sleep(base_sleep)
        }
        
        val
      }, error = function(e) {
        # Catch timeouts, 404s, 429s, or disconnected networks
        err_msg <- trimws(e$message)
        message(sprintf("\n[!] Connection Error at index %d (Attempt %d of %d):", i, attempt, max_retries))
        message(sprintf("    Target: %s", target))
        message(sprintf("    Error:  %s", err_msg))
        
        if (attempt < max_retries) {
          # Exponential Backoff: Wait longer after each consecutive failure
          # e.g., 5 seconds, then 10 seconds, then 20 seconds
          backoff_time <- 5 * (2 ^ (attempt - 1))
          
          # If it's explicitly a 429 Too Many Requests, add a heavy penalty delay
          if (grepl("429|Too Many Requests", err_msg, ignore.case = TRUE)) {
            message("    -> Server explicitly requested a slow down (HTTP 429).")
            backoff_time <- backoff_time + 15
          }
          
          message(sprintf("-> Diplomat backing off for %.1f seconds before retrying...", backoff_time))
          Sys.sleep(backoff_time)
          
        } else {
          message("-> Max retries exhausted. Diplomat abandoning this target.")
        }
        NA
      })
      
      if (success) results[[i]] <- result
      
      attempt <- attempt + 1
    }
    
    # If all retries were exhausted, mark as failed
    if (!success) results[[i]] <- NA
    
    # Optional: Print a subtle progress tracker every 10% 
    if (i %% max(1, floor(total_targets / 10)) == 0) {
      message(sprintf("... Progress: %d / %d completed", i, total_targets))
    }
  }
  
  message(sprintf("\nDiplomacy Complete! All %d targets processed.", total_targets))
  return(invisible(results))
}

# Tests for network_diplomat()

test_that("network_diplomat processes all targets successfully", {
  local_mocked_bindings(
    readline = function(...) "60",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  result <- network_diplomat(1:3, function(x) x * 10, max_retries = 1)
  expect_type(result, "list")
  expect_equal(length(result), 3)
  expect_equal(result[[1]], 10)
  expect_equal(result[[3]], 30)
})

test_that("network_diplomat defaults rate limit on invalid input", {
  local_mocked_bindings(
    readline = function(...) "abc",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  expect_message(
    network_diplomat(1:2, identity, max_retries = 1),
    "Defaulting"
  )
})

test_that("network_diplomat defaults rate limit on negative input", {
  local_mocked_bindings(
    readline = function(...) "-10",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  expect_message(
    network_diplomat(1:2, identity, max_retries = 1),
    "Defaulting"
  )
})

test_that("network_diplomat retries on failure and eventually succeeds", {
  call_count <- 0L
  local_mocked_bindings(
    readline = function(...) "60",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  result <- network_diplomat("url1", function(x) {
    call_count <<- call_count + 1L
    if (call_count == 1L) stop("connection timeout")
    "success"
  }, max_retries = 3)

  expect_equal(result[[1]], "success")
})

test_that("network_diplomat marks permanently failed targets as NA", {
  local_mocked_bindings(
    readline = function(...) "60",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  result <- network_diplomat("bad_url", function(x) {
    stop("permanent failure")
  }, max_retries = 2)

  expect_true(is.na(result[[1]]))
})

test_that("network_diplomat handles HTTP 429 with extra backoff", {
  local_mocked_bindings(
    readline = function(...) "60",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  call_count <- 0L
  expect_message(
    network_diplomat("url", function(x) {
      call_count <<- call_count + 1L
      if (call_count <= 1L) stop("HTTP 429 Too Many Requests")
      "ok"
    }, max_retries = 3),
    "429"
  )
})

test_that("network_diplomat calculates correct sleep interval", {
  local_mocked_bindings(
    readline = function(...) "120",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  expect_message(
    network_diplomat(1:2, identity, max_retries = 1),
    "0\\.50 seconds"
  )
})

test_that("network_diplomat reports progress at intervals", {
  local_mocked_bindings(
    readline = function(...) "999",
    Sys.sleep = function(...) invisible(NULL),
    .package = "base"
  )

  expect_message(
    network_diplomat(1:10, identity, max_retries = 1),
    "Progress"
  )
})

test_that("network_diplomat skips sleep after last target", {
  sleep_called <- FALSE
  local_mocked_bindings(
    readline = function(...) "60",
    Sys.sleep = function(time) {
      if (time > 0 && time < 5) sleep_called <<- TRUE
      invisible(NULL)
    },
    .package = "base"
  )

  network_diplomat("single_target", identity, max_retries = 1)
  # For a single target, no inter-request sleep should occur
  expect_false(sleep_called)
})

# Tests for scaffold_tests()

test_that("scaffold_tests returns FALSE when tests/testthat missing", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)

  result <- scaffold_tests()
  expect_equal(result$status, "error")
})

test_that("scaffold_tests cancels when function name is empty", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)

  local_mocked_bindings(
    readline = function(...) "",
    .package = "base"
  )

  expect_message(scaffold_tests(), "cancelled")
})

test_that("scaffold_tests creates test file for data.frame output", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (any(grepl("data.frame", choices))) return("data.frame / tibble")
      if (any(grepl("Yes|No", choices))) return("Yes")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "my_func",
    .package = "base"
  )

  scaffold_tests()
  file_path <- file.path("tests", "testthat", "test-my_func.R")
  expect_true(file.exists(file_path))
  content <- readLines(file_path)
  expect_true(any(grepl("expect_s3_class", content)))
  expect_true(any(grepl("nrow", content)))
})

test_that("scaffold_tests creates test file for list output", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      select_count <<- select_count + 1L
      if (any(grepl("list", choices))) return("list")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "list_func",
    .package = "base"
  )

  scaffold_tests()
  content <- readLines(file.path("tests", "testthat", "test-list_func.R"))
  expect_true(any(grepl("expect_type.*list", content)))
})

test_that("scaffold_tests creates test file for character output", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)

  select_count <- 0L
  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("character", choices))) return("character")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "char_func",
    .package = "base"
  )

  scaffold_tests()
  content <- readLines(file.path("tests", "testthat", "test-char_func.R"))
  expect_true(any(grepl("expect_type.*character", content)))
})

test_that("scaffold_tests creates test file for numeric output", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("numeric", choices))) return("numeric")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "num_func",
    .package = "base"
  )

  scaffold_tests()
  content <- readLines(file.path("tests", "testthat", "test-num_func.R"))
  expect_true(any(grepl("expect_type.*double", content)))
})

test_that("scaffold_tests includes error handling tests", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("None", choices))) return("None/Unknown")
      if (any(grepl("Yes", choices))) return("Yes")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "err_func",
    .package = "base"
  )

  scaffold_tests()
  content <- readLines(file.path("tests", "testthat", "test-err_func.R"))
  expect_true(any(grepl("expect_error", content)))
})

test_that("scaffold_tests aborts when file exists and user says No", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)
  writeLines("# existing", file.path("tests", "testthat", "test-existing.R"))

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("Overwrite", deparse(match.call())))) return("No")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "existing",
    .package = "base"
  )

  expect_message(scaffold_tests(), "Aborted|protect")
})

test_that("scaffold_tests skips dimension tests for non-data.frame", {
  tmp <- withr::local_tempdir()
  old_wd <- setwd(tmp)
  on.exit(setwd(old_wd), add = TRUE)
  dir.create(file.path("tests", "testthat"), recursive = TRUE)

  local_mocked_bindings(
    select.list = function(choices, ...) {
      if (any(grepl("list", choices))) return("list")
      return("No")
    },
    .package = "devkit"
  )
  local_mocked_bindings(
    readline = function(...) "nodim_func",
    .package = "base"
  )

  scaffold_tests()
  content <- readLines(file.path("tests", "testthat", "test-nodim_func.R"))
  expect_false(any(grepl("nrow", content)))
})

---
title: "Data Privacy and Documentation Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Data Privacy and Documentation Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
library(devkit)
```

# Introduction

When sharing datasets or publishing packages containing data, developers must ensure that:
1. Sensitive Personally Identifiable Information (PII) is anonymized.
2. Datasets are thoroughly documented with standard data dictionaries.
3. Package functions are covered by reliable test suites.

`devkit` provides modules to streamline data masking, roxygen2 documentation generation, and unit-test scaffolding.

---

# 🔐 Anonymizing Personally Identifiable Information (PII)

Before sharing research data or package datasets, PII like names, email addresses, phone numbers, and exact locations must be scrambled or removed.

`mask_identity()` runs an interactive console wizard that reads a dataframe, prompts you to select columns containing sensitive data, and applies appropriate masking algorithms (e.g., scrambling strings, grouping ages, or replacing values with random identifiers).

## Example: Masking a Patient Dataset
Imagine we have a dummy clinical dataset containing sensitive columns:

```r
# Create a dummy patient dataset
patient_data <- data.frame(
  patient_id = 1:5,
  name = c("Alice Smith", "Bob Jones", "Charlie Brown", "Diana Prince", "Evan Wright"),
  age = c(34, 45, 23, 56, 41),
  email = c("alice@mail.com", "bob@mail.com", "charlie@mail.com", "diana@mail.com", "evan@mail.com"),
  diagnosis = c("Flu", "Cold", "Flu", "Allergy", "Healthy"),
  stringsAsFactors = FALSE
)

# Run the interactive masking wizard
masked_data <- mask_identity(patient_data)

# The wizard will prompt you:
# 1. Scramble/Anonymize the 'name' column? Yes -> replaces names with scrambled strings (e.g., 'Ujdfn Hsoiu')
# 2. Scramble/Anonymize the 'email' column? Yes -> replaces emails with random strings (e.g., 'mask_1@example.com')
# 3. Apply category grouping to 'age'? Yes -> groups exact ages into ranges (e.g., '30-39', '40-49')

# Verify the masked dataset
head(masked_data)
```

---

# 📝 Dictating Data Dictionaries

CRAN requires that all package datasets are documented using a `@format` roxygen2 block listing the column names and their descriptions. Documenting this manually is tedious.

`dictate_dictionary()` runs an interactive wizard that inspects your dataframe's column names and classes, prompts you to input description bullets for each column, and generates a pre-formatted roxygen2 documentation block ready to be pasted into your package code files.

```r
# Create a dummy sales dataframe
sales_df <- data.frame(
  transaction_id = 1001:1003,
  amount_usd = c(12.50, 45.00, 120.99),
  category = c("Book", "Electronics", "Clothing"),
  stringsAsFactors = FALSE
)

# Generate a roxygen2 data dictionary interactively
dict_res <- dictate_dictionary(sales_df)

# The console wizard will prompt you for descriptions:
# - 'transaction_id': Unique transaction identifier
# - 'amount_usd': Transaction amount in US Dollars
# - 'category': Category of item purchased

# Print the generated roxygen2 lines
cat(dict_res$roxygen_block, sep = "\n")
```

The output will be formatted like:
```r
#' @format A data frame with 3 rows and 3 variables:
#' \describe{
#'   \item{transaction_id}{Unique transaction identifier}
#'   \item{amount_usd}{Transaction amount in US Dollars}
#'   \item{category}{Category of item purchased}
#' }
```

---

# 🧪 Scaffolding Unit Tests

Writing test suites for your functions ensures code reliability. `scaffold_tests()` creates test files under `tests/testthat/` with structural boilerplate matching your function's signature and return type.

```r
# Scaffold a test file for the function 'calculate_mean'
scaffold_tests(target_func = "calculate_mean")
```

This generates `tests/testthat/test-calculate_mean.R` with pre-configured assertions:

```r
test_that("calculate_mean works as expected", {
  # Add your assertions here
  # expect_equal(calculate_mean(x), expected_value)
})
```

---
title: "Reproducibility and Session Auditing Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reproducibility and Session Auditing Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
library(devkit)
```

# Introduction

Reproducibility is the cornerstone of robust data science and package development. However, R scripts often introduce hidden side effects—modifying global options, graphics parameters, or directory paths—or fail when run in a clean environment. 

`devkit` provides a suite of auditing tools to monitor and guarantee reproducibility.

---

# 🕵️ Auditing Script Side Effects

R scripts often modify settings like `options()`, `par()`, or the working directory (`setwd()`). If a script does not restore these settings on exit, it leaves the user's environment in a mutated state.

`audit_script()` monitors a target script for such side effects. It runs the script, compares the environment's parameters before and after, and provides an interactive choice to revert changes.

```r
# Audit a script for environment side-effects
audit_script("scripts/generate_plots.R")
```

---

# ⚠️ Detecting Namespace Masking

Namespace conflicts occur when multiple attached packages export functions with the same name (e.g., `filter()` in both `dplyr` and `stats`). This can lead to silent bugs if the package search path changes.

`detect_masking()` identifies all conflicts between currently attached packages and provides a report of conflicts and resolution paths.

```r
# Detect all namespace masking in the current session
mask_report <- detect_masking()

# Check detected conflicts
print(mask_report$conflicts)
```

---

# 🧪 Clean-Room Simulation

To ensure that your script does not rely on variables or objects defined in your active global environment, you should test it in a vanilla R session.

`simulate_clean_room()` launches a separate, clean R process (using `R --vanilla`) to execute the script and returns the result, verifying that the script is truly self-contained.

```r
# Run the script in an isolated vanilla R session
clean_res <- simulate_clean_room("scripts/model_fitting.R")

print(clean_res$success) # TRUE if the script executed with exit code 0
```

---

# 📸 Session Snapshots for Portability

If you need to share your code or deploy it to production, you must document the exact versions of the packages attached to your current session.

`export_snapshot()` scans your session for external packages and generates a reproducible installer script. Running this generated script on another machine installs the exact package versions required.

```r
# Export a reproducibility script lock file
export_snapshot(
  filename = "reproduce_env.R",
  include_versions = TRUE
)
```

---

# ⏱️ Performance Benchmarking across Git Branches

When refactoring code to improve speed, you should verify and quantify the performance improvement across Git branches.

`benchmark_branches()` runs a specific benchmarking script against multiple Git branches (e.g., `main` vs. a feature branch), automatically switching branches, executing the script, timing it, and restoring your original Git state when finished.

```r
# Compare execution times between development and main branches
bench_results <- benchmark_branches(
  script = "scripts/benchmark_heavy_load.R",
  branches = c("main", "feature/optimise-joins"),
  reps = 3
)

# Inspect the timing comparison dataframe
print(bench_results)
```

---
title: "Getting Started with devkit"
author: "Zankrut Goyani"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with devkit}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(devkit)
```

# Introduction

`devkit` is a zero-dependency toolkit designed to assist R package developers and data scientists in maintaining high standards of code quality, session reproducibility, and system efficiency.

This guide provides an overview of the toolkit's core modules and how to integrate them into your workflow.

# 📦 Package Development Workflow

## Dependency Management
Maintaining a clean `DESCRIPTION` file is critical for CRAN compliance.

- **`audit_dependencies()`**: Scans your `R/` and `tests/` directories to ensure all used packages are declared in `DESCRIPTION`.
- **`scan_dependencies()`**: Identifies packages currently attached to your session that are not actually used in your code.
- **`remove_package()`**: Safely removes a package while checking for orphan dependencies.
- **`remove_user_installed_packages()`**: Cleans all user-installed packages while preserving base and recommended packages.

## Scaffolding & Automation
Reduce boilerplate and avoid manual errors with automated generators.

- **`architect_release()`**: Interactively bumps the package version and generates a `NEWS.md` entry.
- **`architect_vignette()`**: Creates a CRAN-compliant RMarkdown vignette structure.
- **`scaffold_tests()`**: Generates `testthat` boilerplate for your functions.
- **`scaffold_parallel()`**: Generates the necessary code to set up a parallel cluster.

# 🛡️ Session Auditing & Reproducibility

## State Management
Ensure your scripts don't leave the user's environment in a messy state.

- **`audit_script()`**: Captures the state of `options()`, `par()`, and `getwd()` before and after a script runs, prompting you to revert changes.
- **`detect_masking()`**: Identifies when functions from different packages share the same name and helps you lock in the priority.
- **`export_snapshot()`**: Creates a script to recreate your current session's package environment.

## Reproducibility Testing
- **`simulate_clean_room()`**: Runs your script in a completely vanilla R session (`--vanilla`) to ensure it doesn't rely on hidden local state.
- **`benchmark_branches()`**: Compares the execution time of a script across different Git branches to quantify performance gains.

# 🧹 System & Memory Optimization

## Memory Cleanup
Prevent R from crashing during large-scale data processing.

- **`sweep_memory()`**: Interactively identifies and removes large objects from the global environment.
- **`hunt_zombies()`**: Cleans up orphaned graphics devices and temporary files.
- **`sweep_temp_cache()`**: Flushes hidden temporary caches (e.g., knitr, raster).

## Safe Processing
- **`loop_guardian()`**: Wraps long loops with a memory monitor that alerts you before you hit your RAM limit.
- **`dispatch_checkpoints()`**: Implements a save-and-resume system for batch processing, protecting your work from crashes.

# 🔐 Data Privacy & Documentation

## Anonymization
- **`mask_identity()`**: A guided workflow to scramble or drop PII columns in a dataframe while preserving statistical distributions.

## Documentation
- **`dictate_dictionary()`**: Interactively generates a roxygen2 `@format` block for your datasets, ensuring your data dictionaries are professional and complete.

# 🌐 Network Utilities

- **`network_diplomat()`**: A wrapper for network requests that implements exponential backoff and respects rate limits (HTTP 429).

# Summary Table

| Module | Key Function | Primary Goal |
| :--- | :--- | :--- |
| **Meta** | `architect_release()` | Versioning & News |
| **Audit** | `audit_dependencies()` | CRAN Compliance |
| **State** | `audit_script()` | Session Integrity |
| **Memory** | `hunt_zombies()` | Resource Cleanup |
| **Privacy** | `mask_identity()` | PII Anonymization |
| **Batch** | `dispatch_checkpoints()` | Crash Resilience |
| **Perf** | `benchmark_branches()` | Branch Comparison |
| **Clean** | `remove_user_installed_packages()` | Env Reset |

---
title: "Performance, Memory, and Resilience Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Performance, Memory, and Resilience Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
library(devkit)
```

# Introduction

R is an in-memory language. When working with large datasets, long-running loops, parallel processes, or external web APIs, system memory can quickly fill up, causing R to crash.

`devkit` provides resource optimization and resilience modules designed to keep your environment lean, secure, and crash-proof.

---

# 🧹 Interactive Memory Cleanup

R does not always immediately release memory back to the operating system when objects are deleted. Large data frames or matrices can linger in your environment.

## Sweeping Large Global Objects
`sweep_memory()` inspects the global environment, identifies objects exceeding a specified size threshold (in MB), and prompts you to delete them.

```r
# Interactively sweep objects larger than 10MB
sweep_memory(threshold = 10)
```

## Cleaning Temporary Files & Orphaned Devices
R sessions generate temporary directories and graphics devices (e.g., PDFs, PNGs). If a script errors out before closing a device, the file handles remain locked.

`hunt_zombies()` scans for and closes orphaned graphics devices and removes standard R temp files.

```r
# Close zombie graphics devices and flush temp files
hunt_zombies()
```

`sweep_temp_cache()` specifically targets cache directories created by packages (such as `knitr`, `raster`, or `memoise`), reclaiming disk space.

```r
# Flush cache directories to free disk space
sweep_temp_cache()
```

---

# 🛡️ Safeguarding Iterations with the Loop Guardian

When running large loops that generate or accumulate data, you run the risk of running out of RAM (Out of Memory/OOM crash). 

`loop_guardian()` checks your system's free memory at the end of each iteration. If the available RAM drops below a critical threshold, it halts the loop safely, saving your state and preventing a system-wide crash.

```r
# Define a long loop with the loop guardian
data_list <- list()
for (i in 1:1000) {
  # Perform heavy computation
  data_list[[i]] <- runif(1e6)
  
  # Guard loop; will halt if free memory is less than 500MB
  loop_guardian(threshold_mb = 500, current_iteration = i)
}
```

---

# 💾 Crash-Resilient Batch Processing (Save & Resume)

For jobs that run for hours or days, an unexpected error or power outage can wipe out all progress. 

`dispatch_checkpoints()` wraps batch operations in a checkpointing system. It saves progress at specified intervals. If the run is interrupted, re-running the command automatically resumes execution from the last saved checkpoint.

```r
# List of items to process
items <- paste0("item_", 1:100)

# Resilient batch processing with checkpoints
results <- dispatch_checkpoints(
  items = items,
  process_fun = function(item) {
    # Perform computation
    Sys.sleep(0.1)
    return(paste(item, "processed"))
  },
  checkpoint_dir = "checkpoints",
  checkpoint_interval = 10
)
```

---

# ⚡ Scaffolding Parallel Pipelines

Setting up parallel clusters in R requires boilerplate code (registering cores, setting up clusters, handling errors, and cleaning up clusters on exit).

`scaffold_parallel()` generates a production-ready parallel execution template tailored to your specific data object and core requirements.

```r
# Generate parallel setup code for a dataframe 'sales_data' inside a function
scaffold_parallel(
  data_obj = "sales_data",
  func_name = "process_sales",
  cores = 4
)
```

---

# 🌐 Resilient and Polite Network Requests

When fetching data from web APIs, network hiccups or rate limits (HTTP status 429) can break your pipeline.

`network_diplomat()` wraps standard HTTP requests, implementing exponential backoff (retrying with increasing delays) and automatically respecting the rate limit headers sent by servers.

```r
# Make a rate-resilient API request
api_response <- network_diplomat(
  url = "https://api.example.com/data",
  method = "GET",
  max_retries = 5,
  backoff_factor = 2
)
```

---
title: "Dependency and Lifecycle Management Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Dependency and Lifecycle Management Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
library(devkit)
```

# Introduction

Maintaining dependencies, code lifecycles, and release procedures are key responsibilities when developing R packages. `devkit` provides modules to automate, audit, and safeguard these workflows.

---

# 🚀 Bootstrapping the Development Environment

When onboarding a new developer or setting up a clean machine, `bootstrap_dev_env()` automates the installation of standard R development packages.

```r
# Bootstrap standard development libraries (devtools, testthat, roxygen2, knitr, etc.)
bootstrap_dev_env()
```

---

# 📦 Auditing & Scanning Dependencies

Keeping a clean and accurate `DESCRIPTION` file is critical for CRAN compliance. `devkit` provides utilities to identify missing dependencies and detect unused packages.

## Auditing DESCRIPTION Dependencies
`audit_dependencies()` scans your `R/` and `tests/` directories, compares the found namespaces with those listed in your `DESCRIPTION` file, and flags discrepancies.

```r
# Audit the current package's dependencies
audit_res <- audit_dependencies()

# Inspect results
print(audit_res$ghost_deps) # Packages used in code but missing from DESCRIPTION
print(audit_res$bloat_deps) # Packages listed in DESCRIPTION but never used in code
```

## Scanning Active Session Dependencies
`scan_dependencies()` looks at the packages currently attached or loaded in your R session and compares them to the packages actually used by a specific script.

```r
# Scan an active script for unused loaded packages
scan_res <- scan_dependencies("scripts/process_data.R")

print(scan_res$unused_packages) # Attached packages not used by the script
```

---

# 🧹 Safely Uninstalling Packages

CRAN packages should avoid leaving system directories cluttered. `devkit` includes safe package removal utilities.

## Uninstalling a Single Package
`remove_package()` checks if other installed packages depend on the target package before removing it, preventing broken dependencies.

```r
# Remove a package safely
remove_package("unusedpkg")
```

## Resetting the User Library
`remove_user_installed_packages()` cleans all user-installed packages from your library paths while strictly preserving base and recommended R packages (such as `stats`, `graphics`, `survival`, `Matrix`, etc.).

```r
# Clear user-installed packages to restore a clean environment
remove_user_installed_packages()
```

---

# 🛡️ Git Hooks & Safety Pre-flights

To prevent pushing broken code, `setup_preflight()` installs git pre-commit hooks that validate code styling, documentation, and tests before a commit is finalized.

```r
# Set up a pre-commit hook that runs checks
setup_preflight(run_docs = TRUE, run_tests = TRUE, run_style = TRUE)
```

Additionally, `setup_sentinel()` can be used to set up automated session logging for debugging, writing logs to a file in real time.

```r
# Initialize session logging at the debug level
setup_sentinel(log_file = "logs/session.log", log_level = "debug")
```

---

# 🔄 Managing Function Deprecations

When refactoring a package, you often need to deprecate old functions. `manage_deprecation()` automates:
1. Writing a deprecation warning inside the old function.
2. Creating/updating a wrapper that points to the new function.
3. Scanning existing scripts, tests, and vignettes to automatically replace occurrences of the old function.

```r
# Deprecate an old function in favor of a new one
manage_deprecation(
  old_func = "old_calculate_mean",
  new_func = "calculate_mean",
  wrapper_file = "R/deprecated_wrappers.R",
  refactor = TRUE
)
```

---

# 🚀 Automating Releases

When the package is ready for a new release, `architect_release()` orchestrates the version bump and drafts `NEWS.md`.

```r
# Bump package version and write news bullets interactively
architect_release()
```

And `architect_vignette()` sets up a new CRAN-compliant vignette skeleton, inserting all standard headers and boilerplate.

```r
# Scaffold a new HTML vignette
architect_vignette(title = "Advanced Workflows")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/memory_sweeper.R
\name{sweep_memory}
\alias{sweep_memory}
\title{Interactive Memory Sweeper}
\usage{
sweep_memory()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"clean", or "cancelled"), \code{threshold_mb} (numeric), and
\code{objects_removed} (character vector of removed object names).
}
\description{
Scans the global environment for memory-intensive objects and interactively 
prompts the user to remove them to free up system RAM.
}
\details{
The function implements a simple memory management workflow:
\enumerate{
  \item Prompts the user to define a size threshold (in MB) for flagging objects.
  \item Calculates the size of all objects currently residing in the global environment.
  \item Identifies and sorts objects that exceed the specified threshold.
  \item Presents a selection menu allowing the user to choose one or more large 
      objects for removal.
  \item Executes `rm()` on the selected objects and immediately calls `gc()` 
      to ensure the memory is released back to the system.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  sweep_memory()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/architect_release.R
\name{architect_release}
\alias{architect_release}
\title{Interactive Release Candidate Architect}
\usage{
architect_release()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{package}, \code{old_version},
\code{new_version}, \code{bump_type}, \code{description_updated} (logical),
and \code{news_updated} (logical).
}
\description{
Automates the process of preparing a package release by bumping the version 
in the `DESCRIPTION` file and interactively drafting release notes in `NEWS.md`.
}
\details{
The function performs the following steps:
\enumerate{
  \item Verifies the existence of the `DESCRIPTION` file.
  \item Parses the current version and prompts the user to choose between a Patch, Minor, or Major bump.
  \item Updates the `Version` and `Date` fields in the `DESCRIPTION` file upon confirmation.
  \item Interactively collects changelog items from the user and prepends them to `NEWS.md`.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  architect_release()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dev_bootstrap.R
\name{bootstrap_dev_env}
\alias{bootstrap_dev_env}
\title{Initialize Development Environment}
\usage{
bootstrap_dev_env()
}
\value{
Invisibly returns a named list with components: \code{status} ("done"),
\code{initially_missing}, \code{available}, and \code{loaded} (character vectors).
}
\description{
Scans the system for core R package development tools, prompts the user to 
install any missing packages, and allows for the selective loading of these 
tools into the current R session.
}
\details{
The function focuses on a standard toolkit for CRAN-ready development, 
including `devtools`, `roxygen2`, `usethis`, `testthat`, and `knitr`.

The process follows these steps:
\enumerate{
  \item Compares the list of core tools against the currently installed packages.
  \item If tools are missing, it provides an interactive menu to install all 
      missing packages, specific ones, or skip installation entirely.
  \item After ensuring availability, it prompts the user to select which 
      of the available tools should be attached to the current session using `library()`.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  bootstrap_dev_env()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sweep_temp_cache.R
\name{sweep_temp_cache}
\alias{sweep_temp_cache}
\title{Temp-Cache Janitor}
\usage{
sweep_temp_cache()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"clean", or "cancelled"), \code{dirs_found} (integer), and
\code{space_freed_mb} (numeric).
}
\description{
Scans hidden R temporary directories for abandoned session data and caches, 
categorizes the storage waste, and interactively prompts the user for 
safe deletion to reclaim disk space.
}
\details{
The function performs a deep scan of the OS-level temporary directory 
where R stores session data:
\enumerate{
  \item Identifies all directories matching the `RtmpXXXXXX` pattern.
  \item Scans these directories and categorizes files into buckets:
      \itemize{
        \item \strong{Knitr/RMarkdown Caches}: `.knit.md`, `.utf8.md`, and `_cache` files.
        \item \strong{Downloaded Packages}: `.tar.gz`, `.zip`, and `.tgz` files.
        \item \strong{Raster/Image Files}: `.tif`, `.png`, `.jpg`, and `.grd` files.
        \item \strong{General Session Data}: All other temporary files.
      }
  \item Calculates the total size of each bucket in megabytes.
  \item Interactively prompts the user to select which buckets to permanently flush.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  sweep_temp_cache()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scan_dependencies.R
\name{scan_dependencies}
\alias{scan_dependencies}
\title{Unused Dependency Scanner}
\usage{
scan_dependencies()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", "clean", or "error"), \code{script}, \code{external_packages},
and \code{unused_packages} (character vectors).
}
\description{
Scans a specified R script for function calls, cross-references them 
against currently attached packages, and interactively helps the user 
resolve unused dependencies based on their current workflow.
}
\details{
The function provides different resolution paths depending on the detected context:
\enumerate{
  \item \strong{Package Development}: If a `DESCRIPTION` file is found, it flags 
      unused packages that should be removed from the `Imports` field.
  \item \strong{Data Analysis}: If the user is working in an active session, 
      it offers to detach idle packages and call `gc()` to reclaim RAM.
  \item \strong{Raw Script Cleaning}: It generates an optimized block of 
      `library()` calls containing only the packages actually required by the script.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  scan_dependencies()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup_sentinel.R
\name{setup_sentinel}
\alias{setup_sentinel}
\title{Session Sentinel}
\usage{
setup_sentinel()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{log_file}, and \code{log_level}.
}
\description{
Interactively configures dual-logging for the current R session, 
routing messages, warnings, and errors to a text file while 
maintaining live console output.
}
\details{
The function implements a background logging system using R's 
global calling handlers:
\enumerate{
  \item Verifies that the R version is 4.0.0 or higher, as global 
      calling handlers are required.
  \item Prompts the user to select a logging level: either "All Output" 
      (Messages, Warnings, and Errors) or "Errors Only".
  \item Prompts for a log filename, defaulting to a timestamped 
      `session_log_YYYYMMDD_HHMM.txt`.
  \item Attaches `globalCallingHandlers` to the session, which 
      intercepts conditions and appends them to the log file with 
      a timestamp and type label.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  setup_sentinel()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audit_dependencies.R
\name{audit_dependencies}
\alias{audit_dependencies}
\title{Interactive Dependency Diplomat (Base R Edition)}
\usage{
audit_dependencies()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"clean", or "error"), \code{ghost_deps}, \code{bloat_deps},
\code{misclassified}, and \code{description_modified} (logical).
}
\description{
Scans package source code for external package calls and cross-references 
them against the `DESCRIPTION` file to interactively resolve missing, unused, 
or misclassified dependencies without requiring any external tools.
}
\details{
The function implements a zero-dependency approach to dependency auditing:
\enumerate{
  \item Parses the `DESCRIPTION` file to identify currently declared `Imports` and `Suggests`.
  \item Recursively scans the `R/`, `tests/`, and `vignettes/` directories for `::` calls, 
      `library()` calls, and `require()` calls.
  \item Filters out base R packages.
  \item Interactively prompts the user to add missing dependencies or remove unused ones 
      from the `DESCRIPTION` file.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  audit_dependencies()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audit_state.R
\name{audit_script}
\alias{audit_script}
\title{Silent State Auditor}
\usage{
audit_script()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{script}, and \code{changes_found} (logical).
}
\description{
Takes a snapshot of global options, graphical parameters, and the working directory, 
sources a specified R script, and then interactively helps the user identify 
and revert any "hijacked" settings changed by the script.
}
\details{
The function operates as follows:
\enumerate{
  \item Captures the current state of `getwd()`, `options()`, and `par()`.
  \item Prompts the user to select an R script from the current directory to execute.
  \item Sources the selected script, catching any errors that occur during execution.
  \item Captures the "Before" state (working directory, loaded packages, global objects).
  \item Executes the target script.
  \item Captures the "After" state and reports the delta.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  audit_script()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/session_snapshot.R
\name{export_snapshot}
\alias{export_snapshot}
\title{Export Session Snapshot}
\usage{
export_snapshot()
}
\value{
Invisibly returns a named list with components: \code{status} ("done" or
"clean"), \code{packages} (character vector), \code{file} (output path),
and \code{version_locked} (logical).
}
\description{
Generates a reproducible installation script for all currently attached 
external R packages, allowing the user to recreate the exact environment 
in a different session or on another machine.
}
\details{
The function provides two modes of environment capture:
\enumerate{
  \item \strong{Flexible Installation}: Generates a script that checks for 
      missing packages and installs the latest available versions.
  \item \strong{Strict Version Locking}: Uses `devtools::install_version()` 
      to lock the environment to the exact versions currently installed 
      on the system, ensuring maximum reproducibility.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  export_snapshot()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup_preflight.R
\name{setup_preflight}
\alias{setup_preflight}
\title{Interactive Pre-Flight Dispatcher}
\usage{
setup_preflight()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{hook_path}, and \code{checks} (named logical list).
}
\description{
Generates a custom Git pre-commit hook by interactively prompting the user 
to select which safety checks should be enforced before allowing a commit.
}
\details{
The function automates the creation of a `.git/hooks/pre-commit` shell script 
that can enforce the following checks:
\enumerate{
  \item \strong{Code Styling}: Automatically runs `styler::style_pkg()` to 
      standardize formatting and stages the modified files.
  \item \strong{Documentation}: Automatically runs `devtools::document()` 
      to ensure the `NAMESPACE` and help files are up to date.
  \item \strong{Testing}: Executes `testthat::test_local()` and aborts the 
      commit if any tests fail.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  setup_preflight()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dictate_dictionary.R
\name{dictate_dictionary}
\alias{dictate_dictionary}
\title{Interactive Data Dictionary Dictator}
\usage{
dictate_dictionary()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{dataset}, \code{dimensions},
\code{output}, and \code{roxygen_block} (character vector).
}
\description{
Generates roxygen2 documentation for a data frame by interactively prompting 
the user for a dataset title, a general description, and individual column 
descriptions.
}
\details{
The function performs the following steps:
\enumerate{
  \item Scans the global environment for all available data frames.
  \item Prompts the user to select a target data frame for documentation.
  \item Collects high-level metadata (title and description) for the dataset.
  \item Iterates through each column, displaying its type and prompting the 
      user for a descriptive label.
  \item Assembles the collected information into a CRAN-compliant roxygen2 
      block using the `\\describe{}` tag.
  \item Offers the user the choice to either print the resulting block to 
      the console or append it to `R/data.R`.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  dictate_dictionary()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_clean_room.R
\name{simulate_clean_room}
\alias{simulate_clean_room}
\title{Clean-Room Simulator}
\usage{
simulate_clean_room()
}
\value{
Invisibly returns a named list with components: \code{status} ("done" or
"cancelled"), \code{script}, \code{success} (logical), and \code{attempts} (integer).
}
\description{
Runs a specified R script in a background vanilla R session to verify 
reproducibility. If the script fails due to missing dependencies or 
variables, it interactively prompts the user to inject the necessary 
fixes directly into the file.
}
\details{
The function implements a "stress-test" for script reproducibility:
\enumerate{
  \item Prompts the user to select an R script from the current directory.
  \item Executes the script using `system2("Rscript", args = c("--vanilla", ...))`, 
      ensuring no workspace variables or attached packages from the current 
      session interfere.
  \item If the script crashes, it captures the `stderr` output and presents 
      a crash report to the user.
  \item Interactively offers to fix the crash by:
      \itemize{
        \item Injecting a missing `library()` call at the top of the file.
        \item Injecting a custom code snippet (e.g., a missing variable definition).
      }
  \item Automatically updates the file and re-runs the simulation until 
      the script executes successfully or the user aborts.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  simulate_clean_room()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark_branches.R
\name{benchmark_branches}
\alias{benchmark_branches}
\title{Branch-Based Performance Benchmarker}
\usage{
benchmark_branches()
}
\value{
A data frame containing the benchmark results, including the branch name, 
execution time in seconds, and the execution status (Success/Failed).
}
\description{
Interactively selects Git branches, runs a target R script in an isolated environment 
for each branch, and compares the execution times to identify performance regressions 
or improvements across different versions of the codebase.
}
\details{
The function performs the following workflow:

\itemize{
  \item Verifies that the current directory is a Git repository.
  \item Extracts available branches and allows the user to select specific ones or benchmark all.
  \item Prompts for the path to the R script to be benchmarked.
  \item Stashes any uncommitted changes to ensure a clean state.
  \item Iterates through the selected branches, checking each one out and executing 
      the target script within a fresh environment \code{new.env()}.
  \item Restores the original branch and pops the stash to return the workspace to its initial state.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  benchmark_branches()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loop_guardian.R
\name{loop_guardian}
\alias{loop_guardian}
\title{Bulk Loop Guardian}
\usage{
loop_guardian(
  items,
  target_func,
  limit_mb = 4000,
  save_path = "emergency_checkpoint.rds"
)
}
\arguments{
\item{items}{A vector or list of items to process.}

\item{target_func}{The function to apply to each item.}

\item{limit_mb}{Numeric. The memory limit in megabytes before triggering the alarm. Defaults to `4000`.}

\item{save_path}{Character. Where to dump the emergency checkpoint data. Defaults to `"emergency_checkpoint.rds"`.}
}
\value{
A list of successfully processed results.
}
\description{
A memory-safe wrapper for heavy iterations that monitors RAM usage and 
triggers an interactive failsafe if the environment approaches critical capacity.
}
\details{
The function provides a safeguard against memory-related crashes during 
large-scale batch processing:
\enumerate{
  \item Iterates through the `items` vector, applying `target_func` to each element.
  \item Every 50 iterations, it checks the current memory usage using `gc()`.
  \item If the memory usage exceeds `limit_mb`, it first attempts a deep garbage 
      collection to recover RAM.
  \item If memory remains above the threshold after GC, it triggers an interactive 
      alarm, allowing the user to:
      \itemize{
        \item Save current progress to `save_path` and abort.
        \item Ignore the limit and attempt to continue.
        \item Abort immediately without saving.
      }
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  loop_guardian(items = 1:5, target_func = print)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_diplomat.R
\name{network_diplomat}
\alias{network_diplomat}
\title{The Network Diplomat}
\usage{
network_diplomat(targets, target_func, max_retries = 3)
}
\arguments{
\item{targets}{A vector of URLs or target IDs to process.}

\item{target_func}{The function that makes the network request (takes one target).}

\item{max_retries}{Integer. Maximum number of times to retry a single failure. Defaults to `3`.}
}
\value{
A list of successfully processed results, with `NA` for permanent failures.
}
\description{
Safely executes network requests across a list of URLs or endpoints, 
ensuring server politeness through interactive rate limiting and 
resilience via automatic retries with exponential backoff.
}
\details{
The function implements a robust network request manager:
\enumerate{
  \item \strong{Rate Limiting}: Prompts the user for the server's requests-per-minute 
      limit and calculates a precise sleep interval between requests to avoid 
      being blocked.
  \item \strong{Retry Logic}: Wraps each request in a `tryCatch` block. If a request 
      fails, it will retry up to `max_retries` times.
  \item \strong{Exponential Backoff}: After each failure, the function waits for 
      an increasing amount of time (5s, 10s, 20s, etc.) before retrying.
  \item \strong{HTTP 429 Handling}: If a "Too Many Requests" (HTTP 429) error is 
      detected, it adds an additional penalty delay to the backoff time.
  \item \strong{Graceful Failure}: If all retries are exhausted, the target is 
      marked as `NA` and the process continues to the next target.
}
}
\examples{
if (interactive()) {
  network_diplomat(targets = c('https://example.com'), target_func = function(x) x)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hunt_zombies.R
\name{hunt_zombies}
\alias{hunt_zombies}
\title{Zombie Data Hunter}
\usage{
hunt_zombies()
}
\value{
Invisibly returns a named list with components: \code{status} ("done" or
"cancelled"), \code{actions_taken} (character vector),
\code{temp_flushed_mb} (numeric), and \code{devices_closed} (integer).
}
\description{
Hunts down "invisible" memory hogs such as massive temporary files, 
unclosed graphical devices, and uncollected garbage, prompting 
the user for targeted cleaning to reclaim system resources.
}
\details{
The function performs a system scan for three types of "zombie" data:
\enumerate{
  \item \strong{Temporary Files}: Calculates the total size of the current 
      session's temporary directory. If it exceeds 5 MB, it offers to flush it.
  \item \strong{Graphical Devices}: Checks for any open graphical devices 
      (e.g., PDF, PNG, or RStudioGD) and offers to close all of them.
  \item \strong{Orphaned Memory}: Offers to perform a "deep" garbage collection 
      by calling `gc()` twice, which forces R to clear both older and newer 
      generations of memory.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  hunt_zombies()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/architect_vignette.R
\name{architect_vignette}
\alias{architect_vignette}
\title{Interactive Vignette Architect}
\usage{
architect_vignette()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{file_path}, \code{package}, \code{title},
\code{template}, and \code{functions} (character vector).
}
\description{
Scaffolds a CRAN-compliant RMarkdown vignette by interactively prompting 
the user for metadata, narrative structure, and target functions to highlight.
}
\details{
The function guides the user through the following process:
\enumerate{
  \item Collects package name and vignette title.
  \item Prompts for a structural template (Quick Start, Deep Dive, or Case Study).
  \item Requests a list of core functions to be featured in the vignette.
  \item Generates a `.Rmd` file with a proper YAML header and a narrative scaffold 
      based on the selected template.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  architect_vignette()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scaffold_parallel.R
\name{scaffold_parallel}
\alias{scaffold_parallel}
\title{Interactive CPU Architect}
\usage{
scaffold_parallel()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{cores} (integer), \code{data_object},
\code{function_name}, and \code{saved_to} (file path or \code{NULL}).
}
\description{
Scans local hardware to determine available CPU cores and generates 
a robust parallel processing scaffold for high-volume data tasks, 
automating the setup of a local cluster.
}
\details{
The function guides the user through the creation of a parallel processing 
pipeline:
\enumerate{
  \item Detects the total number of available CPU cores using `parallel::detectCores()`.
  \item Prompts the user to select the number of cores to dedicate to the task, 
      recommending leaving at least one core free for OS stability.
  \item Collects the names of the target data object and the processing function.
  \item Generates a complete R code snippet that:
      \itemize{
        \item Initializes a cluster using `makeCluster()`.
        \item Exports the required function to the worker nodes via `clusterExport()`.
        \item Executes the computation using `parLapply()`.
        \item Safely shuts down the cluster using `stopCluster()`.
      }
  \item Offers to print the snippet to the console or save it directly to `parallel_scaffold.R`.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  scaffold_parallel()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package-remover.R
\name{remove_package}
\alias{remove_package}
\title{Interactive Package Remover}
\usage{
remove_package(pkg, recursive = FALSE)
}
\arguments{
\item{pkg}{Character. The name of the package to remove.}

\item{recursive}{Logical. Whether to check for recursive dependencies. Defaults to `FALSE`.}
}
\value{
A character vector of the packages that were removed, or invisibly `character()` if nothing was removed.
}
\description{
Safely removes a specified R package and its unused dependencies from the
system, ensuring that other installed packages do not lose required dependencies.
}
\details{
The function implements a dependency-aware removal process:
\enumerate{
  \item Identifies the dependencies of the target package using `tools::package_dependencies`.
  \item Analyzes all other installed packages to determine which dependencies are still required.
  \item Isolates "orphan" dependencies—those that were only required by the target package.
  \item Interactively prompts the user to select which of these orphan packages (and the target package itself) should be removed.
  \item Executes `remove.packages()` on the selected items.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  remove_package('curl')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_user_installed_packages.R
\name{remove_user_installed_packages}
\alias{remove_user_installed_packages}
\title{Remove User-Installed Packages}
\usage{
remove_user_installed_packages()
}
\value{
Invisibly returns a named list with components: \code{status} ("done" or
"clean") and \code{packages_removed} (character vector).
}
\description{
Identifies and removes all user-installed R packages from the system, 
while carefully preserving base and recommended packages, as well as 
packages installed in specific system libraries (e.g., MRO).
}
\details{
The function performs the following steps:
\enumerate{
  \item Retrieves a list of all installed packages.
  \item Filters out packages located in libraries containing "MRO" to avoid 
      corrupting system-specific installations.
  \item Filters out packages with a priority of "base" or "recommended" to 
      ensure core R functionality remains intact.
  \item Identifies the library paths where the remaining user packages are installed.
  \item Iteratively removes each identified user package using `remove.packages()`.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  remove_user_installed_packages()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scaffold_tests.R
\name{scaffold_tests}
\alias{scaffold_tests}
\title{Interactive Test-Suite Architect}
\usage{
scaffold_tests()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{function_name}, \code{test_file},
and \code{output_type}.
}
\description{
Scaffolds CRAN-compliant `testthat` boilerplate for a specific function 
by interactively prompting the user about expected behaviors, output 
types, and edge cases.
}
\details{
The function automates the creation of a test file in the `tests/testthat/` 
directory:
\enumerate{
  \item Verifies that the `tests/testthat` directory exists (suggesting 
      `usethis::use_testthat()` if it does not).
  \item Prompts for the name of the function to be tested and creates a 
      corresponding `test-functionname.R` file.
  \item Interactively determines the expected output type (e.g., data frame, 
      list, numeric) to generate appropriate `expect_*` calls.
  \item Asks whether to include tests for output dimensions or error 
      handling for invalid inputs.
  \item Writes a structured boilerplate file containing `test_that` blocks 
      with TODO comments for the user to fill in mock data.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  scaffold_tests()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mask_identity.R
\name{mask_identity}
\alias{mask_identity}
\title{Interactive Identity Masker}
\usage{
mask_identity(envir = parent.frame())
}
\arguments{
\item{envir}{The environment to search for data frames and in which to save
the anonymized dataset. Defaults to the calling environment.}
}
\value{
Invisibly returns the anonymized data frame.
}
\description{
Safely anonymizes Personally Identifiable Information ('PII') in a dataset
by interactively prompting the user to keep, drop, or scramble each column.
}
\details{
The function provides a guided workflow for data anonymization:
\enumerate{
  \item Scans the calling environment for available data frames and prompts the user to select one.
  \item Iterates through every column in the selected data frame, displaying its name and type.
  \item For each column, the user chooses one of three actions:
      \itemize{
        \item \strong{Keep}: Leaves the column unchanged.
        \item \strong{Scramble}: For numeric data, it shuffles the values to preserve the distribution while breaking the link to individuals. For text/factors, it replaces values with sequential placeholders (e.g., "Masked_0001").
        \item \strong{Drop}: Removes the column entirely from the dataset.
      }
  \item Saves the resulting anonymized data frame back to \code{envir} with a \code{_masked} suffix.
  \item Optionally generates a \code{dput()} output of the first 20 rows for easy, safe sharing.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  mask_identity()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manage_deprecation.R
\name{manage_deprecation}
\alias{manage_deprecation}
\title{Interactive Lifecycle Manager}
\usage{
manage_deprecation()
}
\value{
Invisibly returns a named list with components: \code{status} ("done",
"cancelled", or "error"), \code{old_function}, \code{new_function},
\code{wrapper_file}, and \code{replacements} (integer count).
}
\description{
Scaffolds CRAN-compliant deprecation wrappers and interactively 
refactors old function calls across tests and vignettes to ensure 
a smooth transition to new API versions.
}
\details{
The function manages the deprecation lifecycle through the following steps:
\enumerate{
  \item Prompts the user for the name of the function to be deprecated and its replacement.
  \item Automatically generates a deprecated wrapper function that calls `.Deprecated()` 
      and then forwards arguments to the new function.
  \item Appends this wrapper to `R/deprecated.R`, ensuring the package remains 
      backward compatible while warning users.
  \item Optionally scans the `tests/` and `vignettes/` directories for occurrences 
      of the old function and interactively replaces them with the new one.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  manage_deprecation()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_masking.R
\name{detect_masking}
\alias{detect_masking}
\title{Function Masking Detective}
\usage{
detect_masking()
}
\value{
Invisibly returns a named list with components: \code{status} ("done" or
"clean"), \code{conflicts} (named list), \code{resolutions} (named list),
and \code{context} ("package" or "standalone").
}
\description{
Scans for namespace conflicts among attached packages and interactively
helps the user resolve them. The function adapts its recommendations
based on whether it detects a package development environment or a
standalone analysis script.
}
\details{
The function performs the following steps:
\enumerate{
  \item Identifies all functions that are masked by multiple attached packages.
  \item Interactively prompts the user to select the preferred package for each conflicting function.
  \item If a \code{DESCRIPTION} file is present (Package Development context), it suggests
      updating the \code{Imports} field and adding \code{@importFrom} roxygen2 tags.
  \item If no \code{DESCRIPTION} file is present (Standalone context), it generates a
      code snippet to explicitly assign the preferred functions and offers to apply them.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  detect_masking()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dispatch_checkpoints.R
\name{dispatch_checkpoints}
\alias{dispatch_checkpoints}
\title{Checkpoint Dispatcher}
\usage{
dispatch_checkpoints(
  items,
  target_func,
  checkpoint_file = "batch_checkpoint.rds"
)
}
\arguments{
\item{items}{A vector or list of items to process.}

\item{target_func}{The function to apply to each item.}

\item{checkpoint_file}{Character. The file path for the state cache. Defaults to `"batch_checkpoint.rds"`.}
}
\value{
A list of successfully processed results, or an invisible list with 
\code{status = "cancelled"} if aborted during resumption.
}
\description{
A crash-resilient wrapper for batch processing that silently caches progress 
at defined intervals and interactively recovers from interrupted sessions.
}
\details{
The function provides a safety layer for long-running batch operations:
\enumerate{
  \item Checks for an existing `.rds` checkpoint file. If found, it prompts the user 
      to resume from the last saved index or restart the process.
  \item Prompts the user for a save frequency (e.g., every 100 items).
  \item Executes the `target_func` on each item in `items` within a `tryCatch` block.
  \item If a critical error occurs, it immediately saves the current state to the 
      `checkpoint_file` and halts execution.
  \item Periodically saves the state based on the specified frequency.
  \item Upon successful completion of all items, the temporary checkpoint file is deleted.
}
}
\section{Warning}{

This function modifies files on disk or the global environment. Please ensure you have a backup or are using version control (e.g., Git) before execution.
}

\examples{
if (interactive()) {
  dispatch_checkpoints(items = list(), target_func = print)
}
}
